# MIT License
# 
# Copyright (c) 2019 Edward D. Lee, Bryan C. Daniels
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# This file contains equations for the Potts model with 6 spins and 3 states.

from numpy import zeros, exp
from .enumerate import fast_logsumexp

def calc_observables(params):
	assert params.size==33
	h = params[:18]
	J = params[18:]
	observables = zeros(33)
	energies = [		h[0]+h[1]+h[2]+h[3]+h[4]+h[5]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[2]+h[3]+h[4]+h[11]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[2]+h[3]+h[4]+h[17]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[2]+h[3]+h[10]+h[5]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[2]+h[3]+h[10]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[1]+h[2]+h[3]+h[10]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[0]+h[1]+h[2]+h[3]+h[16]+h[5]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[2]+h[3]+h[16]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[0]+h[1]+h[2]+h[3]+h[16]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[1]+h[2]+h[9]+h[4]+h[5]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[2]+h[9]+h[4]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[1]+h[2]+h[9]+h[4]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[0]+h[1]+h[2]+h[9]+h[10]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[1]+h[2]+h[9]+h[10]+h[11]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[2]+h[9]+h[10]+h[17]+J[0]+J[1]+J[5]+J[12],
		h[0]+h[1]+h[2]+h[9]+h[16]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[0]+h[1]+h[2]+h[9]+h[16]+h[11]+J[0]+J[1]+J[5]+J[13],
		h[0]+h[1]+h[2]+h[9]+h[16]+h[17]+J[0]+J[1]+J[5]+J[14],
		h[0]+h[1]+h[2]+h[15]+h[4]+h[5]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[2]+h[15]+h[4]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[0]+h[1]+h[2]+h[15]+h[4]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[1]+h[2]+h[15]+h[10]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[0]+h[1]+h[2]+h[15]+h[10]+h[11]+J[0]+J[1]+J[5]+J[14],
		h[0]+h[1]+h[2]+h[15]+h[10]+h[17]+J[0]+J[1]+J[5]+J[13],
		h[0]+h[1]+h[2]+h[15]+h[16]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[1]+h[2]+h[15]+h[16]+h[11]+J[0]+J[1]+J[5]+J[12],
		h[0]+h[1]+h[2]+h[15]+h[16]+h[17]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[8]+h[3]+h[4]+h[5]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[8]+h[3]+h[4]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[1]+h[8]+h[3]+h[4]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[0]+h[1]+h[8]+h[3]+h[10]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[1]+h[8]+h[3]+h[10]+h[11]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[8]+h[3]+h[10]+h[17]+J[0]+J[2]+J[6]+J[10],
		h[0]+h[1]+h[8]+h[3]+h[16]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[0]+h[1]+h[8]+h[3]+h[16]+h[11]+J[0]+J[2]+J[6]+J[11],
		h[0]+h[1]+h[8]+h[3]+h[16]+h[17]+J[0]+J[2]+J[6]+J[14],
		h[0]+h[1]+h[8]+h[9]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[1]+h[8]+h[9]+h[4]+h[11]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[8]+h[9]+h[4]+h[17]+J[0]+J[3]+J[7]+J[9],
		h[0]+h[1]+h[8]+h[9]+h[10]+h[5]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[8]+h[9]+h[10]+h[11]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[8]+h[9]+h[10]+h[17]+J[0]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[8]+h[9]+h[16]+h[5]+J[0]+J[4]+J[8]+J[9],
		h[0]+h[1]+h[8]+h[9]+h[16]+h[11]+J[0]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[8]+h[9]+h[16]+h[17]+J[0]+J[9]+J[14],
		h[0]+h[1]+h[8]+h[15]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[0]+h[1]+h[8]+h[15]+h[4]+h[11]+J[0]+J[3]+J[7]+J[11],
		h[0]+h[1]+h[8]+h[15]+h[4]+h[17]+J[0]+J[3]+J[7]+J[13],
		h[0]+h[1]+h[8]+h[15]+h[10]+h[5]+J[0]+J[4]+J[8]+J[10],
		h[0]+h[1]+h[8]+h[15]+h[10]+h[11]+J[0]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[8]+h[15]+h[10]+h[17]+J[0]+J[10]+J[13],
		h[0]+h[1]+h[8]+h[15]+h[16]+h[5]+J[0]+J[4]+J[8]+J[12],
		h[0]+h[1]+h[8]+h[15]+h[16]+h[11]+J[0]+J[11]+J[12],
		h[0]+h[1]+h[8]+h[15]+h[16]+h[17]+J[0]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[14]+h[3]+h[4]+h[5]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[14]+h[3]+h[4]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[0]+h[1]+h[14]+h[3]+h[4]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[1]+h[14]+h[3]+h[10]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[0]+h[1]+h[14]+h[3]+h[10]+h[11]+J[0]+J[2]+J[6]+J[14],
		h[0]+h[1]+h[14]+h[3]+h[10]+h[17]+J[0]+J[2]+J[6]+J[11],
		h[0]+h[1]+h[14]+h[3]+h[16]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[1]+h[14]+h[3]+h[16]+h[11]+J[0]+J[2]+J[6]+J[10],
		h[0]+h[1]+h[14]+h[3]+h[16]+h[17]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[14]+h[9]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[0]+h[1]+h[14]+h[9]+h[4]+h[11]+J[0]+J[3]+J[7]+J[13],
		h[0]+h[1]+h[14]+h[9]+h[4]+h[17]+J[0]+J[3]+J[7]+J[11],
		h[0]+h[1]+h[14]+h[9]+h[10]+h[5]+J[0]+J[4]+J[8]+J[12],
		h[0]+h[1]+h[14]+h[9]+h[10]+h[11]+J[0]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[14]+h[9]+h[10]+h[17]+J[0]+J[11]+J[12],
		h[0]+h[1]+h[14]+h[9]+h[16]+h[5]+J[0]+J[4]+J[8]+J[10],
		h[0]+h[1]+h[14]+h[9]+h[16]+h[11]+J[0]+J[10]+J[13],
		h[0]+h[1]+h[14]+h[9]+h[16]+h[17]+J[0]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[14]+h[15]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[1]+h[14]+h[15]+h[4]+h[11]+J[0]+J[3]+J[7]+J[9],
		h[0]+h[1]+h[14]+h[15]+h[4]+h[17]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[14]+h[15]+h[10]+h[5]+J[0]+J[4]+J[8]+J[9],
		h[0]+h[1]+h[14]+h[15]+h[10]+h[11]+J[0]+J[9]+J[14],
		h[0]+h[1]+h[14]+h[15]+h[10]+h[17]+J[0]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[14]+h[15]+h[16]+h[5]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[14]+h[15]+h[16]+h[11]+J[0]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[14]+h[15]+h[16]+h[17]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[2]+h[3]+h[4]+h[5]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[2]+h[3]+h[4]+h[11]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[2]+h[3]+h[4]+h[17]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[2]+h[3]+h[10]+h[5]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[2]+h[3]+h[10]+h[11]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[7]+h[2]+h[3]+h[10]+h[17]+J[1]+J[2]+J[7]+J[9],
		h[0]+h[7]+h[2]+h[3]+h[16]+h[5]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[2]+h[3]+h[16]+h[11]+J[1]+J[2]+J[8]+J[9],
		h[0]+h[7]+h[2]+h[3]+h[16]+h[17]+J[1]+J[2]+J[9]+J[14],
		h[0]+h[7]+h[2]+h[9]+h[4]+h[5]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[2]+h[9]+h[4]+h[11]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[7]+h[2]+h[9]+h[4]+h[17]+J[1]+J[3]+J[6]+J[10],
		h[0]+h[7]+h[2]+h[9]+h[10]+h[5]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[7]+h[2]+h[9]+h[10]+h[11]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[2]+h[9]+h[10]+h[17]+J[1]+J[6]+J[7]+J[12],
		h[0]+h[7]+h[2]+h[9]+h[16]+h[5]+J[1]+J[4]+J[6]+J[11],
		h[0]+h[7]+h[2]+h[9]+h[16]+h[11]+J[1]+J[6]+J[8]+J[13],
		h[0]+h[7]+h[2]+h[9]+h[16]+h[17]+J[1]+J[6]+J[14],
		h[0]+h[7]+h[2]+h[15]+h[4]+h[5]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[2]+h[15]+h[4]+h[11]+J[1]+J[3]+J[8]+J[10],
		h[0]+h[7]+h[2]+h[15]+h[4]+h[17]+J[1]+J[3]+J[10]+J[13],
		h[0]+h[7]+h[2]+h[15]+h[10]+h[5]+J[1]+J[4]+J[7]+J[11],
		h[0]+h[7]+h[2]+h[15]+h[10]+h[11]+J[1]+J[7]+J[8]+J[14],
		h[0]+h[7]+h[2]+h[15]+h[10]+h[17]+J[1]+J[7]+J[13],
		h[0]+h[7]+h[2]+h[15]+h[16]+h[5]+J[1]+J[4]+J[11]+J[12],
		h[0]+h[7]+h[2]+h[15]+h[16]+h[11]+J[1]+J[8]+J[12],
		h[0]+h[7]+h[2]+h[15]+h[16]+h[17]+J[1]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[8]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[8]+h[3]+h[4]+h[11]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[7]+h[8]+h[3]+h[4]+h[17]+J[2]+J[3]+J[5]+J[12],
		h[0]+h[7]+h[8]+h[3]+h[10]+h[5]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[7]+h[8]+h[3]+h[10]+h[11]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[8]+h[3]+h[10]+h[17]+J[2]+J[5]+J[7]+J[10],
		h[0]+h[7]+h[8]+h[3]+h[16]+h[5]+J[2]+J[4]+J[5]+J[13],
		h[0]+h[7]+h[8]+h[3]+h[16]+h[11]+J[2]+J[5]+J[8]+J[11],
		h[0]+h[7]+h[8]+h[3]+h[16]+h[17]+J[2]+J[5]+J[14],
		h[0]+h[7]+h[8]+h[9]+h[4]+h[5]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[7]+h[8]+h[9]+h[4]+h[11]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[8]+h[9]+h[4]+h[17]+J[3]+J[5]+J[6]+J[9],
		h[0]+h[7]+h[8]+h[9]+h[10]+h[5]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[8]+h[9]+h[10]+h[11]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[8]+h[9]+h[10]+h[17]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[8]+h[9]+h[16]+h[5]+J[4]+J[5]+J[6]+J[9],
		h[0]+h[7]+h[8]+h[9]+h[16]+h[11]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[8]+h[9]+h[16]+h[17]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[7]+h[8]+h[15]+h[4]+h[5]+J[3]+J[4]+J[5]+J[14],
		h[0]+h[7]+h[8]+h[15]+h[4]+h[11]+J[3]+J[5]+J[8]+J[11],
		h[0]+h[7]+h[8]+h[15]+h[4]+h[17]+J[3]+J[5]+J[13],
		h[0]+h[7]+h[8]+h[15]+h[10]+h[5]+J[4]+J[5]+J[7]+J[10],
		h[0]+h[7]+h[8]+h[15]+h[10]+h[11]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[8]+h[15]+h[10]+h[17]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[7]+h[8]+h[15]+h[16]+h[5]+J[4]+J[5]+J[12],
		h[0]+h[7]+h[8]+h[15]+h[16]+h[11]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[7]+h[8]+h[15]+h[16]+h[17]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[14]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[14]+h[3]+h[4]+h[11]+J[2]+J[3]+J[8]+J[12],
		h[0]+h[7]+h[14]+h[3]+h[4]+h[17]+J[2]+J[3]+J[11]+J[12],
		h[0]+h[7]+h[14]+h[3]+h[10]+h[5]+J[2]+J[4]+J[7]+J[13],
		h[0]+h[7]+h[14]+h[3]+h[10]+h[11]+J[2]+J[7]+J[8]+J[14],
		h[0]+h[7]+h[14]+h[3]+h[10]+h[17]+J[2]+J[7]+J[11],
		h[0]+h[7]+h[14]+h[3]+h[16]+h[5]+J[2]+J[4]+J[10]+J[13],
		h[0]+h[7]+h[14]+h[3]+h[16]+h[11]+J[2]+J[8]+J[10],
		h[0]+h[7]+h[14]+h[3]+h[16]+h[17]+J[2]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[14]+h[9]+h[4]+h[5]+J[3]+J[4]+J[6]+J[14],
		h[0]+h[7]+h[14]+h[9]+h[4]+h[11]+J[3]+J[6]+J[8]+J[13],
		h[0]+h[7]+h[14]+h[9]+h[4]+h[17]+J[3]+J[6]+J[11],
		h[0]+h[7]+h[14]+h[9]+h[10]+h[5]+J[4]+J[6]+J[7]+J[12],
		h[0]+h[7]+h[14]+h[9]+h[10]+h[11]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[14]+h[9]+h[10]+h[17]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[7]+h[14]+h[9]+h[16]+h[5]+J[4]+J[6]+J[10],
		h[0]+h[7]+h[14]+h[9]+h[16]+h[11]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[7]+h[14]+h[9]+h[16]+h[17]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[14]+h[15]+h[4]+h[5]+J[3]+J[4]+J[9]+J[14],
		h[0]+h[7]+h[14]+h[15]+h[4]+h[11]+J[3]+J[8]+J[9],
		h[0]+h[7]+h[14]+h[15]+h[4]+h[17]+J[3]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[14]+h[15]+h[10]+h[5]+J[4]+J[7]+J[9],
		h[0]+h[7]+h[14]+h[15]+h[10]+h[11]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[7]+h[14]+h[15]+h[10]+h[17]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[14]+h[15]+h[16]+h[5]+J[4]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[14]+h[15]+h[16]+h[11]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[14]+h[15]+h[16]+h[17]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[2]+h[3]+h[4]+h[5]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[2]+h[3]+h[4]+h[11]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[2]+h[3]+h[4]+h[17]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[2]+h[3]+h[10]+h[5]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[2]+h[3]+h[10]+h[11]+J[1]+J[2]+J[9]+J[14],
		h[0]+h[13]+h[2]+h[3]+h[10]+h[17]+J[1]+J[2]+J[8]+J[9],
		h[0]+h[13]+h[2]+h[3]+h[16]+h[5]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[2]+h[3]+h[16]+h[11]+J[1]+J[2]+J[7]+J[9],
		h[0]+h[13]+h[2]+h[3]+h[16]+h[17]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[13]+h[2]+h[9]+h[4]+h[5]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[2]+h[9]+h[4]+h[11]+J[1]+J[3]+J[10]+J[13],
		h[0]+h[13]+h[2]+h[9]+h[4]+h[17]+J[1]+J[3]+J[8]+J[10],
		h[0]+h[13]+h[2]+h[9]+h[10]+h[5]+J[1]+J[4]+J[11]+J[12],
		h[0]+h[13]+h[2]+h[9]+h[10]+h[11]+J[1]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[2]+h[9]+h[10]+h[17]+J[1]+J[8]+J[12],
		h[0]+h[13]+h[2]+h[9]+h[16]+h[5]+J[1]+J[4]+J[7]+J[11],
		h[0]+h[13]+h[2]+h[9]+h[16]+h[11]+J[1]+J[7]+J[13],
		h[0]+h[13]+h[2]+h[9]+h[16]+h[17]+J[1]+J[7]+J[8]+J[14],
		h[0]+h[13]+h[2]+h[15]+h[4]+h[5]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[2]+h[15]+h[4]+h[11]+J[1]+J[3]+J[6]+J[10],
		h[0]+h[13]+h[2]+h[15]+h[4]+h[17]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[13]+h[2]+h[15]+h[10]+h[5]+J[1]+J[4]+J[6]+J[11],
		h[0]+h[13]+h[2]+h[15]+h[10]+h[11]+J[1]+J[6]+J[14],
		h[0]+h[13]+h[2]+h[15]+h[10]+h[17]+J[1]+J[6]+J[8]+J[13],
		h[0]+h[13]+h[2]+h[15]+h[16]+h[5]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[13]+h[2]+h[15]+h[16]+h[11]+J[1]+J[6]+J[7]+J[12],
		h[0]+h[13]+h[2]+h[15]+h[16]+h[17]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[8]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[8]+h[3]+h[4]+h[11]+J[2]+J[3]+J[11]+J[12],
		h[0]+h[13]+h[8]+h[3]+h[4]+h[17]+J[2]+J[3]+J[8]+J[12],
		h[0]+h[13]+h[8]+h[3]+h[10]+h[5]+J[2]+J[4]+J[10]+J[13],
		h[0]+h[13]+h[8]+h[3]+h[10]+h[11]+J[2]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[8]+h[3]+h[10]+h[17]+J[2]+J[8]+J[10],
		h[0]+h[13]+h[8]+h[3]+h[16]+h[5]+J[2]+J[4]+J[7]+J[13],
		h[0]+h[13]+h[8]+h[3]+h[16]+h[11]+J[2]+J[7]+J[11],
		h[0]+h[13]+h[8]+h[3]+h[16]+h[17]+J[2]+J[7]+J[8]+J[14],
		h[0]+h[13]+h[8]+h[9]+h[4]+h[5]+J[3]+J[4]+J[9]+J[14],
		h[0]+h[13]+h[8]+h[9]+h[4]+h[11]+J[3]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[8]+h[9]+h[4]+h[17]+J[3]+J[8]+J[9],
		h[0]+h[13]+h[8]+h[9]+h[10]+h[5]+J[4]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[8]+h[9]+h[10]+h[11]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[8]+h[9]+h[10]+h[17]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[8]+h[9]+h[16]+h[5]+J[4]+J[7]+J[9],
		h[0]+h[13]+h[8]+h[9]+h[16]+h[11]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[8]+h[9]+h[16]+h[17]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[13]+h[8]+h[15]+h[4]+h[5]+J[3]+J[4]+J[6]+J[14],
		h[0]+h[13]+h[8]+h[15]+h[4]+h[11]+J[3]+J[6]+J[11],
		h[0]+h[13]+h[8]+h[15]+h[4]+h[17]+J[3]+J[6]+J[8]+J[13],
		h[0]+h[13]+h[8]+h[15]+h[10]+h[5]+J[4]+J[6]+J[10],
		h[0]+h[13]+h[8]+h[15]+h[10]+h[11]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[8]+h[15]+h[10]+h[17]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[13]+h[8]+h[15]+h[16]+h[5]+J[4]+J[6]+J[7]+J[12],
		h[0]+h[13]+h[8]+h[15]+h[16]+h[11]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[13]+h[8]+h[15]+h[16]+h[17]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[14]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[14]+h[3]+h[4]+h[11]+J[2]+J[3]+J[5]+J[12],
		h[0]+h[13]+h[14]+h[3]+h[4]+h[17]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[13]+h[14]+h[3]+h[10]+h[5]+J[2]+J[4]+J[5]+J[13],
		h[0]+h[13]+h[14]+h[3]+h[10]+h[11]+J[2]+J[5]+J[14],
		h[0]+h[13]+h[14]+h[3]+h[10]+h[17]+J[2]+J[5]+J[8]+J[11],
		h[0]+h[13]+h[14]+h[3]+h[16]+h[5]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[13]+h[14]+h[3]+h[16]+h[11]+J[2]+J[5]+J[7]+J[10],
		h[0]+h[13]+h[14]+h[3]+h[16]+h[17]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[14]+h[9]+h[4]+h[5]+J[3]+J[4]+J[5]+J[14],
		h[0]+h[13]+h[14]+h[9]+h[4]+h[11]+J[3]+J[5]+J[13],
		h[0]+h[13]+h[14]+h[9]+h[4]+h[17]+J[3]+J[5]+J[8]+J[11],
		h[0]+h[13]+h[14]+h[9]+h[10]+h[5]+J[4]+J[5]+J[12],
		h[0]+h[13]+h[14]+h[9]+h[10]+h[11]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[14]+h[9]+h[10]+h[17]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[13]+h[14]+h[9]+h[16]+h[5]+J[4]+J[5]+J[7]+J[10],
		h[0]+h[13]+h[14]+h[9]+h[16]+h[11]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[13]+h[14]+h[9]+h[16]+h[17]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[14]+h[15]+h[4]+h[5]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[13]+h[14]+h[15]+h[4]+h[11]+J[3]+J[5]+J[6]+J[9],
		h[0]+h[13]+h[14]+h[15]+h[4]+h[17]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[14]+h[15]+h[10]+h[5]+J[4]+J[5]+J[6]+J[9],
		h[0]+h[13]+h[14]+h[15]+h[10]+h[11]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[13]+h[14]+h[15]+h[10]+h[17]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[14]+h[15]+h[16]+h[5]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[14]+h[15]+h[16]+h[11]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[14]+h[15]+h[16]+h[17]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[2]+h[3]+h[4]+h[5]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[2]+h[3]+h[4]+h[11]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[2]+h[3]+h[4]+h[17]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[2]+h[3]+h[10]+h[5]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[2]+h[3]+h[10]+h[11]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[1]+h[2]+h[3]+h[10]+h[17]+J[3]+J[5]+J[6]+J[9],
		h[6]+h[1]+h[2]+h[3]+h[16]+h[5]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[2]+h[3]+h[16]+h[11]+J[4]+J[5]+J[6]+J[9],
		h[6]+h[1]+h[2]+h[3]+h[16]+h[17]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[1]+h[2]+h[9]+h[4]+h[5]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[2]+h[9]+h[4]+h[11]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[1]+h[2]+h[9]+h[4]+h[17]+J[2]+J[5]+J[7]+J[10],
		h[6]+h[1]+h[2]+h[9]+h[10]+h[5]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[1]+h[2]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[2]+h[9]+h[10]+h[17]+J[2]+J[3]+J[5]+J[12],
		h[6]+h[1]+h[2]+h[9]+h[16]+h[5]+J[2]+J[5]+J[8]+J[11],
		h[6]+h[1]+h[2]+h[9]+h[16]+h[11]+J[2]+J[4]+J[5]+J[13],
		h[6]+h[1]+h[2]+h[9]+h[16]+h[17]+J[2]+J[5]+J[14],
		h[6]+h[1]+h[2]+h[15]+h[4]+h[5]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[2]+h[15]+h[4]+h[11]+J[4]+J[5]+J[7]+J[10],
		h[6]+h[1]+h[2]+h[15]+h[4]+h[17]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[1]+h[2]+h[15]+h[10]+h[5]+J[3]+J[5]+J[8]+J[11],
		h[6]+h[1]+h[2]+h[15]+h[10]+h[11]+J[3]+J[4]+J[5]+J[14],
		h[6]+h[1]+h[2]+h[15]+h[10]+h[17]+J[3]+J[5]+J[13],
		h[6]+h[1]+h[2]+h[15]+h[16]+h[5]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[1]+h[2]+h[15]+h[16]+h[11]+J[4]+J[5]+J[12],
		h[6]+h[1]+h[2]+h[15]+h[16]+h[17]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[8]+h[3]+h[4]+h[5]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[8]+h[3]+h[4]+h[11]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[1]+h[8]+h[3]+h[4]+h[17]+J[1]+J[6]+J[7]+J[12],
		h[6]+h[1]+h[8]+h[3]+h[10]+h[5]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[1]+h[8]+h[3]+h[10]+h[11]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[8]+h[3]+h[10]+h[17]+J[1]+J[3]+J[6]+J[10],
		h[6]+h[1]+h[8]+h[3]+h[16]+h[5]+J[1]+J[6]+J[8]+J[13],
		h[6]+h[1]+h[8]+h[3]+h[16]+h[11]+J[1]+J[4]+J[6]+J[11],
		h[6]+h[1]+h[8]+h[3]+h[16]+h[17]+J[1]+J[6]+J[14],
		h[6]+h[1]+h[8]+h[9]+h[4]+h[5]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[1]+h[8]+h[9]+h[4]+h[11]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[8]+h[9]+h[4]+h[17]+J[1]+J[2]+J[7]+J[9],
		h[6]+h[1]+h[8]+h[9]+h[10]+h[5]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[8]+h[9]+h[10]+h[11]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[8]+h[9]+h[10]+h[17]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[8]+h[9]+h[16]+h[5]+J[1]+J[2]+J[8]+J[9],
		h[6]+h[1]+h[8]+h[9]+h[16]+h[11]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[8]+h[9]+h[16]+h[17]+J[1]+J[2]+J[9]+J[14],
		h[6]+h[1]+h[8]+h[15]+h[4]+h[5]+J[1]+J[7]+J[8]+J[14],
		h[6]+h[1]+h[8]+h[15]+h[4]+h[11]+J[1]+J[4]+J[7]+J[11],
		h[6]+h[1]+h[8]+h[15]+h[4]+h[17]+J[1]+J[7]+J[13],
		h[6]+h[1]+h[8]+h[15]+h[10]+h[5]+J[1]+J[3]+J[8]+J[10],
		h[6]+h[1]+h[8]+h[15]+h[10]+h[11]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[8]+h[15]+h[10]+h[17]+J[1]+J[3]+J[10]+J[13],
		h[6]+h[1]+h[8]+h[15]+h[16]+h[5]+J[1]+J[8]+J[12],
		h[6]+h[1]+h[8]+h[15]+h[16]+h[11]+J[1]+J[4]+J[11]+J[12],
		h[6]+h[1]+h[8]+h[15]+h[16]+h[17]+J[1]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[14]+h[3]+h[4]+h[5]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[14]+h[3]+h[4]+h[11]+J[4]+J[6]+J[7]+J[12],
		h[6]+h[1]+h[14]+h[3]+h[4]+h[17]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[1]+h[14]+h[3]+h[10]+h[5]+J[3]+J[6]+J[8]+J[13],
		h[6]+h[1]+h[14]+h[3]+h[10]+h[11]+J[3]+J[4]+J[6]+J[14],
		h[6]+h[1]+h[14]+h[3]+h[10]+h[17]+J[3]+J[6]+J[11],
		h[6]+h[1]+h[14]+h[3]+h[16]+h[5]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[1]+h[14]+h[3]+h[16]+h[11]+J[4]+J[6]+J[10],
		h[6]+h[1]+h[14]+h[3]+h[16]+h[17]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[14]+h[9]+h[4]+h[5]+J[2]+J[7]+J[8]+J[14],
		h[6]+h[1]+h[14]+h[9]+h[4]+h[11]+J[2]+J[4]+J[7]+J[13],
		h[6]+h[1]+h[14]+h[9]+h[4]+h[17]+J[2]+J[7]+J[11],
		h[6]+h[1]+h[14]+h[9]+h[10]+h[5]+J[2]+J[3]+J[8]+J[12],
		h[6]+h[1]+h[14]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[14]+h[9]+h[10]+h[17]+J[2]+J[3]+J[11]+J[12],
		h[6]+h[1]+h[14]+h[9]+h[16]+h[5]+J[2]+J[8]+J[10],
		h[6]+h[1]+h[14]+h[9]+h[16]+h[11]+J[2]+J[4]+J[10]+J[13],
		h[6]+h[1]+h[14]+h[9]+h[16]+h[17]+J[2]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[14]+h[15]+h[4]+h[5]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[1]+h[14]+h[15]+h[4]+h[11]+J[4]+J[7]+J[9],
		h[6]+h[1]+h[14]+h[15]+h[4]+h[17]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[14]+h[15]+h[10]+h[5]+J[3]+J[8]+J[9],
		h[6]+h[1]+h[14]+h[15]+h[10]+h[11]+J[3]+J[4]+J[9]+J[14],
		h[6]+h[1]+h[14]+h[15]+h[10]+h[17]+J[3]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[14]+h[15]+h[16]+h[5]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[14]+h[15]+h[16]+h[11]+J[4]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[14]+h[15]+h[16]+h[17]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[2]+h[3]+h[4]+h[5]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[2]+h[3]+h[4]+h[11]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[2]+h[3]+h[4]+h[17]+J[0]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[2]+h[3]+h[10]+h[5]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[2]+h[3]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[7]+h[2]+h[3]+h[10]+h[17]+J[0]+J[3]+J[7]+J[9],
		h[6]+h[7]+h[2]+h[3]+h[16]+h[5]+J[0]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[2]+h[3]+h[16]+h[11]+J[0]+J[4]+J[8]+J[9],
		h[6]+h[7]+h[2]+h[3]+h[16]+h[17]+J[0]+J[9]+J[14],
		h[6]+h[7]+h[2]+h[9]+h[4]+h[5]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[2]+h[9]+h[4]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[7]+h[2]+h[9]+h[4]+h[17]+J[0]+J[2]+J[6]+J[10],
		h[6]+h[7]+h[2]+h[9]+h[10]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[7]+h[2]+h[9]+h[10]+h[11]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[2]+h[9]+h[10]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[6]+h[7]+h[2]+h[9]+h[16]+h[5]+J[0]+J[2]+J[6]+J[11],
		h[6]+h[7]+h[2]+h[9]+h[16]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[6]+h[7]+h[2]+h[9]+h[16]+h[17]+J[0]+J[2]+J[6]+J[14],
		h[6]+h[7]+h[2]+h[15]+h[4]+h[5]+J[0]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[2]+h[15]+h[4]+h[11]+J[0]+J[4]+J[8]+J[10],
		h[6]+h[7]+h[2]+h[15]+h[4]+h[17]+J[0]+J[10]+J[13],
		h[6]+h[7]+h[2]+h[15]+h[10]+h[5]+J[0]+J[3]+J[7]+J[11],
		h[6]+h[7]+h[2]+h[15]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[6]+h[7]+h[2]+h[15]+h[10]+h[17]+J[0]+J[3]+J[7]+J[13],
		h[6]+h[7]+h[2]+h[15]+h[16]+h[5]+J[0]+J[11]+J[12],
		h[6]+h[7]+h[2]+h[15]+h[16]+h[11]+J[0]+J[4]+J[8]+J[12],
		h[6]+h[7]+h[2]+h[15]+h[16]+h[17]+J[0]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[8]+h[3]+h[4]+h[5]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[8]+h[3]+h[4]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[7]+h[8]+h[3]+h[4]+h[17]+J[0]+J[1]+J[5]+J[12],
		h[6]+h[7]+h[8]+h[3]+h[10]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[7]+h[8]+h[3]+h[10]+h[11]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[8]+h[3]+h[10]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[6]+h[7]+h[8]+h[3]+h[16]+h[5]+J[0]+J[1]+J[5]+J[13],
		h[6]+h[7]+h[8]+h[3]+h[16]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[6]+h[7]+h[8]+h[3]+h[16]+h[17]+J[0]+J[1]+J[5]+J[14],
		h[6]+h[7]+h[8]+h[9]+h[4]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[7]+h[8]+h[9]+h[4]+h[11]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[8]+h[9]+h[4]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[6]+h[7]+h[8]+h[9]+h[10]+h[5]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[8]+h[9]+h[10]+h[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[8]+h[9]+h[10]+h[17]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[8]+h[9]+h[16]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[6]+h[7]+h[8]+h[9]+h[16]+h[11]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[8]+h[9]+h[16]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[7]+h[8]+h[15]+h[4]+h[5]+J[0]+J[1]+J[5]+J[14],
		h[6]+h[7]+h[8]+h[15]+h[4]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[6]+h[7]+h[8]+h[15]+h[4]+h[17]+J[0]+J[1]+J[5]+J[13],
		h[6]+h[7]+h[8]+h[15]+h[10]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[6]+h[7]+h[8]+h[15]+h[10]+h[11]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[8]+h[15]+h[10]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[7]+h[8]+h[15]+h[16]+h[5]+J[0]+J[1]+J[5]+J[12],
		h[6]+h[7]+h[8]+h[15]+h[16]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[7]+h[8]+h[15]+h[16]+h[17]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[14]+h[3]+h[4]+h[5]+J[0]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[14]+h[3]+h[4]+h[11]+J[0]+J[4]+J[8]+J[12],
		h[6]+h[7]+h[14]+h[3]+h[4]+h[17]+J[0]+J[11]+J[12],
		h[6]+h[7]+h[14]+h[3]+h[10]+h[5]+J[0]+J[3]+J[7]+J[13],
		h[6]+h[7]+h[14]+h[3]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[6]+h[7]+h[14]+h[3]+h[10]+h[17]+J[0]+J[3]+J[7]+J[11],
		h[6]+h[7]+h[14]+h[3]+h[16]+h[5]+J[0]+J[10]+J[13],
		h[6]+h[7]+h[14]+h[3]+h[16]+h[11]+J[0]+J[4]+J[8]+J[10],
		h[6]+h[7]+h[14]+h[3]+h[16]+h[17]+J[0]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[14]+h[9]+h[4]+h[5]+J[0]+J[2]+J[6]+J[14],
		h[6]+h[7]+h[14]+h[9]+h[4]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[6]+h[7]+h[14]+h[9]+h[4]+h[17]+J[0]+J[2]+J[6]+J[11],
		h[6]+h[7]+h[14]+h[9]+h[10]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[6]+h[7]+h[14]+h[9]+h[10]+h[11]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[14]+h[9]+h[10]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[7]+h[14]+h[9]+h[16]+h[5]+J[0]+J[2]+J[6]+J[10],
		h[6]+h[7]+h[14]+h[9]+h[16]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[7]+h[14]+h[9]+h[16]+h[17]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[14]+h[15]+h[4]+h[5]+J[0]+J[9]+J[14],
		h[6]+h[7]+h[14]+h[15]+h[4]+h[11]+J[0]+J[4]+J[8]+J[9],
		h[6]+h[7]+h[14]+h[15]+h[4]+h[17]+J[0]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[14]+h[15]+h[10]+h[5]+J[0]+J[3]+J[7]+J[9],
		h[6]+h[7]+h[14]+h[15]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[7]+h[14]+h[15]+h[10]+h[17]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[14]+h[15]+h[16]+h[5]+J[0]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[14]+h[15]+h[16]+h[11]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[14]+h[15]+h[16]+h[17]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[2]+h[3]+h[4]+h[5]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[2]+h[3]+h[4]+h[11]+J[4]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[2]+h[3]+h[4]+h[17]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[2]+h[3]+h[10]+h[5]+J[3]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[2]+h[3]+h[10]+h[11]+J[3]+J[4]+J[9]+J[14],
		h[6]+h[13]+h[2]+h[3]+h[10]+h[17]+J[3]+J[8]+J[9],
		h[6]+h[13]+h[2]+h[3]+h[16]+h[5]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[2]+h[3]+h[16]+h[11]+J[4]+J[7]+J[9],
		h[6]+h[13]+h[2]+h[3]+h[16]+h[17]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[13]+h[2]+h[9]+h[4]+h[5]+J[2]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[2]+h[9]+h[4]+h[11]+J[2]+J[4]+J[10]+J[13],
		h[6]+h[13]+h[2]+h[9]+h[4]+h[17]+J[2]+J[8]+J[10],
		h[6]+h[13]+h[2]+h[9]+h[10]+h[5]+J[2]+J[3]+J[11]+J[12],
		h[6]+h[13]+h[2]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[2]+h[9]+h[10]+h[17]+J[2]+J[3]+J[8]+J[12],
		h[6]+h[13]+h[2]+h[9]+h[16]+h[5]+J[2]+J[7]+J[11],
		h[6]+h[13]+h[2]+h[9]+h[16]+h[11]+J[2]+J[4]+J[7]+J[13],
		h[6]+h[13]+h[2]+h[9]+h[16]+h[17]+J[2]+J[7]+J[8]+J[14],
		h[6]+h[13]+h[2]+h[15]+h[4]+h[5]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[2]+h[15]+h[4]+h[11]+J[4]+J[6]+J[10],
		h[6]+h[13]+h[2]+h[15]+h[4]+h[17]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[13]+h[2]+h[15]+h[10]+h[5]+J[3]+J[6]+J[11],
		h[6]+h[13]+h[2]+h[15]+h[10]+h[11]+J[3]+J[4]+J[6]+J[14],
		h[6]+h[13]+h[2]+h[15]+h[10]+h[17]+J[3]+J[6]+J[8]+J[13],
		h[6]+h[13]+h[2]+h[15]+h[16]+h[5]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[13]+h[2]+h[15]+h[16]+h[11]+J[4]+J[6]+J[7]+J[12],
		h[6]+h[13]+h[2]+h[15]+h[16]+h[17]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[8]+h[3]+h[4]+h[5]+J[1]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[8]+h[3]+h[4]+h[11]+J[1]+J[4]+J[11]+J[12],
		h[6]+h[13]+h[8]+h[3]+h[4]+h[17]+J[1]+J[8]+J[12],
		h[6]+h[13]+h[8]+h[3]+h[10]+h[5]+J[1]+J[3]+J[10]+J[13],
		h[6]+h[13]+h[8]+h[3]+h[10]+h[11]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[8]+h[3]+h[10]+h[17]+J[1]+J[3]+J[8]+J[10],
		h[6]+h[13]+h[8]+h[3]+h[16]+h[5]+J[1]+J[7]+J[13],
		h[6]+h[13]+h[8]+h[3]+h[16]+h[11]+J[1]+J[4]+J[7]+J[11],
		h[6]+h[13]+h[8]+h[3]+h[16]+h[17]+J[1]+J[7]+J[8]+J[14],
		h[6]+h[13]+h[8]+h[9]+h[4]+h[5]+J[1]+J[2]+J[9]+J[14],
		h[6]+h[13]+h[8]+h[9]+h[4]+h[11]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[8]+h[9]+h[4]+h[17]+J[1]+J[2]+J[8]+J[9],
		h[6]+h[13]+h[8]+h[9]+h[10]+h[5]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[8]+h[9]+h[10]+h[11]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[8]+h[9]+h[10]+h[17]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[8]+h[9]+h[16]+h[5]+J[1]+J[2]+J[7]+J[9],
		h[6]+h[13]+h[8]+h[9]+h[16]+h[11]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[8]+h[9]+h[16]+h[17]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[13]+h[8]+h[15]+h[4]+h[5]+J[1]+J[6]+J[14],
		h[6]+h[13]+h[8]+h[15]+h[4]+h[11]+J[1]+J[4]+J[6]+J[11],
		h[6]+h[13]+h[8]+h[15]+h[4]+h[17]+J[1]+J[6]+J[8]+J[13],
		h[6]+h[13]+h[8]+h[15]+h[10]+h[5]+J[1]+J[3]+J[6]+J[10],
		h[6]+h[13]+h[8]+h[15]+h[10]+h[11]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[8]+h[15]+h[10]+h[17]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[13]+h[8]+h[15]+h[16]+h[5]+J[1]+J[6]+J[7]+J[12],
		h[6]+h[13]+h[8]+h[15]+h[16]+h[11]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[13]+h[8]+h[15]+h[16]+h[17]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[14]+h[3]+h[4]+h[5]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[14]+h[3]+h[4]+h[11]+J[4]+J[5]+J[12],
		h[6]+h[13]+h[14]+h[3]+h[4]+h[17]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[13]+h[14]+h[3]+h[10]+h[5]+J[3]+J[5]+J[13],
		h[6]+h[13]+h[14]+h[3]+h[10]+h[11]+J[3]+J[4]+J[5]+J[14],
		h[6]+h[13]+h[14]+h[3]+h[10]+h[17]+J[3]+J[5]+J[8]+J[11],
		h[6]+h[13]+h[14]+h[3]+h[16]+h[5]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[13]+h[14]+h[3]+h[16]+h[11]+J[4]+J[5]+J[7]+J[10],
		h[6]+h[13]+h[14]+h[3]+h[16]+h[17]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[14]+h[9]+h[4]+h[5]+J[2]+J[5]+J[14],
		h[6]+h[13]+h[14]+h[9]+h[4]+h[11]+J[2]+J[4]+J[5]+J[13],
		h[6]+h[13]+h[14]+h[9]+h[4]+h[17]+J[2]+J[5]+J[8]+J[11],
		h[6]+h[13]+h[14]+h[9]+h[10]+h[5]+J[2]+J[3]+J[5]+J[12],
		h[6]+h[13]+h[14]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[14]+h[9]+h[10]+h[17]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[13]+h[14]+h[9]+h[16]+h[5]+J[2]+J[5]+J[7]+J[10],
		h[6]+h[13]+h[14]+h[9]+h[16]+h[11]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[13]+h[14]+h[9]+h[16]+h[17]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[14]+h[15]+h[4]+h[5]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[13]+h[14]+h[15]+h[4]+h[11]+J[4]+J[5]+J[6]+J[9],
		h[6]+h[13]+h[14]+h[15]+h[4]+h[17]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[14]+h[15]+h[10]+h[5]+J[3]+J[5]+J[6]+J[9],
		h[6]+h[13]+h[14]+h[15]+h[10]+h[11]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[13]+h[14]+h[15]+h[10]+h[17]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[14]+h[15]+h[16]+h[5]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[14]+h[15]+h[16]+h[11]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[14]+h[15]+h[16]+h[17]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[2]+h[3]+h[4]+h[5]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[2]+h[3]+h[4]+h[11]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[2]+h[3]+h[4]+h[17]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[2]+h[3]+h[10]+h[5]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[2]+h[3]+h[10]+h[11]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[1]+h[2]+h[3]+h[10]+h[17]+J[4]+J[5]+J[6]+J[9],
		h[12]+h[1]+h[2]+h[3]+h[16]+h[5]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[2]+h[3]+h[16]+h[11]+J[3]+J[5]+J[6]+J[9],
		h[12]+h[1]+h[2]+h[3]+h[16]+h[17]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[1]+h[2]+h[9]+h[4]+h[5]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[2]+h[9]+h[4]+h[11]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[1]+h[2]+h[9]+h[4]+h[17]+J[4]+J[5]+J[7]+J[10],
		h[12]+h[1]+h[2]+h[9]+h[10]+h[5]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[1]+h[2]+h[9]+h[10]+h[11]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[2]+h[9]+h[10]+h[17]+J[4]+J[5]+J[12],
		h[12]+h[1]+h[2]+h[9]+h[16]+h[5]+J[3]+J[5]+J[8]+J[11],
		h[12]+h[1]+h[2]+h[9]+h[16]+h[11]+J[3]+J[5]+J[13],
		h[12]+h[1]+h[2]+h[9]+h[16]+h[17]+J[3]+J[4]+J[5]+J[14],
		h[12]+h[1]+h[2]+h[15]+h[4]+h[5]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[2]+h[15]+h[4]+h[11]+J[2]+J[5]+J[7]+J[10],
		h[12]+h[1]+h[2]+h[15]+h[4]+h[17]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[1]+h[2]+h[15]+h[10]+h[5]+J[2]+J[5]+J[8]+J[11],
		h[12]+h[1]+h[2]+h[15]+h[10]+h[11]+J[2]+J[5]+J[14],
		h[12]+h[1]+h[2]+h[15]+h[10]+h[17]+J[2]+J[4]+J[5]+J[13],
		h[12]+h[1]+h[2]+h[15]+h[16]+h[5]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[1]+h[2]+h[15]+h[16]+h[11]+J[2]+J[3]+J[5]+J[12],
		h[12]+h[1]+h[2]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[8]+h[3]+h[4]+h[5]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[8]+h[3]+h[4]+h[11]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[1]+h[8]+h[3]+h[4]+h[17]+J[4]+J[6]+J[7]+J[12],
		h[12]+h[1]+h[8]+h[3]+h[10]+h[5]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[1]+h[8]+h[3]+h[10]+h[11]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[8]+h[3]+h[10]+h[17]+J[4]+J[6]+J[10],
		h[12]+h[1]+h[8]+h[3]+h[16]+h[5]+J[3]+J[6]+J[8]+J[13],
		h[12]+h[1]+h[8]+h[3]+h[16]+h[11]+J[3]+J[6]+J[11],
		h[12]+h[1]+h[8]+h[3]+h[16]+h[17]+J[3]+J[4]+J[6]+J[14],
		h[12]+h[1]+h[8]+h[9]+h[4]+h[5]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[1]+h[8]+h[9]+h[4]+h[11]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[8]+h[9]+h[4]+h[17]+J[4]+J[7]+J[9],
		h[12]+h[1]+h[8]+h[9]+h[10]+h[5]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[8]+h[9]+h[10]+h[11]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[8]+h[9]+h[10]+h[17]+J[4]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[8]+h[9]+h[16]+h[5]+J[3]+J[8]+J[9],
		h[12]+h[1]+h[8]+h[9]+h[16]+h[11]+J[3]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[8]+h[9]+h[16]+h[17]+J[3]+J[4]+J[9]+J[14],
		h[12]+h[1]+h[8]+h[15]+h[4]+h[5]+J[2]+J[7]+J[8]+J[14],
		h[12]+h[1]+h[8]+h[15]+h[4]+h[11]+J[2]+J[7]+J[11],
		h[12]+h[1]+h[8]+h[15]+h[4]+h[17]+J[2]+J[4]+J[7]+J[13],
		h[12]+h[1]+h[8]+h[15]+h[10]+h[5]+J[2]+J[8]+J[10],
		h[12]+h[1]+h[8]+h[15]+h[10]+h[11]+J[2]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[8]+h[15]+h[10]+h[17]+J[2]+J[4]+J[10]+J[13],
		h[12]+h[1]+h[8]+h[15]+h[16]+h[5]+J[2]+J[3]+J[8]+J[12],
		h[12]+h[1]+h[8]+h[15]+h[16]+h[11]+J[2]+J[3]+J[11]+J[12],
		h[12]+h[1]+h[8]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[14]+h[3]+h[4]+h[5]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[14]+h[3]+h[4]+h[11]+J[1]+J[6]+J[7]+J[12],
		h[12]+h[1]+h[14]+h[3]+h[4]+h[17]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[1]+h[14]+h[3]+h[10]+h[5]+J[1]+J[6]+J[8]+J[13],
		h[12]+h[1]+h[14]+h[3]+h[10]+h[11]+J[1]+J[6]+J[14],
		h[12]+h[1]+h[14]+h[3]+h[10]+h[17]+J[1]+J[4]+J[6]+J[11],
		h[12]+h[1]+h[14]+h[3]+h[16]+h[5]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[1]+h[14]+h[3]+h[16]+h[11]+J[1]+J[3]+J[6]+J[10],
		h[12]+h[1]+h[14]+h[3]+h[16]+h[17]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[14]+h[9]+h[4]+h[5]+J[1]+J[7]+J[8]+J[14],
		h[12]+h[1]+h[14]+h[9]+h[4]+h[11]+J[1]+J[7]+J[13],
		h[12]+h[1]+h[14]+h[9]+h[4]+h[17]+J[1]+J[4]+J[7]+J[11],
		h[12]+h[1]+h[14]+h[9]+h[10]+h[5]+J[1]+J[8]+J[12],
		h[12]+h[1]+h[14]+h[9]+h[10]+h[11]+J[1]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[14]+h[9]+h[10]+h[17]+J[1]+J[4]+J[11]+J[12],
		h[12]+h[1]+h[14]+h[9]+h[16]+h[5]+J[1]+J[3]+J[8]+J[10],
		h[12]+h[1]+h[14]+h[9]+h[16]+h[11]+J[1]+J[3]+J[10]+J[13],
		h[12]+h[1]+h[14]+h[9]+h[16]+h[17]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[14]+h[15]+h[4]+h[5]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[1]+h[14]+h[15]+h[4]+h[11]+J[1]+J[2]+J[7]+J[9],
		h[12]+h[1]+h[14]+h[15]+h[4]+h[17]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[14]+h[15]+h[10]+h[5]+J[1]+J[2]+J[8]+J[9],
		h[12]+h[1]+h[14]+h[15]+h[10]+h[11]+J[1]+J[2]+J[9]+J[14],
		h[12]+h[1]+h[14]+h[15]+h[10]+h[17]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[14]+h[15]+h[16]+h[5]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[14]+h[15]+h[16]+h[11]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[14]+h[15]+h[16]+h[17]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[2]+h[3]+h[4]+h[5]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[2]+h[3]+h[4]+h[11]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[2]+h[3]+h[4]+h[17]+J[4]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[2]+h[3]+h[10]+h[5]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[2]+h[3]+h[10]+h[11]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[7]+h[2]+h[3]+h[10]+h[17]+J[4]+J[7]+J[9],
		h[12]+h[7]+h[2]+h[3]+h[16]+h[5]+J[3]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[2]+h[3]+h[16]+h[11]+J[3]+J[8]+J[9],
		h[12]+h[7]+h[2]+h[3]+h[16]+h[17]+J[3]+J[4]+J[9]+J[14],
		h[12]+h[7]+h[2]+h[9]+h[4]+h[5]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[2]+h[9]+h[4]+h[11]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[7]+h[2]+h[9]+h[4]+h[17]+J[4]+J[6]+J[10],
		h[12]+h[7]+h[2]+h[9]+h[10]+h[5]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[7]+h[2]+h[9]+h[10]+h[11]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[2]+h[9]+h[10]+h[17]+J[4]+J[6]+J[7]+J[12],
		h[12]+h[7]+h[2]+h[9]+h[16]+h[5]+J[3]+J[6]+J[11],
		h[12]+h[7]+h[2]+h[9]+h[16]+h[11]+J[3]+J[6]+J[8]+J[13],
		h[12]+h[7]+h[2]+h[9]+h[16]+h[17]+J[3]+J[4]+J[6]+J[14],
		h[12]+h[7]+h[2]+h[15]+h[4]+h[5]+J[2]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[2]+h[15]+h[4]+h[11]+J[2]+J[8]+J[10],
		h[12]+h[7]+h[2]+h[15]+h[4]+h[17]+J[2]+J[4]+J[10]+J[13],
		h[12]+h[7]+h[2]+h[15]+h[10]+h[5]+J[2]+J[7]+J[11],
		h[12]+h[7]+h[2]+h[15]+h[10]+h[11]+J[2]+J[7]+J[8]+J[14],
		h[12]+h[7]+h[2]+h[15]+h[10]+h[17]+J[2]+J[4]+J[7]+J[13],
		h[12]+h[7]+h[2]+h[15]+h[16]+h[5]+J[2]+J[3]+J[11]+J[12],
		h[12]+h[7]+h[2]+h[15]+h[16]+h[11]+J[2]+J[3]+J[8]+J[12],
		h[12]+h[7]+h[2]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[8]+h[3]+h[4]+h[5]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[8]+h[3]+h[4]+h[11]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[7]+h[8]+h[3]+h[4]+h[17]+J[4]+J[5]+J[12],
		h[12]+h[7]+h[8]+h[3]+h[10]+h[5]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[7]+h[8]+h[3]+h[10]+h[11]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[8]+h[3]+h[10]+h[17]+J[4]+J[5]+J[7]+J[10],
		h[12]+h[7]+h[8]+h[3]+h[16]+h[5]+J[3]+J[5]+J[13],
		h[12]+h[7]+h[8]+h[3]+h[16]+h[11]+J[3]+J[5]+J[8]+J[11],
		h[12]+h[7]+h[8]+h[3]+h[16]+h[17]+J[3]+J[4]+J[5]+J[14],
		h[12]+h[7]+h[8]+h[9]+h[4]+h[5]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[7]+h[8]+h[9]+h[4]+h[11]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[8]+h[9]+h[4]+h[17]+J[4]+J[5]+J[6]+J[9],
		h[12]+h[7]+h[8]+h[9]+h[10]+h[5]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[8]+h[9]+h[10]+h[11]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[8]+h[9]+h[10]+h[17]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[8]+h[9]+h[16]+h[5]+J[3]+J[5]+J[6]+J[9],
		h[12]+h[7]+h[8]+h[9]+h[16]+h[11]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[8]+h[9]+h[16]+h[17]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[7]+h[8]+h[15]+h[4]+h[5]+J[2]+J[5]+J[14],
		h[12]+h[7]+h[8]+h[15]+h[4]+h[11]+J[2]+J[5]+J[8]+J[11],
		h[12]+h[7]+h[8]+h[15]+h[4]+h[17]+J[2]+J[4]+J[5]+J[13],
		h[12]+h[7]+h[8]+h[15]+h[10]+h[5]+J[2]+J[5]+J[7]+J[10],
		h[12]+h[7]+h[8]+h[15]+h[10]+h[11]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[8]+h[15]+h[10]+h[17]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[7]+h[8]+h[15]+h[16]+h[5]+J[2]+J[3]+J[5]+J[12],
		h[12]+h[7]+h[8]+h[15]+h[16]+h[11]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[7]+h[8]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[14]+h[3]+h[4]+h[5]+J[1]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[14]+h[3]+h[4]+h[11]+J[1]+J[8]+J[12],
		h[12]+h[7]+h[14]+h[3]+h[4]+h[17]+J[1]+J[4]+J[11]+J[12],
		h[12]+h[7]+h[14]+h[3]+h[10]+h[5]+J[1]+J[7]+J[13],
		h[12]+h[7]+h[14]+h[3]+h[10]+h[11]+J[1]+J[7]+J[8]+J[14],
		h[12]+h[7]+h[14]+h[3]+h[10]+h[17]+J[1]+J[4]+J[7]+J[11],
		h[12]+h[7]+h[14]+h[3]+h[16]+h[5]+J[1]+J[3]+J[10]+J[13],
		h[12]+h[7]+h[14]+h[3]+h[16]+h[11]+J[1]+J[3]+J[8]+J[10],
		h[12]+h[7]+h[14]+h[3]+h[16]+h[17]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[14]+h[9]+h[4]+h[5]+J[1]+J[6]+J[14],
		h[12]+h[7]+h[14]+h[9]+h[4]+h[11]+J[1]+J[6]+J[8]+J[13],
		h[12]+h[7]+h[14]+h[9]+h[4]+h[17]+J[1]+J[4]+J[6]+J[11],
		h[12]+h[7]+h[14]+h[9]+h[10]+h[5]+J[1]+J[6]+J[7]+J[12],
		h[12]+h[7]+h[14]+h[9]+h[10]+h[11]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[14]+h[9]+h[10]+h[17]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[7]+h[14]+h[9]+h[16]+h[5]+J[1]+J[3]+J[6]+J[10],
		h[12]+h[7]+h[14]+h[9]+h[16]+h[11]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[7]+h[14]+h[9]+h[16]+h[17]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[14]+h[15]+h[4]+h[5]+J[1]+J[2]+J[9]+J[14],
		h[12]+h[7]+h[14]+h[15]+h[4]+h[11]+J[1]+J[2]+J[8]+J[9],
		h[12]+h[7]+h[14]+h[15]+h[4]+h[17]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[14]+h[15]+h[10]+h[5]+J[1]+J[2]+J[7]+J[9],
		h[12]+h[7]+h[14]+h[15]+h[10]+h[11]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[7]+h[14]+h[15]+h[10]+h[17]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[14]+h[15]+h[16]+h[5]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[14]+h[15]+h[16]+h[11]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[14]+h[15]+h[16]+h[17]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[2]+h[3]+h[4]+h[5]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[2]+h[3]+h[4]+h[11]+J[0]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[2]+h[3]+h[4]+h[17]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[2]+h[3]+h[10]+h[5]+J[0]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[2]+h[3]+h[10]+h[11]+J[0]+J[9]+J[14],
		h[12]+h[13]+h[2]+h[3]+h[10]+h[17]+J[0]+J[4]+J[8]+J[9],
		h[12]+h[13]+h[2]+h[3]+h[16]+h[5]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[2]+h[3]+h[16]+h[11]+J[0]+J[3]+J[7]+J[9],
		h[12]+h[13]+h[2]+h[3]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[13]+h[2]+h[9]+h[4]+h[5]+J[0]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[2]+h[9]+h[4]+h[11]+J[0]+J[10]+J[13],
		h[12]+h[13]+h[2]+h[9]+h[4]+h[17]+J[0]+J[4]+J[8]+J[10],
		h[12]+h[13]+h[2]+h[9]+h[10]+h[5]+J[0]+J[11]+J[12],
		h[12]+h[13]+h[2]+h[9]+h[10]+h[11]+J[0]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[2]+h[9]+h[10]+h[17]+J[0]+J[4]+J[8]+J[12],
		h[12]+h[13]+h[2]+h[9]+h[16]+h[5]+J[0]+J[3]+J[7]+J[11],
		h[12]+h[13]+h[2]+h[9]+h[16]+h[11]+J[0]+J[3]+J[7]+J[13],
		h[12]+h[13]+h[2]+h[9]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[12]+h[13]+h[2]+h[15]+h[4]+h[5]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[2]+h[15]+h[4]+h[11]+J[0]+J[2]+J[6]+J[10],
		h[12]+h[13]+h[2]+h[15]+h[4]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[13]+h[2]+h[15]+h[10]+h[5]+J[0]+J[2]+J[6]+J[11],
		h[12]+h[13]+h[2]+h[15]+h[10]+h[11]+J[0]+J[2]+J[6]+J[14],
		h[12]+h[13]+h[2]+h[15]+h[10]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[12]+h[13]+h[2]+h[15]+h[16]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[13]+h[2]+h[15]+h[16]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[12]+h[13]+h[2]+h[15]+h[16]+h[17]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[8]+h[3]+h[4]+h[5]+J[0]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[8]+h[3]+h[4]+h[11]+J[0]+J[11]+J[12],
		h[12]+h[13]+h[8]+h[3]+h[4]+h[17]+J[0]+J[4]+J[8]+J[12],
		h[12]+h[13]+h[8]+h[3]+h[10]+h[5]+J[0]+J[10]+J[13],
		h[12]+h[13]+h[8]+h[3]+h[10]+h[11]+J[0]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[8]+h[3]+h[10]+h[17]+J[0]+J[4]+J[8]+J[10],
		h[12]+h[13]+h[8]+h[3]+h[16]+h[5]+J[0]+J[3]+J[7]+J[13],
		h[12]+h[13]+h[8]+h[3]+h[16]+h[11]+J[0]+J[3]+J[7]+J[11],
		h[12]+h[13]+h[8]+h[3]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[12]+h[13]+h[8]+h[9]+h[4]+h[5]+J[0]+J[9]+J[14],
		h[12]+h[13]+h[8]+h[9]+h[4]+h[11]+J[0]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[8]+h[9]+h[4]+h[17]+J[0]+J[4]+J[8]+J[9],
		h[12]+h[13]+h[8]+h[9]+h[10]+h[5]+J[0]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[8]+h[9]+h[10]+h[11]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[8]+h[9]+h[10]+h[17]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[8]+h[9]+h[16]+h[5]+J[0]+J[3]+J[7]+J[9],
		h[12]+h[13]+h[8]+h[9]+h[16]+h[11]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[8]+h[9]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[13]+h[8]+h[15]+h[4]+h[5]+J[0]+J[2]+J[6]+J[14],
		h[12]+h[13]+h[8]+h[15]+h[4]+h[11]+J[0]+J[2]+J[6]+J[11],
		h[12]+h[13]+h[8]+h[15]+h[4]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[12]+h[13]+h[8]+h[15]+h[10]+h[5]+J[0]+J[2]+J[6]+J[10],
		h[12]+h[13]+h[8]+h[15]+h[10]+h[11]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[8]+h[15]+h[10]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[13]+h[8]+h[15]+h[16]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[12]+h[13]+h[8]+h[15]+h[16]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[13]+h[8]+h[15]+h[16]+h[17]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[14]+h[3]+h[4]+h[5]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[14]+h[3]+h[4]+h[11]+J[0]+J[1]+J[5]+J[12],
		h[12]+h[13]+h[14]+h[3]+h[4]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[13]+h[14]+h[3]+h[10]+h[5]+J[0]+J[1]+J[5]+J[13],
		h[12]+h[13]+h[14]+h[3]+h[10]+h[11]+J[0]+J[1]+J[5]+J[14],
		h[12]+h[13]+h[14]+h[3]+h[10]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[12]+h[13]+h[14]+h[3]+h[16]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[13]+h[14]+h[3]+h[16]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[12]+h[13]+h[14]+h[3]+h[16]+h[17]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[14]+h[9]+h[4]+h[5]+J[0]+J[1]+J[5]+J[14],
		h[12]+h[13]+h[14]+h[9]+h[4]+h[11]+J[0]+J[1]+J[5]+J[13],
		h[12]+h[13]+h[14]+h[9]+h[4]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[12]+h[13]+h[14]+h[9]+h[10]+h[5]+J[0]+J[1]+J[5]+J[12],
		h[12]+h[13]+h[14]+h[9]+h[10]+h[11]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[14]+h[9]+h[10]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[13]+h[14]+h[9]+h[16]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[12]+h[13]+h[14]+h[9]+h[16]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[13]+h[14]+h[9]+h[16]+h[17]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[14]+h[15]+h[4]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[13]+h[14]+h[15]+h[4]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[12]+h[13]+h[14]+h[15]+h[4]+h[17]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[14]+h[15]+h[10]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[12]+h[13]+h[14]+h[15]+h[10]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[13]+h[14]+h[15]+h[10]+h[17]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[14]+h[15]+h[16]+h[5]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[14]+h[15]+h[16]+h[11]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[14]+h[15]+h[16]+h[17]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		]
	logZ = fast_logsumexp(energies)[0]
	observables[0] = exp(fast_logsumexp(energies, [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0])[0] - logZ)
	observables[1] = exp(fast_logsumexp(energies, [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0])[0] - logZ)
	observables[2] = exp(fast_logsumexp(energies, [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0])[0] - logZ)
	observables[3] = exp(fast_logsumexp(energies, [1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,
1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,
1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0])[0] - logZ)
	observables[4] = exp(fast_logsumexp(energies, [1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,
0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,
0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,
0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,
1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,
1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,
0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,
0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,
0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,
0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,
0,0])[0] - logZ)
	observables[5] = exp(fast_logsumexp(energies, [1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0])[0] - logZ)
	observables[6] = exp(fast_logsumexp(energies, [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0])[0] - logZ)
	observables[7] = exp(fast_logsumexp(energies, [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0])[0] - logZ)
	observables[8] = exp(fast_logsumexp(energies, [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0])[0] - logZ)
	observables[9] = exp(fast_logsumexp(energies, [0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,
1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,
1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,
1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,
0,0])[0] - logZ)
	observables[10] = exp(fast_logsumexp(energies, [0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,
0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,
1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,
1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,
0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,
0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,
0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,
0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,
0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,
1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,
1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,
0,0])[0] - logZ)
	observables[11] = exp(fast_logsumexp(energies, [0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0])[0] - logZ)
	observables[12] = exp(fast_logsumexp(energies, [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1])[0] - logZ)
	observables[13] = exp(fast_logsumexp(energies, [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1])[0] - logZ)
	observables[14] = exp(fast_logsumexp(energies, [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1])[0] - logZ)
	observables[15] = exp(fast_logsumexp(energies, [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,
1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,
1,1])[0] - logZ)
	observables[16] = exp(fast_logsumexp(energies, [0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,
0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,
0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,
0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,
0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,
0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,
1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,
1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,
0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,
0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1])[0] - logZ)
	observables[17] = exp(fast_logsumexp(energies, [0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1])[0] - logZ)
	observables[18] = exp(fast_logsumexp(energies, [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1])[0] - logZ)
	observables[19] = exp(fast_logsumexp(energies, [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1])[0] - logZ)
	observables[20] = exp(fast_logsumexp(energies, [1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,
1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,
1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,
1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,
1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,
1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,
1,1])[0] - logZ)
	observables[21] = exp(fast_logsumexp(energies, [1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,
0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,
0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,
0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,
0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,
0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,
1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,
0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,
0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1])[0] - logZ)
	observables[22] = exp(fast_logsumexp(energies, [1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1])[0] - logZ)
	observables[23] = exp(fast_logsumexp(energies, [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1])[0] - logZ)
	observables[24] = exp(fast_logsumexp(energies, [1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,
1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,
1,1])[0] - logZ)
	observables[25] = exp(fast_logsumexp(energies, [1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,
1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,
1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,
1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,
1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1])[0] - logZ)
	observables[26] = exp(fast_logsumexp(energies, [1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,
0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1])[0] - logZ)
	observables[27] = exp(fast_logsumexp(energies, [1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,
1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,
1,1])[0] - logZ)
	observables[28] = exp(fast_logsumexp(energies, [1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,
0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,
1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,1,1,1,0,0,
0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,
0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,1,1,
1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,
0,0,1,1,1,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,
0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,
0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,
1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,
0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,
1,1])[0] - logZ)
	observables[29] = exp(fast_logsumexp(energies, [1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,1,0,
0,1])[0] - logZ)
	observables[30] = exp(fast_logsumexp(energies, [1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,
1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,
0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,
0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,
0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,
0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,
0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,
1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,
0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,
0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,
0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,
0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,
0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,
1,1])[0] - logZ)
	observables[31] = exp(fast_logsumexp(energies, [1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,
0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,
1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,
0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,
0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,
0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,
0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,
1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,
0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,
0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,
0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,
0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,
1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,1,0,
0,1])[0] - logZ)
	observables[32] = exp(fast_logsumexp(energies, [1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,
0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,
0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,
0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,
0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,
1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,
0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,
1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,
0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,
1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,
0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,
0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,
0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0,
0,1])[0] - logZ)
	return observables

def p(params):
	assert params.size==33
	h = params[:18]
	J = params[18:]
	p = zeros(729)
	energies = [		h[0]+h[1]+h[2]+h[3]+h[4]+h[5]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[2]+h[3]+h[4]+h[11]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[2]+h[3]+h[4]+h[17]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[2]+h[3]+h[10]+h[5]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[2]+h[3]+h[10]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[1]+h[2]+h[3]+h[10]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[0]+h[1]+h[2]+h[3]+h[16]+h[5]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[2]+h[3]+h[16]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[0]+h[1]+h[2]+h[3]+h[16]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[1]+h[2]+h[9]+h[4]+h[5]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[2]+h[9]+h[4]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[1]+h[2]+h[9]+h[4]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[0]+h[1]+h[2]+h[9]+h[10]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[1]+h[2]+h[9]+h[10]+h[11]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[2]+h[9]+h[10]+h[17]+J[0]+J[1]+J[5]+J[12],
		h[0]+h[1]+h[2]+h[9]+h[16]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[0]+h[1]+h[2]+h[9]+h[16]+h[11]+J[0]+J[1]+J[5]+J[13],
		h[0]+h[1]+h[2]+h[9]+h[16]+h[17]+J[0]+J[1]+J[5]+J[14],
		h[0]+h[1]+h[2]+h[15]+h[4]+h[5]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[2]+h[15]+h[4]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[0]+h[1]+h[2]+h[15]+h[4]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[1]+h[2]+h[15]+h[10]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[0]+h[1]+h[2]+h[15]+h[10]+h[11]+J[0]+J[1]+J[5]+J[14],
		h[0]+h[1]+h[2]+h[15]+h[10]+h[17]+J[0]+J[1]+J[5]+J[13],
		h[0]+h[1]+h[2]+h[15]+h[16]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[1]+h[2]+h[15]+h[16]+h[11]+J[0]+J[1]+J[5]+J[12],
		h[0]+h[1]+h[2]+h[15]+h[16]+h[17]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[8]+h[3]+h[4]+h[5]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[8]+h[3]+h[4]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[1]+h[8]+h[3]+h[4]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[0]+h[1]+h[8]+h[3]+h[10]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[1]+h[8]+h[3]+h[10]+h[11]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[8]+h[3]+h[10]+h[17]+J[0]+J[2]+J[6]+J[10],
		h[0]+h[1]+h[8]+h[3]+h[16]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[0]+h[1]+h[8]+h[3]+h[16]+h[11]+J[0]+J[2]+J[6]+J[11],
		h[0]+h[1]+h[8]+h[3]+h[16]+h[17]+J[0]+J[2]+J[6]+J[14],
		h[0]+h[1]+h[8]+h[9]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[1]+h[8]+h[9]+h[4]+h[11]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[8]+h[9]+h[4]+h[17]+J[0]+J[3]+J[7]+J[9],
		h[0]+h[1]+h[8]+h[9]+h[10]+h[5]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[8]+h[9]+h[10]+h[11]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[8]+h[9]+h[10]+h[17]+J[0]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[8]+h[9]+h[16]+h[5]+J[0]+J[4]+J[8]+J[9],
		h[0]+h[1]+h[8]+h[9]+h[16]+h[11]+J[0]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[8]+h[9]+h[16]+h[17]+J[0]+J[9]+J[14],
		h[0]+h[1]+h[8]+h[15]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[0]+h[1]+h[8]+h[15]+h[4]+h[11]+J[0]+J[3]+J[7]+J[11],
		h[0]+h[1]+h[8]+h[15]+h[4]+h[17]+J[0]+J[3]+J[7]+J[13],
		h[0]+h[1]+h[8]+h[15]+h[10]+h[5]+J[0]+J[4]+J[8]+J[10],
		h[0]+h[1]+h[8]+h[15]+h[10]+h[11]+J[0]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[8]+h[15]+h[10]+h[17]+J[0]+J[10]+J[13],
		h[0]+h[1]+h[8]+h[15]+h[16]+h[5]+J[0]+J[4]+J[8]+J[12],
		h[0]+h[1]+h[8]+h[15]+h[16]+h[11]+J[0]+J[11]+J[12],
		h[0]+h[1]+h[8]+h[15]+h[16]+h[17]+J[0]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[14]+h[3]+h[4]+h[5]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[14]+h[3]+h[4]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[0]+h[1]+h[14]+h[3]+h[4]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[1]+h[14]+h[3]+h[10]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[0]+h[1]+h[14]+h[3]+h[10]+h[11]+J[0]+J[2]+J[6]+J[14],
		h[0]+h[1]+h[14]+h[3]+h[10]+h[17]+J[0]+J[2]+J[6]+J[11],
		h[0]+h[1]+h[14]+h[3]+h[16]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[1]+h[14]+h[3]+h[16]+h[11]+J[0]+J[2]+J[6]+J[10],
		h[0]+h[1]+h[14]+h[3]+h[16]+h[17]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[14]+h[9]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[0]+h[1]+h[14]+h[9]+h[4]+h[11]+J[0]+J[3]+J[7]+J[13],
		h[0]+h[1]+h[14]+h[9]+h[4]+h[17]+J[0]+J[3]+J[7]+J[11],
		h[0]+h[1]+h[14]+h[9]+h[10]+h[5]+J[0]+J[4]+J[8]+J[12],
		h[0]+h[1]+h[14]+h[9]+h[10]+h[11]+J[0]+J[12]+J[13]+J[14],
		h[0]+h[1]+h[14]+h[9]+h[10]+h[17]+J[0]+J[11]+J[12],
		h[0]+h[1]+h[14]+h[9]+h[16]+h[5]+J[0]+J[4]+J[8]+J[10],
		h[0]+h[1]+h[14]+h[9]+h[16]+h[11]+J[0]+J[10]+J[13],
		h[0]+h[1]+h[14]+h[9]+h[16]+h[17]+J[0]+J[10]+J[11]+J[14],
		h[0]+h[1]+h[14]+h[15]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[1]+h[14]+h[15]+h[4]+h[11]+J[0]+J[3]+J[7]+J[9],
		h[0]+h[1]+h[14]+h[15]+h[4]+h[17]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[14]+h[15]+h[10]+h[5]+J[0]+J[4]+J[8]+J[9],
		h[0]+h[1]+h[14]+h[15]+h[10]+h[11]+J[0]+J[9]+J[14],
		h[0]+h[1]+h[14]+h[15]+h[10]+h[17]+J[0]+J[9]+J[11]+J[13],
		h[0]+h[1]+h[14]+h[15]+h[16]+h[5]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[14]+h[15]+h[16]+h[11]+J[0]+J[9]+J[10]+J[12],
		h[0]+h[1]+h[14]+h[15]+h[16]+h[17]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[2]+h[3]+h[4]+h[5]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[2]+h[3]+h[4]+h[11]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[2]+h[3]+h[4]+h[17]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[2]+h[3]+h[10]+h[5]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[2]+h[3]+h[10]+h[11]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[7]+h[2]+h[3]+h[10]+h[17]+J[1]+J[2]+J[7]+J[9],
		h[0]+h[7]+h[2]+h[3]+h[16]+h[5]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[2]+h[3]+h[16]+h[11]+J[1]+J[2]+J[8]+J[9],
		h[0]+h[7]+h[2]+h[3]+h[16]+h[17]+J[1]+J[2]+J[9]+J[14],
		h[0]+h[7]+h[2]+h[9]+h[4]+h[5]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[2]+h[9]+h[4]+h[11]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[7]+h[2]+h[9]+h[4]+h[17]+J[1]+J[3]+J[6]+J[10],
		h[0]+h[7]+h[2]+h[9]+h[10]+h[5]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[7]+h[2]+h[9]+h[10]+h[11]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[2]+h[9]+h[10]+h[17]+J[1]+J[6]+J[7]+J[12],
		h[0]+h[7]+h[2]+h[9]+h[16]+h[5]+J[1]+J[4]+J[6]+J[11],
		h[0]+h[7]+h[2]+h[9]+h[16]+h[11]+J[1]+J[6]+J[8]+J[13],
		h[0]+h[7]+h[2]+h[9]+h[16]+h[17]+J[1]+J[6]+J[14],
		h[0]+h[7]+h[2]+h[15]+h[4]+h[5]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[2]+h[15]+h[4]+h[11]+J[1]+J[3]+J[8]+J[10],
		h[0]+h[7]+h[2]+h[15]+h[4]+h[17]+J[1]+J[3]+J[10]+J[13],
		h[0]+h[7]+h[2]+h[15]+h[10]+h[5]+J[1]+J[4]+J[7]+J[11],
		h[0]+h[7]+h[2]+h[15]+h[10]+h[11]+J[1]+J[7]+J[8]+J[14],
		h[0]+h[7]+h[2]+h[15]+h[10]+h[17]+J[1]+J[7]+J[13],
		h[0]+h[7]+h[2]+h[15]+h[16]+h[5]+J[1]+J[4]+J[11]+J[12],
		h[0]+h[7]+h[2]+h[15]+h[16]+h[11]+J[1]+J[8]+J[12],
		h[0]+h[7]+h[2]+h[15]+h[16]+h[17]+J[1]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[8]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[8]+h[3]+h[4]+h[11]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[7]+h[8]+h[3]+h[4]+h[17]+J[2]+J[3]+J[5]+J[12],
		h[0]+h[7]+h[8]+h[3]+h[10]+h[5]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[7]+h[8]+h[3]+h[10]+h[11]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[8]+h[3]+h[10]+h[17]+J[2]+J[5]+J[7]+J[10],
		h[0]+h[7]+h[8]+h[3]+h[16]+h[5]+J[2]+J[4]+J[5]+J[13],
		h[0]+h[7]+h[8]+h[3]+h[16]+h[11]+J[2]+J[5]+J[8]+J[11],
		h[0]+h[7]+h[8]+h[3]+h[16]+h[17]+J[2]+J[5]+J[14],
		h[0]+h[7]+h[8]+h[9]+h[4]+h[5]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[7]+h[8]+h[9]+h[4]+h[11]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[8]+h[9]+h[4]+h[17]+J[3]+J[5]+J[6]+J[9],
		h[0]+h[7]+h[8]+h[9]+h[10]+h[5]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[8]+h[9]+h[10]+h[11]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[8]+h[9]+h[10]+h[17]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[8]+h[9]+h[16]+h[5]+J[4]+J[5]+J[6]+J[9],
		h[0]+h[7]+h[8]+h[9]+h[16]+h[11]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[8]+h[9]+h[16]+h[17]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[7]+h[8]+h[15]+h[4]+h[5]+J[3]+J[4]+J[5]+J[14],
		h[0]+h[7]+h[8]+h[15]+h[4]+h[11]+J[3]+J[5]+J[8]+J[11],
		h[0]+h[7]+h[8]+h[15]+h[4]+h[17]+J[3]+J[5]+J[13],
		h[0]+h[7]+h[8]+h[15]+h[10]+h[5]+J[4]+J[5]+J[7]+J[10],
		h[0]+h[7]+h[8]+h[15]+h[10]+h[11]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[8]+h[15]+h[10]+h[17]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[7]+h[8]+h[15]+h[16]+h[5]+J[4]+J[5]+J[12],
		h[0]+h[7]+h[8]+h[15]+h[16]+h[11]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[7]+h[8]+h[15]+h[16]+h[17]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[14]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[14]+h[3]+h[4]+h[11]+J[2]+J[3]+J[8]+J[12],
		h[0]+h[7]+h[14]+h[3]+h[4]+h[17]+J[2]+J[3]+J[11]+J[12],
		h[0]+h[7]+h[14]+h[3]+h[10]+h[5]+J[2]+J[4]+J[7]+J[13],
		h[0]+h[7]+h[14]+h[3]+h[10]+h[11]+J[2]+J[7]+J[8]+J[14],
		h[0]+h[7]+h[14]+h[3]+h[10]+h[17]+J[2]+J[7]+J[11],
		h[0]+h[7]+h[14]+h[3]+h[16]+h[5]+J[2]+J[4]+J[10]+J[13],
		h[0]+h[7]+h[14]+h[3]+h[16]+h[11]+J[2]+J[8]+J[10],
		h[0]+h[7]+h[14]+h[3]+h[16]+h[17]+J[2]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[14]+h[9]+h[4]+h[5]+J[3]+J[4]+J[6]+J[14],
		h[0]+h[7]+h[14]+h[9]+h[4]+h[11]+J[3]+J[6]+J[8]+J[13],
		h[0]+h[7]+h[14]+h[9]+h[4]+h[17]+J[3]+J[6]+J[11],
		h[0]+h[7]+h[14]+h[9]+h[10]+h[5]+J[4]+J[6]+J[7]+J[12],
		h[0]+h[7]+h[14]+h[9]+h[10]+h[11]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[7]+h[14]+h[9]+h[10]+h[17]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[7]+h[14]+h[9]+h[16]+h[5]+J[4]+J[6]+J[10],
		h[0]+h[7]+h[14]+h[9]+h[16]+h[11]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[7]+h[14]+h[9]+h[16]+h[17]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[7]+h[14]+h[15]+h[4]+h[5]+J[3]+J[4]+J[9]+J[14],
		h[0]+h[7]+h[14]+h[15]+h[4]+h[11]+J[3]+J[8]+J[9],
		h[0]+h[7]+h[14]+h[15]+h[4]+h[17]+J[3]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[14]+h[15]+h[10]+h[5]+J[4]+J[7]+J[9],
		h[0]+h[7]+h[14]+h[15]+h[10]+h[11]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[7]+h[14]+h[15]+h[10]+h[17]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[7]+h[14]+h[15]+h[16]+h[5]+J[4]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[14]+h[15]+h[16]+h[11]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[7]+h[14]+h[15]+h[16]+h[17]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[2]+h[3]+h[4]+h[5]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[2]+h[3]+h[4]+h[11]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[2]+h[3]+h[4]+h[17]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[2]+h[3]+h[10]+h[5]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[2]+h[3]+h[10]+h[11]+J[1]+J[2]+J[9]+J[14],
		h[0]+h[13]+h[2]+h[3]+h[10]+h[17]+J[1]+J[2]+J[8]+J[9],
		h[0]+h[13]+h[2]+h[3]+h[16]+h[5]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[2]+h[3]+h[16]+h[11]+J[1]+J[2]+J[7]+J[9],
		h[0]+h[13]+h[2]+h[3]+h[16]+h[17]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[13]+h[2]+h[9]+h[4]+h[5]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[2]+h[9]+h[4]+h[11]+J[1]+J[3]+J[10]+J[13],
		h[0]+h[13]+h[2]+h[9]+h[4]+h[17]+J[1]+J[3]+J[8]+J[10],
		h[0]+h[13]+h[2]+h[9]+h[10]+h[5]+J[1]+J[4]+J[11]+J[12],
		h[0]+h[13]+h[2]+h[9]+h[10]+h[11]+J[1]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[2]+h[9]+h[10]+h[17]+J[1]+J[8]+J[12],
		h[0]+h[13]+h[2]+h[9]+h[16]+h[5]+J[1]+J[4]+J[7]+J[11],
		h[0]+h[13]+h[2]+h[9]+h[16]+h[11]+J[1]+J[7]+J[13],
		h[0]+h[13]+h[2]+h[9]+h[16]+h[17]+J[1]+J[7]+J[8]+J[14],
		h[0]+h[13]+h[2]+h[15]+h[4]+h[5]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[2]+h[15]+h[4]+h[11]+J[1]+J[3]+J[6]+J[10],
		h[0]+h[13]+h[2]+h[15]+h[4]+h[17]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[13]+h[2]+h[15]+h[10]+h[5]+J[1]+J[4]+J[6]+J[11],
		h[0]+h[13]+h[2]+h[15]+h[10]+h[11]+J[1]+J[6]+J[14],
		h[0]+h[13]+h[2]+h[15]+h[10]+h[17]+J[1]+J[6]+J[8]+J[13],
		h[0]+h[13]+h[2]+h[15]+h[16]+h[5]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[13]+h[2]+h[15]+h[16]+h[11]+J[1]+J[6]+J[7]+J[12],
		h[0]+h[13]+h[2]+h[15]+h[16]+h[17]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[8]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[8]+h[3]+h[4]+h[11]+J[2]+J[3]+J[11]+J[12],
		h[0]+h[13]+h[8]+h[3]+h[4]+h[17]+J[2]+J[3]+J[8]+J[12],
		h[0]+h[13]+h[8]+h[3]+h[10]+h[5]+J[2]+J[4]+J[10]+J[13],
		h[0]+h[13]+h[8]+h[3]+h[10]+h[11]+J[2]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[8]+h[3]+h[10]+h[17]+J[2]+J[8]+J[10],
		h[0]+h[13]+h[8]+h[3]+h[16]+h[5]+J[2]+J[4]+J[7]+J[13],
		h[0]+h[13]+h[8]+h[3]+h[16]+h[11]+J[2]+J[7]+J[11],
		h[0]+h[13]+h[8]+h[3]+h[16]+h[17]+J[2]+J[7]+J[8]+J[14],
		h[0]+h[13]+h[8]+h[9]+h[4]+h[5]+J[3]+J[4]+J[9]+J[14],
		h[0]+h[13]+h[8]+h[9]+h[4]+h[11]+J[3]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[8]+h[9]+h[4]+h[17]+J[3]+J[8]+J[9],
		h[0]+h[13]+h[8]+h[9]+h[10]+h[5]+J[4]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[8]+h[9]+h[10]+h[11]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[8]+h[9]+h[10]+h[17]+J[8]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[8]+h[9]+h[16]+h[5]+J[4]+J[7]+J[9],
		h[0]+h[13]+h[8]+h[9]+h[16]+h[11]+J[7]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[8]+h[9]+h[16]+h[17]+J[7]+J[8]+J[9]+J[14],
		h[0]+h[13]+h[8]+h[15]+h[4]+h[5]+J[3]+J[4]+J[6]+J[14],
		h[0]+h[13]+h[8]+h[15]+h[4]+h[11]+J[3]+J[6]+J[11],
		h[0]+h[13]+h[8]+h[15]+h[4]+h[17]+J[3]+J[6]+J[8]+J[13],
		h[0]+h[13]+h[8]+h[15]+h[10]+h[5]+J[4]+J[6]+J[10],
		h[0]+h[13]+h[8]+h[15]+h[10]+h[11]+J[6]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[8]+h[15]+h[10]+h[17]+J[6]+J[8]+J[10]+J[13],
		h[0]+h[13]+h[8]+h[15]+h[16]+h[5]+J[4]+J[6]+J[7]+J[12],
		h[0]+h[13]+h[8]+h[15]+h[16]+h[11]+J[6]+J[7]+J[11]+J[12],
		h[0]+h[13]+h[8]+h[15]+h[16]+h[17]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[14]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[14]+h[3]+h[4]+h[11]+J[2]+J[3]+J[5]+J[12],
		h[0]+h[13]+h[14]+h[3]+h[4]+h[17]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[13]+h[14]+h[3]+h[10]+h[5]+J[2]+J[4]+J[5]+J[13],
		h[0]+h[13]+h[14]+h[3]+h[10]+h[11]+J[2]+J[5]+J[14],
		h[0]+h[13]+h[14]+h[3]+h[10]+h[17]+J[2]+J[5]+J[8]+J[11],
		h[0]+h[13]+h[14]+h[3]+h[16]+h[5]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[13]+h[14]+h[3]+h[16]+h[11]+J[2]+J[5]+J[7]+J[10],
		h[0]+h[13]+h[14]+h[3]+h[16]+h[17]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[14]+h[9]+h[4]+h[5]+J[3]+J[4]+J[5]+J[14],
		h[0]+h[13]+h[14]+h[9]+h[4]+h[11]+J[3]+J[5]+J[13],
		h[0]+h[13]+h[14]+h[9]+h[4]+h[17]+J[3]+J[5]+J[8]+J[11],
		h[0]+h[13]+h[14]+h[9]+h[10]+h[5]+J[4]+J[5]+J[12],
		h[0]+h[13]+h[14]+h[9]+h[10]+h[11]+J[5]+J[12]+J[13]+J[14],
		h[0]+h[13]+h[14]+h[9]+h[10]+h[17]+J[5]+J[8]+J[11]+J[12],
		h[0]+h[13]+h[14]+h[9]+h[16]+h[5]+J[4]+J[5]+J[7]+J[10],
		h[0]+h[13]+h[14]+h[9]+h[16]+h[11]+J[5]+J[7]+J[10]+J[13],
		h[0]+h[13]+h[14]+h[9]+h[16]+h[17]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[0]+h[13]+h[14]+h[15]+h[4]+h[5]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[13]+h[14]+h[15]+h[4]+h[11]+J[3]+J[5]+J[6]+J[9],
		h[0]+h[13]+h[14]+h[15]+h[4]+h[17]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[14]+h[15]+h[10]+h[5]+J[4]+J[5]+J[6]+J[9],
		h[0]+h[13]+h[14]+h[15]+h[10]+h[11]+J[5]+J[6]+J[9]+J[14],
		h[0]+h[13]+h[14]+h[15]+h[10]+h[17]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[0]+h[13]+h[14]+h[15]+h[16]+h[5]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[14]+h[15]+h[16]+h[11]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[0]+h[13]+h[14]+h[15]+h[16]+h[17]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[2]+h[3]+h[4]+h[5]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[2]+h[3]+h[4]+h[11]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[2]+h[3]+h[4]+h[17]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[2]+h[3]+h[10]+h[5]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[2]+h[3]+h[10]+h[11]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[1]+h[2]+h[3]+h[10]+h[17]+J[3]+J[5]+J[6]+J[9],
		h[6]+h[1]+h[2]+h[3]+h[16]+h[5]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[2]+h[3]+h[16]+h[11]+J[4]+J[5]+J[6]+J[9],
		h[6]+h[1]+h[2]+h[3]+h[16]+h[17]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[1]+h[2]+h[9]+h[4]+h[5]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[2]+h[9]+h[4]+h[11]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[1]+h[2]+h[9]+h[4]+h[17]+J[2]+J[5]+J[7]+J[10],
		h[6]+h[1]+h[2]+h[9]+h[10]+h[5]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[1]+h[2]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[2]+h[9]+h[10]+h[17]+J[2]+J[3]+J[5]+J[12],
		h[6]+h[1]+h[2]+h[9]+h[16]+h[5]+J[2]+J[5]+J[8]+J[11],
		h[6]+h[1]+h[2]+h[9]+h[16]+h[11]+J[2]+J[4]+J[5]+J[13],
		h[6]+h[1]+h[2]+h[9]+h[16]+h[17]+J[2]+J[5]+J[14],
		h[6]+h[1]+h[2]+h[15]+h[4]+h[5]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[2]+h[15]+h[4]+h[11]+J[4]+J[5]+J[7]+J[10],
		h[6]+h[1]+h[2]+h[15]+h[4]+h[17]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[1]+h[2]+h[15]+h[10]+h[5]+J[3]+J[5]+J[8]+J[11],
		h[6]+h[1]+h[2]+h[15]+h[10]+h[11]+J[3]+J[4]+J[5]+J[14],
		h[6]+h[1]+h[2]+h[15]+h[10]+h[17]+J[3]+J[5]+J[13],
		h[6]+h[1]+h[2]+h[15]+h[16]+h[5]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[1]+h[2]+h[15]+h[16]+h[11]+J[4]+J[5]+J[12],
		h[6]+h[1]+h[2]+h[15]+h[16]+h[17]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[8]+h[3]+h[4]+h[5]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[8]+h[3]+h[4]+h[11]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[1]+h[8]+h[3]+h[4]+h[17]+J[1]+J[6]+J[7]+J[12],
		h[6]+h[1]+h[8]+h[3]+h[10]+h[5]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[1]+h[8]+h[3]+h[10]+h[11]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[8]+h[3]+h[10]+h[17]+J[1]+J[3]+J[6]+J[10],
		h[6]+h[1]+h[8]+h[3]+h[16]+h[5]+J[1]+J[6]+J[8]+J[13],
		h[6]+h[1]+h[8]+h[3]+h[16]+h[11]+J[1]+J[4]+J[6]+J[11],
		h[6]+h[1]+h[8]+h[3]+h[16]+h[17]+J[1]+J[6]+J[14],
		h[6]+h[1]+h[8]+h[9]+h[4]+h[5]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[1]+h[8]+h[9]+h[4]+h[11]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[8]+h[9]+h[4]+h[17]+J[1]+J[2]+J[7]+J[9],
		h[6]+h[1]+h[8]+h[9]+h[10]+h[5]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[8]+h[9]+h[10]+h[11]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[8]+h[9]+h[10]+h[17]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[8]+h[9]+h[16]+h[5]+J[1]+J[2]+J[8]+J[9],
		h[6]+h[1]+h[8]+h[9]+h[16]+h[11]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[8]+h[9]+h[16]+h[17]+J[1]+J[2]+J[9]+J[14],
		h[6]+h[1]+h[8]+h[15]+h[4]+h[5]+J[1]+J[7]+J[8]+J[14],
		h[6]+h[1]+h[8]+h[15]+h[4]+h[11]+J[1]+J[4]+J[7]+J[11],
		h[6]+h[1]+h[8]+h[15]+h[4]+h[17]+J[1]+J[7]+J[13],
		h[6]+h[1]+h[8]+h[15]+h[10]+h[5]+J[1]+J[3]+J[8]+J[10],
		h[6]+h[1]+h[8]+h[15]+h[10]+h[11]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[8]+h[15]+h[10]+h[17]+J[1]+J[3]+J[10]+J[13],
		h[6]+h[1]+h[8]+h[15]+h[16]+h[5]+J[1]+J[8]+J[12],
		h[6]+h[1]+h[8]+h[15]+h[16]+h[11]+J[1]+J[4]+J[11]+J[12],
		h[6]+h[1]+h[8]+h[15]+h[16]+h[17]+J[1]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[14]+h[3]+h[4]+h[5]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[14]+h[3]+h[4]+h[11]+J[4]+J[6]+J[7]+J[12],
		h[6]+h[1]+h[14]+h[3]+h[4]+h[17]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[1]+h[14]+h[3]+h[10]+h[5]+J[3]+J[6]+J[8]+J[13],
		h[6]+h[1]+h[14]+h[3]+h[10]+h[11]+J[3]+J[4]+J[6]+J[14],
		h[6]+h[1]+h[14]+h[3]+h[10]+h[17]+J[3]+J[6]+J[11],
		h[6]+h[1]+h[14]+h[3]+h[16]+h[5]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[1]+h[14]+h[3]+h[16]+h[11]+J[4]+J[6]+J[10],
		h[6]+h[1]+h[14]+h[3]+h[16]+h[17]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[14]+h[9]+h[4]+h[5]+J[2]+J[7]+J[8]+J[14],
		h[6]+h[1]+h[14]+h[9]+h[4]+h[11]+J[2]+J[4]+J[7]+J[13],
		h[6]+h[1]+h[14]+h[9]+h[4]+h[17]+J[2]+J[7]+J[11],
		h[6]+h[1]+h[14]+h[9]+h[10]+h[5]+J[2]+J[3]+J[8]+J[12],
		h[6]+h[1]+h[14]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[6]+h[1]+h[14]+h[9]+h[10]+h[17]+J[2]+J[3]+J[11]+J[12],
		h[6]+h[1]+h[14]+h[9]+h[16]+h[5]+J[2]+J[8]+J[10],
		h[6]+h[1]+h[14]+h[9]+h[16]+h[11]+J[2]+J[4]+J[10]+J[13],
		h[6]+h[1]+h[14]+h[9]+h[16]+h[17]+J[2]+J[10]+J[11]+J[14],
		h[6]+h[1]+h[14]+h[15]+h[4]+h[5]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[1]+h[14]+h[15]+h[4]+h[11]+J[4]+J[7]+J[9],
		h[6]+h[1]+h[14]+h[15]+h[4]+h[17]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[14]+h[15]+h[10]+h[5]+J[3]+J[8]+J[9],
		h[6]+h[1]+h[14]+h[15]+h[10]+h[11]+J[3]+J[4]+J[9]+J[14],
		h[6]+h[1]+h[14]+h[15]+h[10]+h[17]+J[3]+J[9]+J[11]+J[13],
		h[6]+h[1]+h[14]+h[15]+h[16]+h[5]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[14]+h[15]+h[16]+h[11]+J[4]+J[9]+J[10]+J[12],
		h[6]+h[1]+h[14]+h[15]+h[16]+h[17]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[2]+h[3]+h[4]+h[5]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[2]+h[3]+h[4]+h[11]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[2]+h[3]+h[4]+h[17]+J[0]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[2]+h[3]+h[10]+h[5]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[2]+h[3]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[7]+h[2]+h[3]+h[10]+h[17]+J[0]+J[3]+J[7]+J[9],
		h[6]+h[7]+h[2]+h[3]+h[16]+h[5]+J[0]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[2]+h[3]+h[16]+h[11]+J[0]+J[4]+J[8]+J[9],
		h[6]+h[7]+h[2]+h[3]+h[16]+h[17]+J[0]+J[9]+J[14],
		h[6]+h[7]+h[2]+h[9]+h[4]+h[5]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[2]+h[9]+h[4]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[7]+h[2]+h[9]+h[4]+h[17]+J[0]+J[2]+J[6]+J[10],
		h[6]+h[7]+h[2]+h[9]+h[10]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[7]+h[2]+h[9]+h[10]+h[11]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[2]+h[9]+h[10]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[6]+h[7]+h[2]+h[9]+h[16]+h[5]+J[0]+J[2]+J[6]+J[11],
		h[6]+h[7]+h[2]+h[9]+h[16]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[6]+h[7]+h[2]+h[9]+h[16]+h[17]+J[0]+J[2]+J[6]+J[14],
		h[6]+h[7]+h[2]+h[15]+h[4]+h[5]+J[0]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[2]+h[15]+h[4]+h[11]+J[0]+J[4]+J[8]+J[10],
		h[6]+h[7]+h[2]+h[15]+h[4]+h[17]+J[0]+J[10]+J[13],
		h[6]+h[7]+h[2]+h[15]+h[10]+h[5]+J[0]+J[3]+J[7]+J[11],
		h[6]+h[7]+h[2]+h[15]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[6]+h[7]+h[2]+h[15]+h[10]+h[17]+J[0]+J[3]+J[7]+J[13],
		h[6]+h[7]+h[2]+h[15]+h[16]+h[5]+J[0]+J[11]+J[12],
		h[6]+h[7]+h[2]+h[15]+h[16]+h[11]+J[0]+J[4]+J[8]+J[12],
		h[6]+h[7]+h[2]+h[15]+h[16]+h[17]+J[0]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[8]+h[3]+h[4]+h[5]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[8]+h[3]+h[4]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[7]+h[8]+h[3]+h[4]+h[17]+J[0]+J[1]+J[5]+J[12],
		h[6]+h[7]+h[8]+h[3]+h[10]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[7]+h[8]+h[3]+h[10]+h[11]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[8]+h[3]+h[10]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[6]+h[7]+h[8]+h[3]+h[16]+h[5]+J[0]+J[1]+J[5]+J[13],
		h[6]+h[7]+h[8]+h[3]+h[16]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[6]+h[7]+h[8]+h[3]+h[16]+h[17]+J[0]+J[1]+J[5]+J[14],
		h[6]+h[7]+h[8]+h[9]+h[4]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[7]+h[8]+h[9]+h[4]+h[11]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[8]+h[9]+h[4]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[6]+h[7]+h[8]+h[9]+h[10]+h[5]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[8]+h[9]+h[10]+h[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[8]+h[9]+h[10]+h[17]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[8]+h[9]+h[16]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[6]+h[7]+h[8]+h[9]+h[16]+h[11]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[8]+h[9]+h[16]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[7]+h[8]+h[15]+h[4]+h[5]+J[0]+J[1]+J[5]+J[14],
		h[6]+h[7]+h[8]+h[15]+h[4]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[6]+h[7]+h[8]+h[15]+h[4]+h[17]+J[0]+J[1]+J[5]+J[13],
		h[6]+h[7]+h[8]+h[15]+h[10]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[6]+h[7]+h[8]+h[15]+h[10]+h[11]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[8]+h[15]+h[10]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[7]+h[8]+h[15]+h[16]+h[5]+J[0]+J[1]+J[5]+J[12],
		h[6]+h[7]+h[8]+h[15]+h[16]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[7]+h[8]+h[15]+h[16]+h[17]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[14]+h[3]+h[4]+h[5]+J[0]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[14]+h[3]+h[4]+h[11]+J[0]+J[4]+J[8]+J[12],
		h[6]+h[7]+h[14]+h[3]+h[4]+h[17]+J[0]+J[11]+J[12],
		h[6]+h[7]+h[14]+h[3]+h[10]+h[5]+J[0]+J[3]+J[7]+J[13],
		h[6]+h[7]+h[14]+h[3]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[6]+h[7]+h[14]+h[3]+h[10]+h[17]+J[0]+J[3]+J[7]+J[11],
		h[6]+h[7]+h[14]+h[3]+h[16]+h[5]+J[0]+J[10]+J[13],
		h[6]+h[7]+h[14]+h[3]+h[16]+h[11]+J[0]+J[4]+J[8]+J[10],
		h[6]+h[7]+h[14]+h[3]+h[16]+h[17]+J[0]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[14]+h[9]+h[4]+h[5]+J[0]+J[2]+J[6]+J[14],
		h[6]+h[7]+h[14]+h[9]+h[4]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[6]+h[7]+h[14]+h[9]+h[4]+h[17]+J[0]+J[2]+J[6]+J[11],
		h[6]+h[7]+h[14]+h[9]+h[10]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[6]+h[7]+h[14]+h[9]+h[10]+h[11]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[7]+h[14]+h[9]+h[10]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[7]+h[14]+h[9]+h[16]+h[5]+J[0]+J[2]+J[6]+J[10],
		h[6]+h[7]+h[14]+h[9]+h[16]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[7]+h[14]+h[9]+h[16]+h[17]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[7]+h[14]+h[15]+h[4]+h[5]+J[0]+J[9]+J[14],
		h[6]+h[7]+h[14]+h[15]+h[4]+h[11]+J[0]+J[4]+J[8]+J[9],
		h[6]+h[7]+h[14]+h[15]+h[4]+h[17]+J[0]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[14]+h[15]+h[10]+h[5]+J[0]+J[3]+J[7]+J[9],
		h[6]+h[7]+h[14]+h[15]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[7]+h[14]+h[15]+h[10]+h[17]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[7]+h[14]+h[15]+h[16]+h[5]+J[0]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[14]+h[15]+h[16]+h[11]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[7]+h[14]+h[15]+h[16]+h[17]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[2]+h[3]+h[4]+h[5]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[2]+h[3]+h[4]+h[11]+J[4]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[2]+h[3]+h[4]+h[17]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[2]+h[3]+h[10]+h[5]+J[3]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[2]+h[3]+h[10]+h[11]+J[3]+J[4]+J[9]+J[14],
		h[6]+h[13]+h[2]+h[3]+h[10]+h[17]+J[3]+J[8]+J[9],
		h[6]+h[13]+h[2]+h[3]+h[16]+h[5]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[2]+h[3]+h[16]+h[11]+J[4]+J[7]+J[9],
		h[6]+h[13]+h[2]+h[3]+h[16]+h[17]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[13]+h[2]+h[9]+h[4]+h[5]+J[2]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[2]+h[9]+h[4]+h[11]+J[2]+J[4]+J[10]+J[13],
		h[6]+h[13]+h[2]+h[9]+h[4]+h[17]+J[2]+J[8]+J[10],
		h[6]+h[13]+h[2]+h[9]+h[10]+h[5]+J[2]+J[3]+J[11]+J[12],
		h[6]+h[13]+h[2]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[2]+h[9]+h[10]+h[17]+J[2]+J[3]+J[8]+J[12],
		h[6]+h[13]+h[2]+h[9]+h[16]+h[5]+J[2]+J[7]+J[11],
		h[6]+h[13]+h[2]+h[9]+h[16]+h[11]+J[2]+J[4]+J[7]+J[13],
		h[6]+h[13]+h[2]+h[9]+h[16]+h[17]+J[2]+J[7]+J[8]+J[14],
		h[6]+h[13]+h[2]+h[15]+h[4]+h[5]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[2]+h[15]+h[4]+h[11]+J[4]+J[6]+J[10],
		h[6]+h[13]+h[2]+h[15]+h[4]+h[17]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[13]+h[2]+h[15]+h[10]+h[5]+J[3]+J[6]+J[11],
		h[6]+h[13]+h[2]+h[15]+h[10]+h[11]+J[3]+J[4]+J[6]+J[14],
		h[6]+h[13]+h[2]+h[15]+h[10]+h[17]+J[3]+J[6]+J[8]+J[13],
		h[6]+h[13]+h[2]+h[15]+h[16]+h[5]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[13]+h[2]+h[15]+h[16]+h[11]+J[4]+J[6]+J[7]+J[12],
		h[6]+h[13]+h[2]+h[15]+h[16]+h[17]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[8]+h[3]+h[4]+h[5]+J[1]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[8]+h[3]+h[4]+h[11]+J[1]+J[4]+J[11]+J[12],
		h[6]+h[13]+h[8]+h[3]+h[4]+h[17]+J[1]+J[8]+J[12],
		h[6]+h[13]+h[8]+h[3]+h[10]+h[5]+J[1]+J[3]+J[10]+J[13],
		h[6]+h[13]+h[8]+h[3]+h[10]+h[11]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[8]+h[3]+h[10]+h[17]+J[1]+J[3]+J[8]+J[10],
		h[6]+h[13]+h[8]+h[3]+h[16]+h[5]+J[1]+J[7]+J[13],
		h[6]+h[13]+h[8]+h[3]+h[16]+h[11]+J[1]+J[4]+J[7]+J[11],
		h[6]+h[13]+h[8]+h[3]+h[16]+h[17]+J[1]+J[7]+J[8]+J[14],
		h[6]+h[13]+h[8]+h[9]+h[4]+h[5]+J[1]+J[2]+J[9]+J[14],
		h[6]+h[13]+h[8]+h[9]+h[4]+h[11]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[8]+h[9]+h[4]+h[17]+J[1]+J[2]+J[8]+J[9],
		h[6]+h[13]+h[8]+h[9]+h[10]+h[5]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[8]+h[9]+h[10]+h[11]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[8]+h[9]+h[10]+h[17]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[8]+h[9]+h[16]+h[5]+J[1]+J[2]+J[7]+J[9],
		h[6]+h[13]+h[8]+h[9]+h[16]+h[11]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[8]+h[9]+h[16]+h[17]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[6]+h[13]+h[8]+h[15]+h[4]+h[5]+J[1]+J[6]+J[14],
		h[6]+h[13]+h[8]+h[15]+h[4]+h[11]+J[1]+J[4]+J[6]+J[11],
		h[6]+h[13]+h[8]+h[15]+h[4]+h[17]+J[1]+J[6]+J[8]+J[13],
		h[6]+h[13]+h[8]+h[15]+h[10]+h[5]+J[1]+J[3]+J[6]+J[10],
		h[6]+h[13]+h[8]+h[15]+h[10]+h[11]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[8]+h[15]+h[10]+h[17]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[6]+h[13]+h[8]+h[15]+h[16]+h[5]+J[1]+J[6]+J[7]+J[12],
		h[6]+h[13]+h[8]+h[15]+h[16]+h[11]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[6]+h[13]+h[8]+h[15]+h[16]+h[17]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[14]+h[3]+h[4]+h[5]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[14]+h[3]+h[4]+h[11]+J[4]+J[5]+J[12],
		h[6]+h[13]+h[14]+h[3]+h[4]+h[17]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[13]+h[14]+h[3]+h[10]+h[5]+J[3]+J[5]+J[13],
		h[6]+h[13]+h[14]+h[3]+h[10]+h[11]+J[3]+J[4]+J[5]+J[14],
		h[6]+h[13]+h[14]+h[3]+h[10]+h[17]+J[3]+J[5]+J[8]+J[11],
		h[6]+h[13]+h[14]+h[3]+h[16]+h[5]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[13]+h[14]+h[3]+h[16]+h[11]+J[4]+J[5]+J[7]+J[10],
		h[6]+h[13]+h[14]+h[3]+h[16]+h[17]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[14]+h[9]+h[4]+h[5]+J[2]+J[5]+J[14],
		h[6]+h[13]+h[14]+h[9]+h[4]+h[11]+J[2]+J[4]+J[5]+J[13],
		h[6]+h[13]+h[14]+h[9]+h[4]+h[17]+J[2]+J[5]+J[8]+J[11],
		h[6]+h[13]+h[14]+h[9]+h[10]+h[5]+J[2]+J[3]+J[5]+J[12],
		h[6]+h[13]+h[14]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[6]+h[13]+h[14]+h[9]+h[10]+h[17]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[6]+h[13]+h[14]+h[9]+h[16]+h[5]+J[2]+J[5]+J[7]+J[10],
		h[6]+h[13]+h[14]+h[9]+h[16]+h[11]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[6]+h[13]+h[14]+h[9]+h[16]+h[17]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[6]+h[13]+h[14]+h[15]+h[4]+h[5]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[13]+h[14]+h[15]+h[4]+h[11]+J[4]+J[5]+J[6]+J[9],
		h[6]+h[13]+h[14]+h[15]+h[4]+h[17]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[14]+h[15]+h[10]+h[5]+J[3]+J[5]+J[6]+J[9],
		h[6]+h[13]+h[14]+h[15]+h[10]+h[11]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[6]+h[13]+h[14]+h[15]+h[10]+h[17]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[6]+h[13]+h[14]+h[15]+h[16]+h[5]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[14]+h[15]+h[16]+h[11]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[6]+h[13]+h[14]+h[15]+h[16]+h[17]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[2]+h[3]+h[4]+h[5]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[2]+h[3]+h[4]+h[11]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[2]+h[3]+h[4]+h[17]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[2]+h[3]+h[10]+h[5]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[2]+h[3]+h[10]+h[11]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[1]+h[2]+h[3]+h[10]+h[17]+J[4]+J[5]+J[6]+J[9],
		h[12]+h[1]+h[2]+h[3]+h[16]+h[5]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[2]+h[3]+h[16]+h[11]+J[3]+J[5]+J[6]+J[9],
		h[12]+h[1]+h[2]+h[3]+h[16]+h[17]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[1]+h[2]+h[9]+h[4]+h[5]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[2]+h[9]+h[4]+h[11]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[1]+h[2]+h[9]+h[4]+h[17]+J[4]+J[5]+J[7]+J[10],
		h[12]+h[1]+h[2]+h[9]+h[10]+h[5]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[1]+h[2]+h[9]+h[10]+h[11]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[2]+h[9]+h[10]+h[17]+J[4]+J[5]+J[12],
		h[12]+h[1]+h[2]+h[9]+h[16]+h[5]+J[3]+J[5]+J[8]+J[11],
		h[12]+h[1]+h[2]+h[9]+h[16]+h[11]+J[3]+J[5]+J[13],
		h[12]+h[1]+h[2]+h[9]+h[16]+h[17]+J[3]+J[4]+J[5]+J[14],
		h[12]+h[1]+h[2]+h[15]+h[4]+h[5]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[2]+h[15]+h[4]+h[11]+J[2]+J[5]+J[7]+J[10],
		h[12]+h[1]+h[2]+h[15]+h[4]+h[17]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[1]+h[2]+h[15]+h[10]+h[5]+J[2]+J[5]+J[8]+J[11],
		h[12]+h[1]+h[2]+h[15]+h[10]+h[11]+J[2]+J[5]+J[14],
		h[12]+h[1]+h[2]+h[15]+h[10]+h[17]+J[2]+J[4]+J[5]+J[13],
		h[12]+h[1]+h[2]+h[15]+h[16]+h[5]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[1]+h[2]+h[15]+h[16]+h[11]+J[2]+J[3]+J[5]+J[12],
		h[12]+h[1]+h[2]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[8]+h[3]+h[4]+h[5]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[8]+h[3]+h[4]+h[11]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[1]+h[8]+h[3]+h[4]+h[17]+J[4]+J[6]+J[7]+J[12],
		h[12]+h[1]+h[8]+h[3]+h[10]+h[5]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[1]+h[8]+h[3]+h[10]+h[11]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[8]+h[3]+h[10]+h[17]+J[4]+J[6]+J[10],
		h[12]+h[1]+h[8]+h[3]+h[16]+h[5]+J[3]+J[6]+J[8]+J[13],
		h[12]+h[1]+h[8]+h[3]+h[16]+h[11]+J[3]+J[6]+J[11],
		h[12]+h[1]+h[8]+h[3]+h[16]+h[17]+J[3]+J[4]+J[6]+J[14],
		h[12]+h[1]+h[8]+h[9]+h[4]+h[5]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[1]+h[8]+h[9]+h[4]+h[11]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[8]+h[9]+h[4]+h[17]+J[4]+J[7]+J[9],
		h[12]+h[1]+h[8]+h[9]+h[10]+h[5]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[8]+h[9]+h[10]+h[11]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[8]+h[9]+h[10]+h[17]+J[4]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[8]+h[9]+h[16]+h[5]+J[3]+J[8]+J[9],
		h[12]+h[1]+h[8]+h[9]+h[16]+h[11]+J[3]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[8]+h[9]+h[16]+h[17]+J[3]+J[4]+J[9]+J[14],
		h[12]+h[1]+h[8]+h[15]+h[4]+h[5]+J[2]+J[7]+J[8]+J[14],
		h[12]+h[1]+h[8]+h[15]+h[4]+h[11]+J[2]+J[7]+J[11],
		h[12]+h[1]+h[8]+h[15]+h[4]+h[17]+J[2]+J[4]+J[7]+J[13],
		h[12]+h[1]+h[8]+h[15]+h[10]+h[5]+J[2]+J[8]+J[10],
		h[12]+h[1]+h[8]+h[15]+h[10]+h[11]+J[2]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[8]+h[15]+h[10]+h[17]+J[2]+J[4]+J[10]+J[13],
		h[12]+h[1]+h[8]+h[15]+h[16]+h[5]+J[2]+J[3]+J[8]+J[12],
		h[12]+h[1]+h[8]+h[15]+h[16]+h[11]+J[2]+J[3]+J[11]+J[12],
		h[12]+h[1]+h[8]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[14]+h[3]+h[4]+h[5]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[14]+h[3]+h[4]+h[11]+J[1]+J[6]+J[7]+J[12],
		h[12]+h[1]+h[14]+h[3]+h[4]+h[17]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[1]+h[14]+h[3]+h[10]+h[5]+J[1]+J[6]+J[8]+J[13],
		h[12]+h[1]+h[14]+h[3]+h[10]+h[11]+J[1]+J[6]+J[14],
		h[12]+h[1]+h[14]+h[3]+h[10]+h[17]+J[1]+J[4]+J[6]+J[11],
		h[12]+h[1]+h[14]+h[3]+h[16]+h[5]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[1]+h[14]+h[3]+h[16]+h[11]+J[1]+J[3]+J[6]+J[10],
		h[12]+h[1]+h[14]+h[3]+h[16]+h[17]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[14]+h[9]+h[4]+h[5]+J[1]+J[7]+J[8]+J[14],
		h[12]+h[1]+h[14]+h[9]+h[4]+h[11]+J[1]+J[7]+J[13],
		h[12]+h[1]+h[14]+h[9]+h[4]+h[17]+J[1]+J[4]+J[7]+J[11],
		h[12]+h[1]+h[14]+h[9]+h[10]+h[5]+J[1]+J[8]+J[12],
		h[12]+h[1]+h[14]+h[9]+h[10]+h[11]+J[1]+J[12]+J[13]+J[14],
		h[12]+h[1]+h[14]+h[9]+h[10]+h[17]+J[1]+J[4]+J[11]+J[12],
		h[12]+h[1]+h[14]+h[9]+h[16]+h[5]+J[1]+J[3]+J[8]+J[10],
		h[12]+h[1]+h[14]+h[9]+h[16]+h[11]+J[1]+J[3]+J[10]+J[13],
		h[12]+h[1]+h[14]+h[9]+h[16]+h[17]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[12]+h[1]+h[14]+h[15]+h[4]+h[5]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[1]+h[14]+h[15]+h[4]+h[11]+J[1]+J[2]+J[7]+J[9],
		h[12]+h[1]+h[14]+h[15]+h[4]+h[17]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[14]+h[15]+h[10]+h[5]+J[1]+J[2]+J[8]+J[9],
		h[12]+h[1]+h[14]+h[15]+h[10]+h[11]+J[1]+J[2]+J[9]+J[14],
		h[12]+h[1]+h[14]+h[15]+h[10]+h[17]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[12]+h[1]+h[14]+h[15]+h[16]+h[5]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[14]+h[15]+h[16]+h[11]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[12]+h[1]+h[14]+h[15]+h[16]+h[17]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[2]+h[3]+h[4]+h[5]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[2]+h[3]+h[4]+h[11]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[2]+h[3]+h[4]+h[17]+J[4]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[2]+h[3]+h[10]+h[5]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[2]+h[3]+h[10]+h[11]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[7]+h[2]+h[3]+h[10]+h[17]+J[4]+J[7]+J[9],
		h[12]+h[7]+h[2]+h[3]+h[16]+h[5]+J[3]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[2]+h[3]+h[16]+h[11]+J[3]+J[8]+J[9],
		h[12]+h[7]+h[2]+h[3]+h[16]+h[17]+J[3]+J[4]+J[9]+J[14],
		h[12]+h[7]+h[2]+h[9]+h[4]+h[5]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[2]+h[9]+h[4]+h[11]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[7]+h[2]+h[9]+h[4]+h[17]+J[4]+J[6]+J[10],
		h[12]+h[7]+h[2]+h[9]+h[10]+h[5]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[7]+h[2]+h[9]+h[10]+h[11]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[2]+h[9]+h[10]+h[17]+J[4]+J[6]+J[7]+J[12],
		h[12]+h[7]+h[2]+h[9]+h[16]+h[5]+J[3]+J[6]+J[11],
		h[12]+h[7]+h[2]+h[9]+h[16]+h[11]+J[3]+J[6]+J[8]+J[13],
		h[12]+h[7]+h[2]+h[9]+h[16]+h[17]+J[3]+J[4]+J[6]+J[14],
		h[12]+h[7]+h[2]+h[15]+h[4]+h[5]+J[2]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[2]+h[15]+h[4]+h[11]+J[2]+J[8]+J[10],
		h[12]+h[7]+h[2]+h[15]+h[4]+h[17]+J[2]+J[4]+J[10]+J[13],
		h[12]+h[7]+h[2]+h[15]+h[10]+h[5]+J[2]+J[7]+J[11],
		h[12]+h[7]+h[2]+h[15]+h[10]+h[11]+J[2]+J[7]+J[8]+J[14],
		h[12]+h[7]+h[2]+h[15]+h[10]+h[17]+J[2]+J[4]+J[7]+J[13],
		h[12]+h[7]+h[2]+h[15]+h[16]+h[5]+J[2]+J[3]+J[11]+J[12],
		h[12]+h[7]+h[2]+h[15]+h[16]+h[11]+J[2]+J[3]+J[8]+J[12],
		h[12]+h[7]+h[2]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[8]+h[3]+h[4]+h[5]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[8]+h[3]+h[4]+h[11]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[7]+h[8]+h[3]+h[4]+h[17]+J[4]+J[5]+J[12],
		h[12]+h[7]+h[8]+h[3]+h[10]+h[5]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[7]+h[8]+h[3]+h[10]+h[11]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[8]+h[3]+h[10]+h[17]+J[4]+J[5]+J[7]+J[10],
		h[12]+h[7]+h[8]+h[3]+h[16]+h[5]+J[3]+J[5]+J[13],
		h[12]+h[7]+h[8]+h[3]+h[16]+h[11]+J[3]+J[5]+J[8]+J[11],
		h[12]+h[7]+h[8]+h[3]+h[16]+h[17]+J[3]+J[4]+J[5]+J[14],
		h[12]+h[7]+h[8]+h[9]+h[4]+h[5]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[7]+h[8]+h[9]+h[4]+h[11]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[8]+h[9]+h[4]+h[17]+J[4]+J[5]+J[6]+J[9],
		h[12]+h[7]+h[8]+h[9]+h[10]+h[5]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[8]+h[9]+h[10]+h[11]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[8]+h[9]+h[10]+h[17]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[8]+h[9]+h[16]+h[5]+J[3]+J[5]+J[6]+J[9],
		h[12]+h[7]+h[8]+h[9]+h[16]+h[11]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[8]+h[9]+h[16]+h[17]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[7]+h[8]+h[15]+h[4]+h[5]+J[2]+J[5]+J[14],
		h[12]+h[7]+h[8]+h[15]+h[4]+h[11]+J[2]+J[5]+J[8]+J[11],
		h[12]+h[7]+h[8]+h[15]+h[4]+h[17]+J[2]+J[4]+J[5]+J[13],
		h[12]+h[7]+h[8]+h[15]+h[10]+h[5]+J[2]+J[5]+J[7]+J[10],
		h[12]+h[7]+h[8]+h[15]+h[10]+h[11]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[8]+h[15]+h[10]+h[17]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[7]+h[8]+h[15]+h[16]+h[5]+J[2]+J[3]+J[5]+J[12],
		h[12]+h[7]+h[8]+h[15]+h[16]+h[11]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[7]+h[8]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[14]+h[3]+h[4]+h[5]+J[1]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[14]+h[3]+h[4]+h[11]+J[1]+J[8]+J[12],
		h[12]+h[7]+h[14]+h[3]+h[4]+h[17]+J[1]+J[4]+J[11]+J[12],
		h[12]+h[7]+h[14]+h[3]+h[10]+h[5]+J[1]+J[7]+J[13],
		h[12]+h[7]+h[14]+h[3]+h[10]+h[11]+J[1]+J[7]+J[8]+J[14],
		h[12]+h[7]+h[14]+h[3]+h[10]+h[17]+J[1]+J[4]+J[7]+J[11],
		h[12]+h[7]+h[14]+h[3]+h[16]+h[5]+J[1]+J[3]+J[10]+J[13],
		h[12]+h[7]+h[14]+h[3]+h[16]+h[11]+J[1]+J[3]+J[8]+J[10],
		h[12]+h[7]+h[14]+h[3]+h[16]+h[17]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[14]+h[9]+h[4]+h[5]+J[1]+J[6]+J[14],
		h[12]+h[7]+h[14]+h[9]+h[4]+h[11]+J[1]+J[6]+J[8]+J[13],
		h[12]+h[7]+h[14]+h[9]+h[4]+h[17]+J[1]+J[4]+J[6]+J[11],
		h[12]+h[7]+h[14]+h[9]+h[10]+h[5]+J[1]+J[6]+J[7]+J[12],
		h[12]+h[7]+h[14]+h[9]+h[10]+h[11]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[7]+h[14]+h[9]+h[10]+h[17]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[7]+h[14]+h[9]+h[16]+h[5]+J[1]+J[3]+J[6]+J[10],
		h[12]+h[7]+h[14]+h[9]+h[16]+h[11]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[7]+h[14]+h[9]+h[16]+h[17]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[7]+h[14]+h[15]+h[4]+h[5]+J[1]+J[2]+J[9]+J[14],
		h[12]+h[7]+h[14]+h[15]+h[4]+h[11]+J[1]+J[2]+J[8]+J[9],
		h[12]+h[7]+h[14]+h[15]+h[4]+h[17]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[14]+h[15]+h[10]+h[5]+J[1]+J[2]+J[7]+J[9],
		h[12]+h[7]+h[14]+h[15]+h[10]+h[11]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[7]+h[14]+h[15]+h[10]+h[17]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[7]+h[14]+h[15]+h[16]+h[5]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[14]+h[15]+h[16]+h[11]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[7]+h[14]+h[15]+h[16]+h[17]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[2]+h[3]+h[4]+h[5]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[2]+h[3]+h[4]+h[11]+J[0]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[2]+h[3]+h[4]+h[17]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[2]+h[3]+h[10]+h[5]+J[0]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[2]+h[3]+h[10]+h[11]+J[0]+J[9]+J[14],
		h[12]+h[13]+h[2]+h[3]+h[10]+h[17]+J[0]+J[4]+J[8]+J[9],
		h[12]+h[13]+h[2]+h[3]+h[16]+h[5]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[2]+h[3]+h[16]+h[11]+J[0]+J[3]+J[7]+J[9],
		h[12]+h[13]+h[2]+h[3]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[13]+h[2]+h[9]+h[4]+h[5]+J[0]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[2]+h[9]+h[4]+h[11]+J[0]+J[10]+J[13],
		h[12]+h[13]+h[2]+h[9]+h[4]+h[17]+J[0]+J[4]+J[8]+J[10],
		h[12]+h[13]+h[2]+h[9]+h[10]+h[5]+J[0]+J[11]+J[12],
		h[12]+h[13]+h[2]+h[9]+h[10]+h[11]+J[0]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[2]+h[9]+h[10]+h[17]+J[0]+J[4]+J[8]+J[12],
		h[12]+h[13]+h[2]+h[9]+h[16]+h[5]+J[0]+J[3]+J[7]+J[11],
		h[12]+h[13]+h[2]+h[9]+h[16]+h[11]+J[0]+J[3]+J[7]+J[13],
		h[12]+h[13]+h[2]+h[9]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[12]+h[13]+h[2]+h[15]+h[4]+h[5]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[2]+h[15]+h[4]+h[11]+J[0]+J[2]+J[6]+J[10],
		h[12]+h[13]+h[2]+h[15]+h[4]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[13]+h[2]+h[15]+h[10]+h[5]+J[0]+J[2]+J[6]+J[11],
		h[12]+h[13]+h[2]+h[15]+h[10]+h[11]+J[0]+J[2]+J[6]+J[14],
		h[12]+h[13]+h[2]+h[15]+h[10]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[12]+h[13]+h[2]+h[15]+h[16]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[13]+h[2]+h[15]+h[16]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[12]+h[13]+h[2]+h[15]+h[16]+h[17]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[8]+h[3]+h[4]+h[5]+J[0]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[8]+h[3]+h[4]+h[11]+J[0]+J[11]+J[12],
		h[12]+h[13]+h[8]+h[3]+h[4]+h[17]+J[0]+J[4]+J[8]+J[12],
		h[12]+h[13]+h[8]+h[3]+h[10]+h[5]+J[0]+J[10]+J[13],
		h[12]+h[13]+h[8]+h[3]+h[10]+h[11]+J[0]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[8]+h[3]+h[10]+h[17]+J[0]+J[4]+J[8]+J[10],
		h[12]+h[13]+h[8]+h[3]+h[16]+h[5]+J[0]+J[3]+J[7]+J[13],
		h[12]+h[13]+h[8]+h[3]+h[16]+h[11]+J[0]+J[3]+J[7]+J[11],
		h[12]+h[13]+h[8]+h[3]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14],
		h[12]+h[13]+h[8]+h[9]+h[4]+h[5]+J[0]+J[9]+J[14],
		h[12]+h[13]+h[8]+h[9]+h[4]+h[11]+J[0]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[8]+h[9]+h[4]+h[17]+J[0]+J[4]+J[8]+J[9],
		h[12]+h[13]+h[8]+h[9]+h[10]+h[5]+J[0]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[8]+h[9]+h[10]+h[11]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[8]+h[9]+h[10]+h[17]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[8]+h[9]+h[16]+h[5]+J[0]+J[3]+J[7]+J[9],
		h[12]+h[13]+h[8]+h[9]+h[16]+h[11]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[8]+h[9]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14],
		h[12]+h[13]+h[8]+h[15]+h[4]+h[5]+J[0]+J[2]+J[6]+J[14],
		h[12]+h[13]+h[8]+h[15]+h[4]+h[11]+J[0]+J[2]+J[6]+J[11],
		h[12]+h[13]+h[8]+h[15]+h[4]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13],
		h[12]+h[13]+h[8]+h[15]+h[10]+h[5]+J[0]+J[2]+J[6]+J[10],
		h[12]+h[13]+h[8]+h[15]+h[10]+h[11]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[8]+h[15]+h[10]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13],
		h[12]+h[13]+h[8]+h[15]+h[16]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12],
		h[12]+h[13]+h[8]+h[15]+h[16]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12],
		h[12]+h[13]+h[8]+h[15]+h[16]+h[17]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[14]+h[3]+h[4]+h[5]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[14]+h[3]+h[4]+h[11]+J[0]+J[1]+J[5]+J[12],
		h[12]+h[13]+h[14]+h[3]+h[4]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[13]+h[14]+h[3]+h[10]+h[5]+J[0]+J[1]+J[5]+J[13],
		h[12]+h[13]+h[14]+h[3]+h[10]+h[11]+J[0]+J[1]+J[5]+J[14],
		h[12]+h[13]+h[14]+h[3]+h[10]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[12]+h[13]+h[14]+h[3]+h[16]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[13]+h[14]+h[3]+h[16]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[12]+h[13]+h[14]+h[3]+h[16]+h[17]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[14]+h[9]+h[4]+h[5]+J[0]+J[1]+J[5]+J[14],
		h[12]+h[13]+h[14]+h[9]+h[4]+h[11]+J[0]+J[1]+J[5]+J[13],
		h[12]+h[13]+h[14]+h[9]+h[4]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11],
		h[12]+h[13]+h[14]+h[9]+h[10]+h[5]+J[0]+J[1]+J[5]+J[12],
		h[12]+h[13]+h[14]+h[9]+h[10]+h[11]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14],
		h[12]+h[13]+h[14]+h[9]+h[10]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12],
		h[12]+h[13]+h[14]+h[9]+h[16]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10],
		h[12]+h[13]+h[14]+h[9]+h[16]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13],
		h[12]+h[13]+h[14]+h[9]+h[16]+h[17]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14],
		h[12]+h[13]+h[14]+h[15]+h[4]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[13]+h[14]+h[15]+h[4]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[12]+h[13]+h[14]+h[15]+h[4]+h[17]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[14]+h[15]+h[10]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9],
		h[12]+h[13]+h[14]+h[15]+h[10]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14],
		h[12]+h[13]+h[14]+h[15]+h[10]+h[17]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13],
		h[12]+h[13]+h[14]+h[15]+h[16]+h[5]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[14]+h[15]+h[16]+h[11]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12],
		h[12]+h[13]+h[14]+h[15]+h[16]+h[17]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14],
		]
	logZ = fast_logsumexp(energies)[0]
	p[0] = exp(h[0]+h[1]+h[2]+h[3]+h[4]+h[5]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[1] = exp(h[0]+h[1]+h[2]+h[3]+h[4]+h[11]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[2] = exp(h[0]+h[1]+h[2]+h[3]+h[4]+h[17]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[3] = exp(h[0]+h[1]+h[2]+h[3]+h[10]+h[5]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[4] = exp(h[0]+h[1]+h[2]+h[3]+h[10]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[5] = exp(h[0]+h[1]+h[2]+h[3]+h[10]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9] - logZ)
	p[6] = exp(h[0]+h[1]+h[2]+h[3]+h[16]+h[5]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[7] = exp(h[0]+h[1]+h[2]+h[3]+h[16]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9] - logZ)
	p[8] = exp(h[0]+h[1]+h[2]+h[3]+h[16]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[9] = exp(h[0]+h[1]+h[2]+h[9]+h[4]+h[5]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[10] = exp(h[0]+h[1]+h[2]+h[9]+h[4]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[11] = exp(h[0]+h[1]+h[2]+h[9]+h[4]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10] - logZ)
	p[12] = exp(h[0]+h[1]+h[2]+h[9]+h[10]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[13] = exp(h[0]+h[1]+h[2]+h[9]+h[10]+h[11]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[14] = exp(h[0]+h[1]+h[2]+h[9]+h[10]+h[17]+J[0]+J[1]+J[5]+J[12] - logZ)
	p[15] = exp(h[0]+h[1]+h[2]+h[9]+h[16]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11] - logZ)
	p[16] = exp(h[0]+h[1]+h[2]+h[9]+h[16]+h[11]+J[0]+J[1]+J[5]+J[13] - logZ)
	p[17] = exp(h[0]+h[1]+h[2]+h[9]+h[16]+h[17]+J[0]+J[1]+J[5]+J[14] - logZ)
	p[18] = exp(h[0]+h[1]+h[2]+h[15]+h[4]+h[5]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[19] = exp(h[0]+h[1]+h[2]+h[15]+h[4]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10] - logZ)
	p[20] = exp(h[0]+h[1]+h[2]+h[15]+h[4]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[21] = exp(h[0]+h[1]+h[2]+h[15]+h[10]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11] - logZ)
	p[22] = exp(h[0]+h[1]+h[2]+h[15]+h[10]+h[11]+J[0]+J[1]+J[5]+J[14] - logZ)
	p[23] = exp(h[0]+h[1]+h[2]+h[15]+h[10]+h[17]+J[0]+J[1]+J[5]+J[13] - logZ)
	p[24] = exp(h[0]+h[1]+h[2]+h[15]+h[16]+h[5]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[25] = exp(h[0]+h[1]+h[2]+h[15]+h[16]+h[11]+J[0]+J[1]+J[5]+J[12] - logZ)
	p[26] = exp(h[0]+h[1]+h[2]+h[15]+h[16]+h[17]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[27] = exp(h[0]+h[1]+h[8]+h[3]+h[4]+h[5]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[28] = exp(h[0]+h[1]+h[8]+h[3]+h[4]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[29] = exp(h[0]+h[1]+h[8]+h[3]+h[4]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12] - logZ)
	p[30] = exp(h[0]+h[1]+h[8]+h[3]+h[10]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[31] = exp(h[0]+h[1]+h[8]+h[3]+h[10]+h[11]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[32] = exp(h[0]+h[1]+h[8]+h[3]+h[10]+h[17]+J[0]+J[2]+J[6]+J[10] - logZ)
	p[33] = exp(h[0]+h[1]+h[8]+h[3]+h[16]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13] - logZ)
	p[34] = exp(h[0]+h[1]+h[8]+h[3]+h[16]+h[11]+J[0]+J[2]+J[6]+J[11] - logZ)
	p[35] = exp(h[0]+h[1]+h[8]+h[3]+h[16]+h[17]+J[0]+J[2]+J[6]+J[14] - logZ)
	p[36] = exp(h[0]+h[1]+h[8]+h[9]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[37] = exp(h[0]+h[1]+h[8]+h[9]+h[4]+h[11]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[38] = exp(h[0]+h[1]+h[8]+h[9]+h[4]+h[17]+J[0]+J[3]+J[7]+J[9] - logZ)
	p[39] = exp(h[0]+h[1]+h[8]+h[9]+h[10]+h[5]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[40] = exp(h[0]+h[1]+h[8]+h[9]+h[10]+h[11]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[41] = exp(h[0]+h[1]+h[8]+h[9]+h[10]+h[17]+J[0]+J[9]+J[10]+J[12] - logZ)
	p[42] = exp(h[0]+h[1]+h[8]+h[9]+h[16]+h[5]+J[0]+J[4]+J[8]+J[9] - logZ)
	p[43] = exp(h[0]+h[1]+h[8]+h[9]+h[16]+h[11]+J[0]+J[9]+J[11]+J[13] - logZ)
	p[44] = exp(h[0]+h[1]+h[8]+h[9]+h[16]+h[17]+J[0]+J[9]+J[14] - logZ)
	p[45] = exp(h[0]+h[1]+h[8]+h[15]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14] - logZ)
	p[46] = exp(h[0]+h[1]+h[8]+h[15]+h[4]+h[11]+J[0]+J[3]+J[7]+J[11] - logZ)
	p[47] = exp(h[0]+h[1]+h[8]+h[15]+h[4]+h[17]+J[0]+J[3]+J[7]+J[13] - logZ)
	p[48] = exp(h[0]+h[1]+h[8]+h[15]+h[10]+h[5]+J[0]+J[4]+J[8]+J[10] - logZ)
	p[49] = exp(h[0]+h[1]+h[8]+h[15]+h[10]+h[11]+J[0]+J[10]+J[11]+J[14] - logZ)
	p[50] = exp(h[0]+h[1]+h[8]+h[15]+h[10]+h[17]+J[0]+J[10]+J[13] - logZ)
	p[51] = exp(h[0]+h[1]+h[8]+h[15]+h[16]+h[5]+J[0]+J[4]+J[8]+J[12] - logZ)
	p[52] = exp(h[0]+h[1]+h[8]+h[15]+h[16]+h[11]+J[0]+J[11]+J[12] - logZ)
	p[53] = exp(h[0]+h[1]+h[8]+h[15]+h[16]+h[17]+J[0]+J[12]+J[13]+J[14] - logZ)
	p[54] = exp(h[0]+h[1]+h[14]+h[3]+h[4]+h[5]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[55] = exp(h[0]+h[1]+h[14]+h[3]+h[4]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12] - logZ)
	p[56] = exp(h[0]+h[1]+h[14]+h[3]+h[4]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[57] = exp(h[0]+h[1]+h[14]+h[3]+h[10]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13] - logZ)
	p[58] = exp(h[0]+h[1]+h[14]+h[3]+h[10]+h[11]+J[0]+J[2]+J[6]+J[14] - logZ)
	p[59] = exp(h[0]+h[1]+h[14]+h[3]+h[10]+h[17]+J[0]+J[2]+J[6]+J[11] - logZ)
	p[60] = exp(h[0]+h[1]+h[14]+h[3]+h[16]+h[5]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[61] = exp(h[0]+h[1]+h[14]+h[3]+h[16]+h[11]+J[0]+J[2]+J[6]+J[10] - logZ)
	p[62] = exp(h[0]+h[1]+h[14]+h[3]+h[16]+h[17]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[63] = exp(h[0]+h[1]+h[14]+h[9]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14] - logZ)
	p[64] = exp(h[0]+h[1]+h[14]+h[9]+h[4]+h[11]+J[0]+J[3]+J[7]+J[13] - logZ)
	p[65] = exp(h[0]+h[1]+h[14]+h[9]+h[4]+h[17]+J[0]+J[3]+J[7]+J[11] - logZ)
	p[66] = exp(h[0]+h[1]+h[14]+h[9]+h[10]+h[5]+J[0]+J[4]+J[8]+J[12] - logZ)
	p[67] = exp(h[0]+h[1]+h[14]+h[9]+h[10]+h[11]+J[0]+J[12]+J[13]+J[14] - logZ)
	p[68] = exp(h[0]+h[1]+h[14]+h[9]+h[10]+h[17]+J[0]+J[11]+J[12] - logZ)
	p[69] = exp(h[0]+h[1]+h[14]+h[9]+h[16]+h[5]+J[0]+J[4]+J[8]+J[10] - logZ)
	p[70] = exp(h[0]+h[1]+h[14]+h[9]+h[16]+h[11]+J[0]+J[10]+J[13] - logZ)
	p[71] = exp(h[0]+h[1]+h[14]+h[9]+h[16]+h[17]+J[0]+J[10]+J[11]+J[14] - logZ)
	p[72] = exp(h[0]+h[1]+h[14]+h[15]+h[4]+h[5]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[73] = exp(h[0]+h[1]+h[14]+h[15]+h[4]+h[11]+J[0]+J[3]+J[7]+J[9] - logZ)
	p[74] = exp(h[0]+h[1]+h[14]+h[15]+h[4]+h[17]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[75] = exp(h[0]+h[1]+h[14]+h[15]+h[10]+h[5]+J[0]+J[4]+J[8]+J[9] - logZ)
	p[76] = exp(h[0]+h[1]+h[14]+h[15]+h[10]+h[11]+J[0]+J[9]+J[14] - logZ)
	p[77] = exp(h[0]+h[1]+h[14]+h[15]+h[10]+h[17]+J[0]+J[9]+J[11]+J[13] - logZ)
	p[78] = exp(h[0]+h[1]+h[14]+h[15]+h[16]+h[5]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[79] = exp(h[0]+h[1]+h[14]+h[15]+h[16]+h[11]+J[0]+J[9]+J[10]+J[12] - logZ)
	p[80] = exp(h[0]+h[1]+h[14]+h[15]+h[16]+h[17]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[81] = exp(h[0]+h[7]+h[2]+h[3]+h[4]+h[5]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[82] = exp(h[0]+h[7]+h[2]+h[3]+h[4]+h[11]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[83] = exp(h[0]+h[7]+h[2]+h[3]+h[4]+h[17]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12] - logZ)
	p[84] = exp(h[0]+h[7]+h[2]+h[3]+h[10]+h[5]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[85] = exp(h[0]+h[7]+h[2]+h[3]+h[10]+h[11]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[86] = exp(h[0]+h[7]+h[2]+h[3]+h[10]+h[17]+J[1]+J[2]+J[7]+J[9] - logZ)
	p[87] = exp(h[0]+h[7]+h[2]+h[3]+h[16]+h[5]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13] - logZ)
	p[88] = exp(h[0]+h[7]+h[2]+h[3]+h[16]+h[11]+J[1]+J[2]+J[8]+J[9] - logZ)
	p[89] = exp(h[0]+h[7]+h[2]+h[3]+h[16]+h[17]+J[1]+J[2]+J[9]+J[14] - logZ)
	p[90] = exp(h[0]+h[7]+h[2]+h[9]+h[4]+h[5]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[91] = exp(h[0]+h[7]+h[2]+h[9]+h[4]+h[11]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[92] = exp(h[0]+h[7]+h[2]+h[9]+h[4]+h[17]+J[1]+J[3]+J[6]+J[10] - logZ)
	p[93] = exp(h[0]+h[7]+h[2]+h[9]+h[10]+h[5]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[94] = exp(h[0]+h[7]+h[2]+h[9]+h[10]+h[11]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[95] = exp(h[0]+h[7]+h[2]+h[9]+h[10]+h[17]+J[1]+J[6]+J[7]+J[12] - logZ)
	p[96] = exp(h[0]+h[7]+h[2]+h[9]+h[16]+h[5]+J[1]+J[4]+J[6]+J[11] - logZ)
	p[97] = exp(h[0]+h[7]+h[2]+h[9]+h[16]+h[11]+J[1]+J[6]+J[8]+J[13] - logZ)
	p[98] = exp(h[0]+h[7]+h[2]+h[9]+h[16]+h[17]+J[1]+J[6]+J[14] - logZ)
	p[99] = exp(h[0]+h[7]+h[2]+h[15]+h[4]+h[5]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14] - logZ)
	p[100] = exp(h[0]+h[7]+h[2]+h[15]+h[4]+h[11]+J[1]+J[3]+J[8]+J[10] - logZ)
	p[101] = exp(h[0]+h[7]+h[2]+h[15]+h[4]+h[17]+J[1]+J[3]+J[10]+J[13] - logZ)
	p[102] = exp(h[0]+h[7]+h[2]+h[15]+h[10]+h[5]+J[1]+J[4]+J[7]+J[11] - logZ)
	p[103] = exp(h[0]+h[7]+h[2]+h[15]+h[10]+h[11]+J[1]+J[7]+J[8]+J[14] - logZ)
	p[104] = exp(h[0]+h[7]+h[2]+h[15]+h[10]+h[17]+J[1]+J[7]+J[13] - logZ)
	p[105] = exp(h[0]+h[7]+h[2]+h[15]+h[16]+h[5]+J[1]+J[4]+J[11]+J[12] - logZ)
	p[106] = exp(h[0]+h[7]+h[2]+h[15]+h[16]+h[11]+J[1]+J[8]+J[12] - logZ)
	p[107] = exp(h[0]+h[7]+h[2]+h[15]+h[16]+h[17]+J[1]+J[12]+J[13]+J[14] - logZ)
	p[108] = exp(h[0]+h[7]+h[8]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[109] = exp(h[0]+h[7]+h[8]+h[3]+h[4]+h[11]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[110] = exp(h[0]+h[7]+h[8]+h[3]+h[4]+h[17]+J[2]+J[3]+J[5]+J[12] - logZ)
	p[111] = exp(h[0]+h[7]+h[8]+h[3]+h[10]+h[5]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[112] = exp(h[0]+h[7]+h[8]+h[3]+h[10]+h[11]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[113] = exp(h[0]+h[7]+h[8]+h[3]+h[10]+h[17]+J[2]+J[5]+J[7]+J[10] - logZ)
	p[114] = exp(h[0]+h[7]+h[8]+h[3]+h[16]+h[5]+J[2]+J[4]+J[5]+J[13] - logZ)
	p[115] = exp(h[0]+h[7]+h[8]+h[3]+h[16]+h[11]+J[2]+J[5]+J[8]+J[11] - logZ)
	p[116] = exp(h[0]+h[7]+h[8]+h[3]+h[16]+h[17]+J[2]+J[5]+J[14] - logZ)
	p[117] = exp(h[0]+h[7]+h[8]+h[9]+h[4]+h[5]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[118] = exp(h[0]+h[7]+h[8]+h[9]+h[4]+h[11]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[119] = exp(h[0]+h[7]+h[8]+h[9]+h[4]+h[17]+J[3]+J[5]+J[6]+J[9] - logZ)
	p[120] = exp(h[0]+h[7]+h[8]+h[9]+h[10]+h[5]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[121] = exp(h[0]+h[7]+h[8]+h[9]+h[10]+h[11]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[122] = exp(h[0]+h[7]+h[8]+h[9]+h[10]+h[17]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[123] = exp(h[0]+h[7]+h[8]+h[9]+h[16]+h[5]+J[4]+J[5]+J[6]+J[9] - logZ)
	p[124] = exp(h[0]+h[7]+h[8]+h[9]+h[16]+h[11]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[125] = exp(h[0]+h[7]+h[8]+h[9]+h[16]+h[17]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[126] = exp(h[0]+h[7]+h[8]+h[15]+h[4]+h[5]+J[3]+J[4]+J[5]+J[14] - logZ)
	p[127] = exp(h[0]+h[7]+h[8]+h[15]+h[4]+h[11]+J[3]+J[5]+J[8]+J[11] - logZ)
	p[128] = exp(h[0]+h[7]+h[8]+h[15]+h[4]+h[17]+J[3]+J[5]+J[13] - logZ)
	p[129] = exp(h[0]+h[7]+h[8]+h[15]+h[10]+h[5]+J[4]+J[5]+J[7]+J[10] - logZ)
	p[130] = exp(h[0]+h[7]+h[8]+h[15]+h[10]+h[11]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[131] = exp(h[0]+h[7]+h[8]+h[15]+h[10]+h[17]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[132] = exp(h[0]+h[7]+h[8]+h[15]+h[16]+h[5]+J[4]+J[5]+J[12] - logZ)
	p[133] = exp(h[0]+h[7]+h[8]+h[15]+h[16]+h[11]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[134] = exp(h[0]+h[7]+h[8]+h[15]+h[16]+h[17]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[135] = exp(h[0]+h[7]+h[14]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14] - logZ)
	p[136] = exp(h[0]+h[7]+h[14]+h[3]+h[4]+h[11]+J[2]+J[3]+J[8]+J[12] - logZ)
	p[137] = exp(h[0]+h[7]+h[14]+h[3]+h[4]+h[17]+J[2]+J[3]+J[11]+J[12] - logZ)
	p[138] = exp(h[0]+h[7]+h[14]+h[3]+h[10]+h[5]+J[2]+J[4]+J[7]+J[13] - logZ)
	p[139] = exp(h[0]+h[7]+h[14]+h[3]+h[10]+h[11]+J[2]+J[7]+J[8]+J[14] - logZ)
	p[140] = exp(h[0]+h[7]+h[14]+h[3]+h[10]+h[17]+J[2]+J[7]+J[11] - logZ)
	p[141] = exp(h[0]+h[7]+h[14]+h[3]+h[16]+h[5]+J[2]+J[4]+J[10]+J[13] - logZ)
	p[142] = exp(h[0]+h[7]+h[14]+h[3]+h[16]+h[11]+J[2]+J[8]+J[10] - logZ)
	p[143] = exp(h[0]+h[7]+h[14]+h[3]+h[16]+h[17]+J[2]+J[10]+J[11]+J[14] - logZ)
	p[144] = exp(h[0]+h[7]+h[14]+h[9]+h[4]+h[5]+J[3]+J[4]+J[6]+J[14] - logZ)
	p[145] = exp(h[0]+h[7]+h[14]+h[9]+h[4]+h[11]+J[3]+J[6]+J[8]+J[13] - logZ)
	p[146] = exp(h[0]+h[7]+h[14]+h[9]+h[4]+h[17]+J[3]+J[6]+J[11] - logZ)
	p[147] = exp(h[0]+h[7]+h[14]+h[9]+h[10]+h[5]+J[4]+J[6]+J[7]+J[12] - logZ)
	p[148] = exp(h[0]+h[7]+h[14]+h[9]+h[10]+h[11]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[149] = exp(h[0]+h[7]+h[14]+h[9]+h[10]+h[17]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[150] = exp(h[0]+h[7]+h[14]+h[9]+h[16]+h[5]+J[4]+J[6]+J[10] - logZ)
	p[151] = exp(h[0]+h[7]+h[14]+h[9]+h[16]+h[11]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[152] = exp(h[0]+h[7]+h[14]+h[9]+h[16]+h[17]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[153] = exp(h[0]+h[7]+h[14]+h[15]+h[4]+h[5]+J[3]+J[4]+J[9]+J[14] - logZ)
	p[154] = exp(h[0]+h[7]+h[14]+h[15]+h[4]+h[11]+J[3]+J[8]+J[9] - logZ)
	p[155] = exp(h[0]+h[7]+h[14]+h[15]+h[4]+h[17]+J[3]+J[9]+J[11]+J[13] - logZ)
	p[156] = exp(h[0]+h[7]+h[14]+h[15]+h[10]+h[5]+J[4]+J[7]+J[9] - logZ)
	p[157] = exp(h[0]+h[7]+h[14]+h[15]+h[10]+h[11]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[158] = exp(h[0]+h[7]+h[14]+h[15]+h[10]+h[17]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[159] = exp(h[0]+h[7]+h[14]+h[15]+h[16]+h[5]+J[4]+J[9]+J[10]+J[12] - logZ)
	p[160] = exp(h[0]+h[7]+h[14]+h[15]+h[16]+h[11]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[161] = exp(h[0]+h[7]+h[14]+h[15]+h[16]+h[17]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[162] = exp(h[0]+h[13]+h[2]+h[3]+h[4]+h[5]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[163] = exp(h[0]+h[13]+h[2]+h[3]+h[4]+h[11]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12] - logZ)
	p[164] = exp(h[0]+h[13]+h[2]+h[3]+h[4]+h[17]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[165] = exp(h[0]+h[13]+h[2]+h[3]+h[10]+h[5]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13] - logZ)
	p[166] = exp(h[0]+h[13]+h[2]+h[3]+h[10]+h[11]+J[1]+J[2]+J[9]+J[14] - logZ)
	p[167] = exp(h[0]+h[13]+h[2]+h[3]+h[10]+h[17]+J[1]+J[2]+J[8]+J[9] - logZ)
	p[168] = exp(h[0]+h[13]+h[2]+h[3]+h[16]+h[5]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[169] = exp(h[0]+h[13]+h[2]+h[3]+h[16]+h[11]+J[1]+J[2]+J[7]+J[9] - logZ)
	p[170] = exp(h[0]+h[13]+h[2]+h[3]+h[16]+h[17]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[171] = exp(h[0]+h[13]+h[2]+h[9]+h[4]+h[5]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14] - logZ)
	p[172] = exp(h[0]+h[13]+h[2]+h[9]+h[4]+h[11]+J[1]+J[3]+J[10]+J[13] - logZ)
	p[173] = exp(h[0]+h[13]+h[2]+h[9]+h[4]+h[17]+J[1]+J[3]+J[8]+J[10] - logZ)
	p[174] = exp(h[0]+h[13]+h[2]+h[9]+h[10]+h[5]+J[1]+J[4]+J[11]+J[12] - logZ)
	p[175] = exp(h[0]+h[13]+h[2]+h[9]+h[10]+h[11]+J[1]+J[12]+J[13]+J[14] - logZ)
	p[176] = exp(h[0]+h[13]+h[2]+h[9]+h[10]+h[17]+J[1]+J[8]+J[12] - logZ)
	p[177] = exp(h[0]+h[13]+h[2]+h[9]+h[16]+h[5]+J[1]+J[4]+J[7]+J[11] - logZ)
	p[178] = exp(h[0]+h[13]+h[2]+h[9]+h[16]+h[11]+J[1]+J[7]+J[13] - logZ)
	p[179] = exp(h[0]+h[13]+h[2]+h[9]+h[16]+h[17]+J[1]+J[7]+J[8]+J[14] - logZ)
	p[180] = exp(h[0]+h[13]+h[2]+h[15]+h[4]+h[5]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[181] = exp(h[0]+h[13]+h[2]+h[15]+h[4]+h[11]+J[1]+J[3]+J[6]+J[10] - logZ)
	p[182] = exp(h[0]+h[13]+h[2]+h[15]+h[4]+h[17]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[183] = exp(h[0]+h[13]+h[2]+h[15]+h[10]+h[5]+J[1]+J[4]+J[6]+J[11] - logZ)
	p[184] = exp(h[0]+h[13]+h[2]+h[15]+h[10]+h[11]+J[1]+J[6]+J[14] - logZ)
	p[185] = exp(h[0]+h[13]+h[2]+h[15]+h[10]+h[17]+J[1]+J[6]+J[8]+J[13] - logZ)
	p[186] = exp(h[0]+h[13]+h[2]+h[15]+h[16]+h[5]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[187] = exp(h[0]+h[13]+h[2]+h[15]+h[16]+h[11]+J[1]+J[6]+J[7]+J[12] - logZ)
	p[188] = exp(h[0]+h[13]+h[2]+h[15]+h[16]+h[17]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[189] = exp(h[0]+h[13]+h[8]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14] - logZ)
	p[190] = exp(h[0]+h[13]+h[8]+h[3]+h[4]+h[11]+J[2]+J[3]+J[11]+J[12] - logZ)
	p[191] = exp(h[0]+h[13]+h[8]+h[3]+h[4]+h[17]+J[2]+J[3]+J[8]+J[12] - logZ)
	p[192] = exp(h[0]+h[13]+h[8]+h[3]+h[10]+h[5]+J[2]+J[4]+J[10]+J[13] - logZ)
	p[193] = exp(h[0]+h[13]+h[8]+h[3]+h[10]+h[11]+J[2]+J[10]+J[11]+J[14] - logZ)
	p[194] = exp(h[0]+h[13]+h[8]+h[3]+h[10]+h[17]+J[2]+J[8]+J[10] - logZ)
	p[195] = exp(h[0]+h[13]+h[8]+h[3]+h[16]+h[5]+J[2]+J[4]+J[7]+J[13] - logZ)
	p[196] = exp(h[0]+h[13]+h[8]+h[3]+h[16]+h[11]+J[2]+J[7]+J[11] - logZ)
	p[197] = exp(h[0]+h[13]+h[8]+h[3]+h[16]+h[17]+J[2]+J[7]+J[8]+J[14] - logZ)
	p[198] = exp(h[0]+h[13]+h[8]+h[9]+h[4]+h[5]+J[3]+J[4]+J[9]+J[14] - logZ)
	p[199] = exp(h[0]+h[13]+h[8]+h[9]+h[4]+h[11]+J[3]+J[9]+J[11]+J[13] - logZ)
	p[200] = exp(h[0]+h[13]+h[8]+h[9]+h[4]+h[17]+J[3]+J[8]+J[9] - logZ)
	p[201] = exp(h[0]+h[13]+h[8]+h[9]+h[10]+h[5]+J[4]+J[9]+J[10]+J[12] - logZ)
	p[202] = exp(h[0]+h[13]+h[8]+h[9]+h[10]+h[11]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[203] = exp(h[0]+h[13]+h[8]+h[9]+h[10]+h[17]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[204] = exp(h[0]+h[13]+h[8]+h[9]+h[16]+h[5]+J[4]+J[7]+J[9] - logZ)
	p[205] = exp(h[0]+h[13]+h[8]+h[9]+h[16]+h[11]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[206] = exp(h[0]+h[13]+h[8]+h[9]+h[16]+h[17]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[207] = exp(h[0]+h[13]+h[8]+h[15]+h[4]+h[5]+J[3]+J[4]+J[6]+J[14] - logZ)
	p[208] = exp(h[0]+h[13]+h[8]+h[15]+h[4]+h[11]+J[3]+J[6]+J[11] - logZ)
	p[209] = exp(h[0]+h[13]+h[8]+h[15]+h[4]+h[17]+J[3]+J[6]+J[8]+J[13] - logZ)
	p[210] = exp(h[0]+h[13]+h[8]+h[15]+h[10]+h[5]+J[4]+J[6]+J[10] - logZ)
	p[211] = exp(h[0]+h[13]+h[8]+h[15]+h[10]+h[11]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[212] = exp(h[0]+h[13]+h[8]+h[15]+h[10]+h[17]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[213] = exp(h[0]+h[13]+h[8]+h[15]+h[16]+h[5]+J[4]+J[6]+J[7]+J[12] - logZ)
	p[214] = exp(h[0]+h[13]+h[8]+h[15]+h[16]+h[11]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[215] = exp(h[0]+h[13]+h[8]+h[15]+h[16]+h[17]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[216] = exp(h[0]+h[13]+h[14]+h[3]+h[4]+h[5]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[217] = exp(h[0]+h[13]+h[14]+h[3]+h[4]+h[11]+J[2]+J[3]+J[5]+J[12] - logZ)
	p[218] = exp(h[0]+h[13]+h[14]+h[3]+h[4]+h[17]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[219] = exp(h[0]+h[13]+h[14]+h[3]+h[10]+h[5]+J[2]+J[4]+J[5]+J[13] - logZ)
	p[220] = exp(h[0]+h[13]+h[14]+h[3]+h[10]+h[11]+J[2]+J[5]+J[14] - logZ)
	p[221] = exp(h[0]+h[13]+h[14]+h[3]+h[10]+h[17]+J[2]+J[5]+J[8]+J[11] - logZ)
	p[222] = exp(h[0]+h[13]+h[14]+h[3]+h[16]+h[5]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[223] = exp(h[0]+h[13]+h[14]+h[3]+h[16]+h[11]+J[2]+J[5]+J[7]+J[10] - logZ)
	p[224] = exp(h[0]+h[13]+h[14]+h[3]+h[16]+h[17]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[225] = exp(h[0]+h[13]+h[14]+h[9]+h[4]+h[5]+J[3]+J[4]+J[5]+J[14] - logZ)
	p[226] = exp(h[0]+h[13]+h[14]+h[9]+h[4]+h[11]+J[3]+J[5]+J[13] - logZ)
	p[227] = exp(h[0]+h[13]+h[14]+h[9]+h[4]+h[17]+J[3]+J[5]+J[8]+J[11] - logZ)
	p[228] = exp(h[0]+h[13]+h[14]+h[9]+h[10]+h[5]+J[4]+J[5]+J[12] - logZ)
	p[229] = exp(h[0]+h[13]+h[14]+h[9]+h[10]+h[11]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[230] = exp(h[0]+h[13]+h[14]+h[9]+h[10]+h[17]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[231] = exp(h[0]+h[13]+h[14]+h[9]+h[16]+h[5]+J[4]+J[5]+J[7]+J[10] - logZ)
	p[232] = exp(h[0]+h[13]+h[14]+h[9]+h[16]+h[11]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[233] = exp(h[0]+h[13]+h[14]+h[9]+h[16]+h[17]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[234] = exp(h[0]+h[13]+h[14]+h[15]+h[4]+h[5]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[235] = exp(h[0]+h[13]+h[14]+h[15]+h[4]+h[11]+J[3]+J[5]+J[6]+J[9] - logZ)
	p[236] = exp(h[0]+h[13]+h[14]+h[15]+h[4]+h[17]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[237] = exp(h[0]+h[13]+h[14]+h[15]+h[10]+h[5]+J[4]+J[5]+J[6]+J[9] - logZ)
	p[238] = exp(h[0]+h[13]+h[14]+h[15]+h[10]+h[11]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[239] = exp(h[0]+h[13]+h[14]+h[15]+h[10]+h[17]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[240] = exp(h[0]+h[13]+h[14]+h[15]+h[16]+h[5]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[241] = exp(h[0]+h[13]+h[14]+h[15]+h[16]+h[11]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[242] = exp(h[0]+h[13]+h[14]+h[15]+h[16]+h[17]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[243] = exp(h[6]+h[1]+h[2]+h[3]+h[4]+h[5]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[244] = exp(h[6]+h[1]+h[2]+h[3]+h[4]+h[11]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[245] = exp(h[6]+h[1]+h[2]+h[3]+h[4]+h[17]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[246] = exp(h[6]+h[1]+h[2]+h[3]+h[10]+h[5]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[247] = exp(h[6]+h[1]+h[2]+h[3]+h[10]+h[11]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[248] = exp(h[6]+h[1]+h[2]+h[3]+h[10]+h[17]+J[3]+J[5]+J[6]+J[9] - logZ)
	p[249] = exp(h[6]+h[1]+h[2]+h[3]+h[16]+h[5]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[250] = exp(h[6]+h[1]+h[2]+h[3]+h[16]+h[11]+J[4]+J[5]+J[6]+J[9] - logZ)
	p[251] = exp(h[6]+h[1]+h[2]+h[3]+h[16]+h[17]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[252] = exp(h[6]+h[1]+h[2]+h[9]+h[4]+h[5]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[253] = exp(h[6]+h[1]+h[2]+h[9]+h[4]+h[11]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[254] = exp(h[6]+h[1]+h[2]+h[9]+h[4]+h[17]+J[2]+J[5]+J[7]+J[10] - logZ)
	p[255] = exp(h[6]+h[1]+h[2]+h[9]+h[10]+h[5]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[256] = exp(h[6]+h[1]+h[2]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[257] = exp(h[6]+h[1]+h[2]+h[9]+h[10]+h[17]+J[2]+J[3]+J[5]+J[12] - logZ)
	p[258] = exp(h[6]+h[1]+h[2]+h[9]+h[16]+h[5]+J[2]+J[5]+J[8]+J[11] - logZ)
	p[259] = exp(h[6]+h[1]+h[2]+h[9]+h[16]+h[11]+J[2]+J[4]+J[5]+J[13] - logZ)
	p[260] = exp(h[6]+h[1]+h[2]+h[9]+h[16]+h[17]+J[2]+J[5]+J[14] - logZ)
	p[261] = exp(h[6]+h[1]+h[2]+h[15]+h[4]+h[5]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[262] = exp(h[6]+h[1]+h[2]+h[15]+h[4]+h[11]+J[4]+J[5]+J[7]+J[10] - logZ)
	p[263] = exp(h[6]+h[1]+h[2]+h[15]+h[4]+h[17]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[264] = exp(h[6]+h[1]+h[2]+h[15]+h[10]+h[5]+J[3]+J[5]+J[8]+J[11] - logZ)
	p[265] = exp(h[6]+h[1]+h[2]+h[15]+h[10]+h[11]+J[3]+J[4]+J[5]+J[14] - logZ)
	p[266] = exp(h[6]+h[1]+h[2]+h[15]+h[10]+h[17]+J[3]+J[5]+J[13] - logZ)
	p[267] = exp(h[6]+h[1]+h[2]+h[15]+h[16]+h[5]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[268] = exp(h[6]+h[1]+h[2]+h[15]+h[16]+h[11]+J[4]+J[5]+J[12] - logZ)
	p[269] = exp(h[6]+h[1]+h[2]+h[15]+h[16]+h[17]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[270] = exp(h[6]+h[1]+h[8]+h[3]+h[4]+h[5]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[271] = exp(h[6]+h[1]+h[8]+h[3]+h[4]+h[11]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[272] = exp(h[6]+h[1]+h[8]+h[3]+h[4]+h[17]+J[1]+J[6]+J[7]+J[12] - logZ)
	p[273] = exp(h[6]+h[1]+h[8]+h[3]+h[10]+h[5]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[274] = exp(h[6]+h[1]+h[8]+h[3]+h[10]+h[11]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[275] = exp(h[6]+h[1]+h[8]+h[3]+h[10]+h[17]+J[1]+J[3]+J[6]+J[10] - logZ)
	p[276] = exp(h[6]+h[1]+h[8]+h[3]+h[16]+h[5]+J[1]+J[6]+J[8]+J[13] - logZ)
	p[277] = exp(h[6]+h[1]+h[8]+h[3]+h[16]+h[11]+J[1]+J[4]+J[6]+J[11] - logZ)
	p[278] = exp(h[6]+h[1]+h[8]+h[3]+h[16]+h[17]+J[1]+J[6]+J[14] - logZ)
	p[279] = exp(h[6]+h[1]+h[8]+h[9]+h[4]+h[5]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[280] = exp(h[6]+h[1]+h[8]+h[9]+h[4]+h[11]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[281] = exp(h[6]+h[1]+h[8]+h[9]+h[4]+h[17]+J[1]+J[2]+J[7]+J[9] - logZ)
	p[282] = exp(h[6]+h[1]+h[8]+h[9]+h[10]+h[5]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[283] = exp(h[6]+h[1]+h[8]+h[9]+h[10]+h[11]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[284] = exp(h[6]+h[1]+h[8]+h[9]+h[10]+h[17]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12] - logZ)
	p[285] = exp(h[6]+h[1]+h[8]+h[9]+h[16]+h[5]+J[1]+J[2]+J[8]+J[9] - logZ)
	p[286] = exp(h[6]+h[1]+h[8]+h[9]+h[16]+h[11]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13] - logZ)
	p[287] = exp(h[6]+h[1]+h[8]+h[9]+h[16]+h[17]+J[1]+J[2]+J[9]+J[14] - logZ)
	p[288] = exp(h[6]+h[1]+h[8]+h[15]+h[4]+h[5]+J[1]+J[7]+J[8]+J[14] - logZ)
	p[289] = exp(h[6]+h[1]+h[8]+h[15]+h[4]+h[11]+J[1]+J[4]+J[7]+J[11] - logZ)
	p[290] = exp(h[6]+h[1]+h[8]+h[15]+h[4]+h[17]+J[1]+J[7]+J[13] - logZ)
	p[291] = exp(h[6]+h[1]+h[8]+h[15]+h[10]+h[5]+J[1]+J[3]+J[8]+J[10] - logZ)
	p[292] = exp(h[6]+h[1]+h[8]+h[15]+h[10]+h[11]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14] - logZ)
	p[293] = exp(h[6]+h[1]+h[8]+h[15]+h[10]+h[17]+J[1]+J[3]+J[10]+J[13] - logZ)
	p[294] = exp(h[6]+h[1]+h[8]+h[15]+h[16]+h[5]+J[1]+J[8]+J[12] - logZ)
	p[295] = exp(h[6]+h[1]+h[8]+h[15]+h[16]+h[11]+J[1]+J[4]+J[11]+J[12] - logZ)
	p[296] = exp(h[6]+h[1]+h[8]+h[15]+h[16]+h[17]+J[1]+J[12]+J[13]+J[14] - logZ)
	p[297] = exp(h[6]+h[1]+h[14]+h[3]+h[4]+h[5]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[298] = exp(h[6]+h[1]+h[14]+h[3]+h[4]+h[11]+J[4]+J[6]+J[7]+J[12] - logZ)
	p[299] = exp(h[6]+h[1]+h[14]+h[3]+h[4]+h[17]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[300] = exp(h[6]+h[1]+h[14]+h[3]+h[10]+h[5]+J[3]+J[6]+J[8]+J[13] - logZ)
	p[301] = exp(h[6]+h[1]+h[14]+h[3]+h[10]+h[11]+J[3]+J[4]+J[6]+J[14] - logZ)
	p[302] = exp(h[6]+h[1]+h[14]+h[3]+h[10]+h[17]+J[3]+J[6]+J[11] - logZ)
	p[303] = exp(h[6]+h[1]+h[14]+h[3]+h[16]+h[5]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[304] = exp(h[6]+h[1]+h[14]+h[3]+h[16]+h[11]+J[4]+J[6]+J[10] - logZ)
	p[305] = exp(h[6]+h[1]+h[14]+h[3]+h[16]+h[17]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[306] = exp(h[6]+h[1]+h[14]+h[9]+h[4]+h[5]+J[2]+J[7]+J[8]+J[14] - logZ)
	p[307] = exp(h[6]+h[1]+h[14]+h[9]+h[4]+h[11]+J[2]+J[4]+J[7]+J[13] - logZ)
	p[308] = exp(h[6]+h[1]+h[14]+h[9]+h[4]+h[17]+J[2]+J[7]+J[11] - logZ)
	p[309] = exp(h[6]+h[1]+h[14]+h[9]+h[10]+h[5]+J[2]+J[3]+J[8]+J[12] - logZ)
	p[310] = exp(h[6]+h[1]+h[14]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14] - logZ)
	p[311] = exp(h[6]+h[1]+h[14]+h[9]+h[10]+h[17]+J[2]+J[3]+J[11]+J[12] - logZ)
	p[312] = exp(h[6]+h[1]+h[14]+h[9]+h[16]+h[5]+J[2]+J[8]+J[10] - logZ)
	p[313] = exp(h[6]+h[1]+h[14]+h[9]+h[16]+h[11]+J[2]+J[4]+J[10]+J[13] - logZ)
	p[314] = exp(h[6]+h[1]+h[14]+h[9]+h[16]+h[17]+J[2]+J[10]+J[11]+J[14] - logZ)
	p[315] = exp(h[6]+h[1]+h[14]+h[15]+h[4]+h[5]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[316] = exp(h[6]+h[1]+h[14]+h[15]+h[4]+h[11]+J[4]+J[7]+J[9] - logZ)
	p[317] = exp(h[6]+h[1]+h[14]+h[15]+h[4]+h[17]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[318] = exp(h[6]+h[1]+h[14]+h[15]+h[10]+h[5]+J[3]+J[8]+J[9] - logZ)
	p[319] = exp(h[6]+h[1]+h[14]+h[15]+h[10]+h[11]+J[3]+J[4]+J[9]+J[14] - logZ)
	p[320] = exp(h[6]+h[1]+h[14]+h[15]+h[10]+h[17]+J[3]+J[9]+J[11]+J[13] - logZ)
	p[321] = exp(h[6]+h[1]+h[14]+h[15]+h[16]+h[5]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[322] = exp(h[6]+h[1]+h[14]+h[15]+h[16]+h[11]+J[4]+J[9]+J[10]+J[12] - logZ)
	p[323] = exp(h[6]+h[1]+h[14]+h[15]+h[16]+h[17]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[324] = exp(h[6]+h[7]+h[2]+h[3]+h[4]+h[5]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[325] = exp(h[6]+h[7]+h[2]+h[3]+h[4]+h[11]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[326] = exp(h[6]+h[7]+h[2]+h[3]+h[4]+h[17]+J[0]+J[9]+J[10]+J[12] - logZ)
	p[327] = exp(h[6]+h[7]+h[2]+h[3]+h[10]+h[5]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[328] = exp(h[6]+h[7]+h[2]+h[3]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[329] = exp(h[6]+h[7]+h[2]+h[3]+h[10]+h[17]+J[0]+J[3]+J[7]+J[9] - logZ)
	p[330] = exp(h[6]+h[7]+h[2]+h[3]+h[16]+h[5]+J[0]+J[9]+J[11]+J[13] - logZ)
	p[331] = exp(h[6]+h[7]+h[2]+h[3]+h[16]+h[11]+J[0]+J[4]+J[8]+J[9] - logZ)
	p[332] = exp(h[6]+h[7]+h[2]+h[3]+h[16]+h[17]+J[0]+J[9]+J[14] - logZ)
	p[333] = exp(h[6]+h[7]+h[2]+h[9]+h[4]+h[5]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[334] = exp(h[6]+h[7]+h[2]+h[9]+h[4]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[335] = exp(h[6]+h[7]+h[2]+h[9]+h[4]+h[17]+J[0]+J[2]+J[6]+J[10] - logZ)
	p[336] = exp(h[6]+h[7]+h[2]+h[9]+h[10]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[337] = exp(h[6]+h[7]+h[2]+h[9]+h[10]+h[11]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[338] = exp(h[6]+h[7]+h[2]+h[9]+h[10]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12] - logZ)
	p[339] = exp(h[6]+h[7]+h[2]+h[9]+h[16]+h[5]+J[0]+J[2]+J[6]+J[11] - logZ)
	p[340] = exp(h[6]+h[7]+h[2]+h[9]+h[16]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13] - logZ)
	p[341] = exp(h[6]+h[7]+h[2]+h[9]+h[16]+h[17]+J[0]+J[2]+J[6]+J[14] - logZ)
	p[342] = exp(h[6]+h[7]+h[2]+h[15]+h[4]+h[5]+J[0]+J[10]+J[11]+J[14] - logZ)
	p[343] = exp(h[6]+h[7]+h[2]+h[15]+h[4]+h[11]+J[0]+J[4]+J[8]+J[10] - logZ)
	p[344] = exp(h[6]+h[7]+h[2]+h[15]+h[4]+h[17]+J[0]+J[10]+J[13] - logZ)
	p[345] = exp(h[6]+h[7]+h[2]+h[15]+h[10]+h[5]+J[0]+J[3]+J[7]+J[11] - logZ)
	p[346] = exp(h[6]+h[7]+h[2]+h[15]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14] - logZ)
	p[347] = exp(h[6]+h[7]+h[2]+h[15]+h[10]+h[17]+J[0]+J[3]+J[7]+J[13] - logZ)
	p[348] = exp(h[6]+h[7]+h[2]+h[15]+h[16]+h[5]+J[0]+J[11]+J[12] - logZ)
	p[349] = exp(h[6]+h[7]+h[2]+h[15]+h[16]+h[11]+J[0]+J[4]+J[8]+J[12] - logZ)
	p[350] = exp(h[6]+h[7]+h[2]+h[15]+h[16]+h[17]+J[0]+J[12]+J[13]+J[14] - logZ)
	p[351] = exp(h[6]+h[7]+h[8]+h[3]+h[4]+h[5]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[352] = exp(h[6]+h[7]+h[8]+h[3]+h[4]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[353] = exp(h[6]+h[7]+h[8]+h[3]+h[4]+h[17]+J[0]+J[1]+J[5]+J[12] - logZ)
	p[354] = exp(h[6]+h[7]+h[8]+h[3]+h[10]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[355] = exp(h[6]+h[7]+h[8]+h[3]+h[10]+h[11]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[356] = exp(h[6]+h[7]+h[8]+h[3]+h[10]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10] - logZ)
	p[357] = exp(h[6]+h[7]+h[8]+h[3]+h[16]+h[5]+J[0]+J[1]+J[5]+J[13] - logZ)
	p[358] = exp(h[6]+h[7]+h[8]+h[3]+h[16]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11] - logZ)
	p[359] = exp(h[6]+h[7]+h[8]+h[3]+h[16]+h[17]+J[0]+J[1]+J[5]+J[14] - logZ)
	p[360] = exp(h[6]+h[7]+h[8]+h[9]+h[4]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[361] = exp(h[6]+h[7]+h[8]+h[9]+h[4]+h[11]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[362] = exp(h[6]+h[7]+h[8]+h[9]+h[4]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9] - logZ)
	p[363] = exp(h[6]+h[7]+h[8]+h[9]+h[10]+h[5]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[364] = exp(h[6]+h[7]+h[8]+h[9]+h[10]+h[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[365] = exp(h[6]+h[7]+h[8]+h[9]+h[10]+h[17]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[366] = exp(h[6]+h[7]+h[8]+h[9]+h[16]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9] - logZ)
	p[367] = exp(h[6]+h[7]+h[8]+h[9]+h[16]+h[11]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[368] = exp(h[6]+h[7]+h[8]+h[9]+h[16]+h[17]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[369] = exp(h[6]+h[7]+h[8]+h[15]+h[4]+h[5]+J[0]+J[1]+J[5]+J[14] - logZ)
	p[370] = exp(h[6]+h[7]+h[8]+h[15]+h[4]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11] - logZ)
	p[371] = exp(h[6]+h[7]+h[8]+h[15]+h[4]+h[17]+J[0]+J[1]+J[5]+J[13] - logZ)
	p[372] = exp(h[6]+h[7]+h[8]+h[15]+h[10]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10] - logZ)
	p[373] = exp(h[6]+h[7]+h[8]+h[15]+h[10]+h[11]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[374] = exp(h[6]+h[7]+h[8]+h[15]+h[10]+h[17]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[375] = exp(h[6]+h[7]+h[8]+h[15]+h[16]+h[5]+J[0]+J[1]+J[5]+J[12] - logZ)
	p[376] = exp(h[6]+h[7]+h[8]+h[15]+h[16]+h[11]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[377] = exp(h[6]+h[7]+h[8]+h[15]+h[16]+h[17]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[378] = exp(h[6]+h[7]+h[14]+h[3]+h[4]+h[5]+J[0]+J[12]+J[13]+J[14] - logZ)
	p[379] = exp(h[6]+h[7]+h[14]+h[3]+h[4]+h[11]+J[0]+J[4]+J[8]+J[12] - logZ)
	p[380] = exp(h[6]+h[7]+h[14]+h[3]+h[4]+h[17]+J[0]+J[11]+J[12] - logZ)
	p[381] = exp(h[6]+h[7]+h[14]+h[3]+h[10]+h[5]+J[0]+J[3]+J[7]+J[13] - logZ)
	p[382] = exp(h[6]+h[7]+h[14]+h[3]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14] - logZ)
	p[383] = exp(h[6]+h[7]+h[14]+h[3]+h[10]+h[17]+J[0]+J[3]+J[7]+J[11] - logZ)
	p[384] = exp(h[6]+h[7]+h[14]+h[3]+h[16]+h[5]+J[0]+J[10]+J[13] - logZ)
	p[385] = exp(h[6]+h[7]+h[14]+h[3]+h[16]+h[11]+J[0]+J[4]+J[8]+J[10] - logZ)
	p[386] = exp(h[6]+h[7]+h[14]+h[3]+h[16]+h[17]+J[0]+J[10]+J[11]+J[14] - logZ)
	p[387] = exp(h[6]+h[7]+h[14]+h[9]+h[4]+h[5]+J[0]+J[2]+J[6]+J[14] - logZ)
	p[388] = exp(h[6]+h[7]+h[14]+h[9]+h[4]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13] - logZ)
	p[389] = exp(h[6]+h[7]+h[14]+h[9]+h[4]+h[17]+J[0]+J[2]+J[6]+J[11] - logZ)
	p[390] = exp(h[6]+h[7]+h[14]+h[9]+h[10]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12] - logZ)
	p[391] = exp(h[6]+h[7]+h[14]+h[9]+h[10]+h[11]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[392] = exp(h[6]+h[7]+h[14]+h[9]+h[10]+h[17]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[393] = exp(h[6]+h[7]+h[14]+h[9]+h[16]+h[5]+J[0]+J[2]+J[6]+J[10] - logZ)
	p[394] = exp(h[6]+h[7]+h[14]+h[9]+h[16]+h[11]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[395] = exp(h[6]+h[7]+h[14]+h[9]+h[16]+h[17]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[396] = exp(h[6]+h[7]+h[14]+h[15]+h[4]+h[5]+J[0]+J[9]+J[14] - logZ)
	p[397] = exp(h[6]+h[7]+h[14]+h[15]+h[4]+h[11]+J[0]+J[4]+J[8]+J[9] - logZ)
	p[398] = exp(h[6]+h[7]+h[14]+h[15]+h[4]+h[17]+J[0]+J[9]+J[11]+J[13] - logZ)
	p[399] = exp(h[6]+h[7]+h[14]+h[15]+h[10]+h[5]+J[0]+J[3]+J[7]+J[9] - logZ)
	p[400] = exp(h[6]+h[7]+h[14]+h[15]+h[10]+h[11]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[401] = exp(h[6]+h[7]+h[14]+h[15]+h[10]+h[17]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[402] = exp(h[6]+h[7]+h[14]+h[15]+h[16]+h[5]+J[0]+J[9]+J[10]+J[12] - logZ)
	p[403] = exp(h[6]+h[7]+h[14]+h[15]+h[16]+h[11]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[404] = exp(h[6]+h[7]+h[14]+h[15]+h[16]+h[17]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[405] = exp(h[6]+h[13]+h[2]+h[3]+h[4]+h[5]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[406] = exp(h[6]+h[13]+h[2]+h[3]+h[4]+h[11]+J[4]+J[9]+J[10]+J[12] - logZ)
	p[407] = exp(h[6]+h[13]+h[2]+h[3]+h[4]+h[17]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[408] = exp(h[6]+h[13]+h[2]+h[3]+h[10]+h[5]+J[3]+J[9]+J[11]+J[13] - logZ)
	p[409] = exp(h[6]+h[13]+h[2]+h[3]+h[10]+h[11]+J[3]+J[4]+J[9]+J[14] - logZ)
	p[410] = exp(h[6]+h[13]+h[2]+h[3]+h[10]+h[17]+J[3]+J[8]+J[9] - logZ)
	p[411] = exp(h[6]+h[13]+h[2]+h[3]+h[16]+h[5]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[412] = exp(h[6]+h[13]+h[2]+h[3]+h[16]+h[11]+J[4]+J[7]+J[9] - logZ)
	p[413] = exp(h[6]+h[13]+h[2]+h[3]+h[16]+h[17]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[414] = exp(h[6]+h[13]+h[2]+h[9]+h[4]+h[5]+J[2]+J[10]+J[11]+J[14] - logZ)
	p[415] = exp(h[6]+h[13]+h[2]+h[9]+h[4]+h[11]+J[2]+J[4]+J[10]+J[13] - logZ)
	p[416] = exp(h[6]+h[13]+h[2]+h[9]+h[4]+h[17]+J[2]+J[8]+J[10] - logZ)
	p[417] = exp(h[6]+h[13]+h[2]+h[9]+h[10]+h[5]+J[2]+J[3]+J[11]+J[12] - logZ)
	p[418] = exp(h[6]+h[13]+h[2]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14] - logZ)
	p[419] = exp(h[6]+h[13]+h[2]+h[9]+h[10]+h[17]+J[2]+J[3]+J[8]+J[12] - logZ)
	p[420] = exp(h[6]+h[13]+h[2]+h[9]+h[16]+h[5]+J[2]+J[7]+J[11] - logZ)
	p[421] = exp(h[6]+h[13]+h[2]+h[9]+h[16]+h[11]+J[2]+J[4]+J[7]+J[13] - logZ)
	p[422] = exp(h[6]+h[13]+h[2]+h[9]+h[16]+h[17]+J[2]+J[7]+J[8]+J[14] - logZ)
	p[423] = exp(h[6]+h[13]+h[2]+h[15]+h[4]+h[5]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[424] = exp(h[6]+h[13]+h[2]+h[15]+h[4]+h[11]+J[4]+J[6]+J[10] - logZ)
	p[425] = exp(h[6]+h[13]+h[2]+h[15]+h[4]+h[17]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[426] = exp(h[6]+h[13]+h[2]+h[15]+h[10]+h[5]+J[3]+J[6]+J[11] - logZ)
	p[427] = exp(h[6]+h[13]+h[2]+h[15]+h[10]+h[11]+J[3]+J[4]+J[6]+J[14] - logZ)
	p[428] = exp(h[6]+h[13]+h[2]+h[15]+h[10]+h[17]+J[3]+J[6]+J[8]+J[13] - logZ)
	p[429] = exp(h[6]+h[13]+h[2]+h[15]+h[16]+h[5]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[430] = exp(h[6]+h[13]+h[2]+h[15]+h[16]+h[11]+J[4]+J[6]+J[7]+J[12] - logZ)
	p[431] = exp(h[6]+h[13]+h[2]+h[15]+h[16]+h[17]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[432] = exp(h[6]+h[13]+h[8]+h[3]+h[4]+h[5]+J[1]+J[12]+J[13]+J[14] - logZ)
	p[433] = exp(h[6]+h[13]+h[8]+h[3]+h[4]+h[11]+J[1]+J[4]+J[11]+J[12] - logZ)
	p[434] = exp(h[6]+h[13]+h[8]+h[3]+h[4]+h[17]+J[1]+J[8]+J[12] - logZ)
	p[435] = exp(h[6]+h[13]+h[8]+h[3]+h[10]+h[5]+J[1]+J[3]+J[10]+J[13] - logZ)
	p[436] = exp(h[6]+h[13]+h[8]+h[3]+h[10]+h[11]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14] - logZ)
	p[437] = exp(h[6]+h[13]+h[8]+h[3]+h[10]+h[17]+J[1]+J[3]+J[8]+J[10] - logZ)
	p[438] = exp(h[6]+h[13]+h[8]+h[3]+h[16]+h[5]+J[1]+J[7]+J[13] - logZ)
	p[439] = exp(h[6]+h[13]+h[8]+h[3]+h[16]+h[11]+J[1]+J[4]+J[7]+J[11] - logZ)
	p[440] = exp(h[6]+h[13]+h[8]+h[3]+h[16]+h[17]+J[1]+J[7]+J[8]+J[14] - logZ)
	p[441] = exp(h[6]+h[13]+h[8]+h[9]+h[4]+h[5]+J[1]+J[2]+J[9]+J[14] - logZ)
	p[442] = exp(h[6]+h[13]+h[8]+h[9]+h[4]+h[11]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13] - logZ)
	p[443] = exp(h[6]+h[13]+h[8]+h[9]+h[4]+h[17]+J[1]+J[2]+J[8]+J[9] - logZ)
	p[444] = exp(h[6]+h[13]+h[8]+h[9]+h[10]+h[5]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12] - logZ)
	p[445] = exp(h[6]+h[13]+h[8]+h[9]+h[10]+h[11]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[446] = exp(h[6]+h[13]+h[8]+h[9]+h[10]+h[17]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[447] = exp(h[6]+h[13]+h[8]+h[9]+h[16]+h[5]+J[1]+J[2]+J[7]+J[9] - logZ)
	p[448] = exp(h[6]+h[13]+h[8]+h[9]+h[16]+h[11]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[449] = exp(h[6]+h[13]+h[8]+h[9]+h[16]+h[17]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[450] = exp(h[6]+h[13]+h[8]+h[15]+h[4]+h[5]+J[1]+J[6]+J[14] - logZ)
	p[451] = exp(h[6]+h[13]+h[8]+h[15]+h[4]+h[11]+J[1]+J[4]+J[6]+J[11] - logZ)
	p[452] = exp(h[6]+h[13]+h[8]+h[15]+h[4]+h[17]+J[1]+J[6]+J[8]+J[13] - logZ)
	p[453] = exp(h[6]+h[13]+h[8]+h[15]+h[10]+h[5]+J[1]+J[3]+J[6]+J[10] - logZ)
	p[454] = exp(h[6]+h[13]+h[8]+h[15]+h[10]+h[11]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[455] = exp(h[6]+h[13]+h[8]+h[15]+h[10]+h[17]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[456] = exp(h[6]+h[13]+h[8]+h[15]+h[16]+h[5]+J[1]+J[6]+J[7]+J[12] - logZ)
	p[457] = exp(h[6]+h[13]+h[8]+h[15]+h[16]+h[11]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[458] = exp(h[6]+h[13]+h[8]+h[15]+h[16]+h[17]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[459] = exp(h[6]+h[13]+h[14]+h[3]+h[4]+h[5]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[460] = exp(h[6]+h[13]+h[14]+h[3]+h[4]+h[11]+J[4]+J[5]+J[12] - logZ)
	p[461] = exp(h[6]+h[13]+h[14]+h[3]+h[4]+h[17]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[462] = exp(h[6]+h[13]+h[14]+h[3]+h[10]+h[5]+J[3]+J[5]+J[13] - logZ)
	p[463] = exp(h[6]+h[13]+h[14]+h[3]+h[10]+h[11]+J[3]+J[4]+J[5]+J[14] - logZ)
	p[464] = exp(h[6]+h[13]+h[14]+h[3]+h[10]+h[17]+J[3]+J[5]+J[8]+J[11] - logZ)
	p[465] = exp(h[6]+h[13]+h[14]+h[3]+h[16]+h[5]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[466] = exp(h[6]+h[13]+h[14]+h[3]+h[16]+h[11]+J[4]+J[5]+J[7]+J[10] - logZ)
	p[467] = exp(h[6]+h[13]+h[14]+h[3]+h[16]+h[17]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[468] = exp(h[6]+h[13]+h[14]+h[9]+h[4]+h[5]+J[2]+J[5]+J[14] - logZ)
	p[469] = exp(h[6]+h[13]+h[14]+h[9]+h[4]+h[11]+J[2]+J[4]+J[5]+J[13] - logZ)
	p[470] = exp(h[6]+h[13]+h[14]+h[9]+h[4]+h[17]+J[2]+J[5]+J[8]+J[11] - logZ)
	p[471] = exp(h[6]+h[13]+h[14]+h[9]+h[10]+h[5]+J[2]+J[3]+J[5]+J[12] - logZ)
	p[472] = exp(h[6]+h[13]+h[14]+h[9]+h[10]+h[11]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[473] = exp(h[6]+h[13]+h[14]+h[9]+h[10]+h[17]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[474] = exp(h[6]+h[13]+h[14]+h[9]+h[16]+h[5]+J[2]+J[5]+J[7]+J[10] - logZ)
	p[475] = exp(h[6]+h[13]+h[14]+h[9]+h[16]+h[11]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[476] = exp(h[6]+h[13]+h[14]+h[9]+h[16]+h[17]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[477] = exp(h[6]+h[13]+h[14]+h[15]+h[4]+h[5]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[478] = exp(h[6]+h[13]+h[14]+h[15]+h[4]+h[11]+J[4]+J[5]+J[6]+J[9] - logZ)
	p[479] = exp(h[6]+h[13]+h[14]+h[15]+h[4]+h[17]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[480] = exp(h[6]+h[13]+h[14]+h[15]+h[10]+h[5]+J[3]+J[5]+J[6]+J[9] - logZ)
	p[481] = exp(h[6]+h[13]+h[14]+h[15]+h[10]+h[11]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[482] = exp(h[6]+h[13]+h[14]+h[15]+h[10]+h[17]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[483] = exp(h[6]+h[13]+h[14]+h[15]+h[16]+h[5]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[484] = exp(h[6]+h[13]+h[14]+h[15]+h[16]+h[11]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[485] = exp(h[6]+h[13]+h[14]+h[15]+h[16]+h[17]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[486] = exp(h[12]+h[1]+h[2]+h[3]+h[4]+h[5]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[487] = exp(h[12]+h[1]+h[2]+h[3]+h[4]+h[11]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[488] = exp(h[12]+h[1]+h[2]+h[3]+h[4]+h[17]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[489] = exp(h[12]+h[1]+h[2]+h[3]+h[10]+h[5]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[490] = exp(h[12]+h[1]+h[2]+h[3]+h[10]+h[11]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[491] = exp(h[12]+h[1]+h[2]+h[3]+h[10]+h[17]+J[4]+J[5]+J[6]+J[9] - logZ)
	p[492] = exp(h[12]+h[1]+h[2]+h[3]+h[16]+h[5]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[493] = exp(h[12]+h[1]+h[2]+h[3]+h[16]+h[11]+J[3]+J[5]+J[6]+J[9] - logZ)
	p[494] = exp(h[12]+h[1]+h[2]+h[3]+h[16]+h[17]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[495] = exp(h[12]+h[1]+h[2]+h[9]+h[4]+h[5]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[496] = exp(h[12]+h[1]+h[2]+h[9]+h[4]+h[11]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[497] = exp(h[12]+h[1]+h[2]+h[9]+h[4]+h[17]+J[4]+J[5]+J[7]+J[10] - logZ)
	p[498] = exp(h[12]+h[1]+h[2]+h[9]+h[10]+h[5]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[499] = exp(h[12]+h[1]+h[2]+h[9]+h[10]+h[11]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[500] = exp(h[12]+h[1]+h[2]+h[9]+h[10]+h[17]+J[4]+J[5]+J[12] - logZ)
	p[501] = exp(h[12]+h[1]+h[2]+h[9]+h[16]+h[5]+J[3]+J[5]+J[8]+J[11] - logZ)
	p[502] = exp(h[12]+h[1]+h[2]+h[9]+h[16]+h[11]+J[3]+J[5]+J[13] - logZ)
	p[503] = exp(h[12]+h[1]+h[2]+h[9]+h[16]+h[17]+J[3]+J[4]+J[5]+J[14] - logZ)
	p[504] = exp(h[12]+h[1]+h[2]+h[15]+h[4]+h[5]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[505] = exp(h[12]+h[1]+h[2]+h[15]+h[4]+h[11]+J[2]+J[5]+J[7]+J[10] - logZ)
	p[506] = exp(h[12]+h[1]+h[2]+h[15]+h[4]+h[17]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[507] = exp(h[12]+h[1]+h[2]+h[15]+h[10]+h[5]+J[2]+J[5]+J[8]+J[11] - logZ)
	p[508] = exp(h[12]+h[1]+h[2]+h[15]+h[10]+h[11]+J[2]+J[5]+J[14] - logZ)
	p[509] = exp(h[12]+h[1]+h[2]+h[15]+h[10]+h[17]+J[2]+J[4]+J[5]+J[13] - logZ)
	p[510] = exp(h[12]+h[1]+h[2]+h[15]+h[16]+h[5]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[511] = exp(h[12]+h[1]+h[2]+h[15]+h[16]+h[11]+J[2]+J[3]+J[5]+J[12] - logZ)
	p[512] = exp(h[12]+h[1]+h[2]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[513] = exp(h[12]+h[1]+h[8]+h[3]+h[4]+h[5]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[514] = exp(h[12]+h[1]+h[8]+h[3]+h[4]+h[11]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[515] = exp(h[12]+h[1]+h[8]+h[3]+h[4]+h[17]+J[4]+J[6]+J[7]+J[12] - logZ)
	p[516] = exp(h[12]+h[1]+h[8]+h[3]+h[10]+h[5]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[517] = exp(h[12]+h[1]+h[8]+h[3]+h[10]+h[11]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[518] = exp(h[12]+h[1]+h[8]+h[3]+h[10]+h[17]+J[4]+J[6]+J[10] - logZ)
	p[519] = exp(h[12]+h[1]+h[8]+h[3]+h[16]+h[5]+J[3]+J[6]+J[8]+J[13] - logZ)
	p[520] = exp(h[12]+h[1]+h[8]+h[3]+h[16]+h[11]+J[3]+J[6]+J[11] - logZ)
	p[521] = exp(h[12]+h[1]+h[8]+h[3]+h[16]+h[17]+J[3]+J[4]+J[6]+J[14] - logZ)
	p[522] = exp(h[12]+h[1]+h[8]+h[9]+h[4]+h[5]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[523] = exp(h[12]+h[1]+h[8]+h[9]+h[4]+h[11]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[524] = exp(h[12]+h[1]+h[8]+h[9]+h[4]+h[17]+J[4]+J[7]+J[9] - logZ)
	p[525] = exp(h[12]+h[1]+h[8]+h[9]+h[10]+h[5]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[526] = exp(h[12]+h[1]+h[8]+h[9]+h[10]+h[11]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[527] = exp(h[12]+h[1]+h[8]+h[9]+h[10]+h[17]+J[4]+J[9]+J[10]+J[12] - logZ)
	p[528] = exp(h[12]+h[1]+h[8]+h[9]+h[16]+h[5]+J[3]+J[8]+J[9] - logZ)
	p[529] = exp(h[12]+h[1]+h[8]+h[9]+h[16]+h[11]+J[3]+J[9]+J[11]+J[13] - logZ)
	p[530] = exp(h[12]+h[1]+h[8]+h[9]+h[16]+h[17]+J[3]+J[4]+J[9]+J[14] - logZ)
	p[531] = exp(h[12]+h[1]+h[8]+h[15]+h[4]+h[5]+J[2]+J[7]+J[8]+J[14] - logZ)
	p[532] = exp(h[12]+h[1]+h[8]+h[15]+h[4]+h[11]+J[2]+J[7]+J[11] - logZ)
	p[533] = exp(h[12]+h[1]+h[8]+h[15]+h[4]+h[17]+J[2]+J[4]+J[7]+J[13] - logZ)
	p[534] = exp(h[12]+h[1]+h[8]+h[15]+h[10]+h[5]+J[2]+J[8]+J[10] - logZ)
	p[535] = exp(h[12]+h[1]+h[8]+h[15]+h[10]+h[11]+J[2]+J[10]+J[11]+J[14] - logZ)
	p[536] = exp(h[12]+h[1]+h[8]+h[15]+h[10]+h[17]+J[2]+J[4]+J[10]+J[13] - logZ)
	p[537] = exp(h[12]+h[1]+h[8]+h[15]+h[16]+h[5]+J[2]+J[3]+J[8]+J[12] - logZ)
	p[538] = exp(h[12]+h[1]+h[8]+h[15]+h[16]+h[11]+J[2]+J[3]+J[11]+J[12] - logZ)
	p[539] = exp(h[12]+h[1]+h[8]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14] - logZ)
	p[540] = exp(h[12]+h[1]+h[14]+h[3]+h[4]+h[5]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[541] = exp(h[12]+h[1]+h[14]+h[3]+h[4]+h[11]+J[1]+J[6]+J[7]+J[12] - logZ)
	p[542] = exp(h[12]+h[1]+h[14]+h[3]+h[4]+h[17]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[543] = exp(h[12]+h[1]+h[14]+h[3]+h[10]+h[5]+J[1]+J[6]+J[8]+J[13] - logZ)
	p[544] = exp(h[12]+h[1]+h[14]+h[3]+h[10]+h[11]+J[1]+J[6]+J[14] - logZ)
	p[545] = exp(h[12]+h[1]+h[14]+h[3]+h[10]+h[17]+J[1]+J[4]+J[6]+J[11] - logZ)
	p[546] = exp(h[12]+h[1]+h[14]+h[3]+h[16]+h[5]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[547] = exp(h[12]+h[1]+h[14]+h[3]+h[16]+h[11]+J[1]+J[3]+J[6]+J[10] - logZ)
	p[548] = exp(h[12]+h[1]+h[14]+h[3]+h[16]+h[17]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[549] = exp(h[12]+h[1]+h[14]+h[9]+h[4]+h[5]+J[1]+J[7]+J[8]+J[14] - logZ)
	p[550] = exp(h[12]+h[1]+h[14]+h[9]+h[4]+h[11]+J[1]+J[7]+J[13] - logZ)
	p[551] = exp(h[12]+h[1]+h[14]+h[9]+h[4]+h[17]+J[1]+J[4]+J[7]+J[11] - logZ)
	p[552] = exp(h[12]+h[1]+h[14]+h[9]+h[10]+h[5]+J[1]+J[8]+J[12] - logZ)
	p[553] = exp(h[12]+h[1]+h[14]+h[9]+h[10]+h[11]+J[1]+J[12]+J[13]+J[14] - logZ)
	p[554] = exp(h[12]+h[1]+h[14]+h[9]+h[10]+h[17]+J[1]+J[4]+J[11]+J[12] - logZ)
	p[555] = exp(h[12]+h[1]+h[14]+h[9]+h[16]+h[5]+J[1]+J[3]+J[8]+J[10] - logZ)
	p[556] = exp(h[12]+h[1]+h[14]+h[9]+h[16]+h[11]+J[1]+J[3]+J[10]+J[13] - logZ)
	p[557] = exp(h[12]+h[1]+h[14]+h[9]+h[16]+h[17]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14] - logZ)
	p[558] = exp(h[12]+h[1]+h[14]+h[15]+h[4]+h[5]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[559] = exp(h[12]+h[1]+h[14]+h[15]+h[4]+h[11]+J[1]+J[2]+J[7]+J[9] - logZ)
	p[560] = exp(h[12]+h[1]+h[14]+h[15]+h[4]+h[17]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[561] = exp(h[12]+h[1]+h[14]+h[15]+h[10]+h[5]+J[1]+J[2]+J[8]+J[9] - logZ)
	p[562] = exp(h[12]+h[1]+h[14]+h[15]+h[10]+h[11]+J[1]+J[2]+J[9]+J[14] - logZ)
	p[563] = exp(h[12]+h[1]+h[14]+h[15]+h[10]+h[17]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13] - logZ)
	p[564] = exp(h[12]+h[1]+h[14]+h[15]+h[16]+h[5]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[565] = exp(h[12]+h[1]+h[14]+h[15]+h[16]+h[11]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12] - logZ)
	p[566] = exp(h[12]+h[1]+h[14]+h[15]+h[16]+h[17]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[567] = exp(h[12]+h[7]+h[2]+h[3]+h[4]+h[5]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[568] = exp(h[12]+h[7]+h[2]+h[3]+h[4]+h[11]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[569] = exp(h[12]+h[7]+h[2]+h[3]+h[4]+h[17]+J[4]+J[9]+J[10]+J[12] - logZ)
	p[570] = exp(h[12]+h[7]+h[2]+h[3]+h[10]+h[5]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[571] = exp(h[12]+h[7]+h[2]+h[3]+h[10]+h[11]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[572] = exp(h[12]+h[7]+h[2]+h[3]+h[10]+h[17]+J[4]+J[7]+J[9] - logZ)
	p[573] = exp(h[12]+h[7]+h[2]+h[3]+h[16]+h[5]+J[3]+J[9]+J[11]+J[13] - logZ)
	p[574] = exp(h[12]+h[7]+h[2]+h[3]+h[16]+h[11]+J[3]+J[8]+J[9] - logZ)
	p[575] = exp(h[12]+h[7]+h[2]+h[3]+h[16]+h[17]+J[3]+J[4]+J[9]+J[14] - logZ)
	p[576] = exp(h[12]+h[7]+h[2]+h[9]+h[4]+h[5]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[577] = exp(h[12]+h[7]+h[2]+h[9]+h[4]+h[11]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[578] = exp(h[12]+h[7]+h[2]+h[9]+h[4]+h[17]+J[4]+J[6]+J[10] - logZ)
	p[579] = exp(h[12]+h[7]+h[2]+h[9]+h[10]+h[5]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[580] = exp(h[12]+h[7]+h[2]+h[9]+h[10]+h[11]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[581] = exp(h[12]+h[7]+h[2]+h[9]+h[10]+h[17]+J[4]+J[6]+J[7]+J[12] - logZ)
	p[582] = exp(h[12]+h[7]+h[2]+h[9]+h[16]+h[5]+J[3]+J[6]+J[11] - logZ)
	p[583] = exp(h[12]+h[7]+h[2]+h[9]+h[16]+h[11]+J[3]+J[6]+J[8]+J[13] - logZ)
	p[584] = exp(h[12]+h[7]+h[2]+h[9]+h[16]+h[17]+J[3]+J[4]+J[6]+J[14] - logZ)
	p[585] = exp(h[12]+h[7]+h[2]+h[15]+h[4]+h[5]+J[2]+J[10]+J[11]+J[14] - logZ)
	p[586] = exp(h[12]+h[7]+h[2]+h[15]+h[4]+h[11]+J[2]+J[8]+J[10] - logZ)
	p[587] = exp(h[12]+h[7]+h[2]+h[15]+h[4]+h[17]+J[2]+J[4]+J[10]+J[13] - logZ)
	p[588] = exp(h[12]+h[7]+h[2]+h[15]+h[10]+h[5]+J[2]+J[7]+J[11] - logZ)
	p[589] = exp(h[12]+h[7]+h[2]+h[15]+h[10]+h[11]+J[2]+J[7]+J[8]+J[14] - logZ)
	p[590] = exp(h[12]+h[7]+h[2]+h[15]+h[10]+h[17]+J[2]+J[4]+J[7]+J[13] - logZ)
	p[591] = exp(h[12]+h[7]+h[2]+h[15]+h[16]+h[5]+J[2]+J[3]+J[11]+J[12] - logZ)
	p[592] = exp(h[12]+h[7]+h[2]+h[15]+h[16]+h[11]+J[2]+J[3]+J[8]+J[12] - logZ)
	p[593] = exp(h[12]+h[7]+h[2]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[12]+J[13]+J[14] - logZ)
	p[594] = exp(h[12]+h[7]+h[8]+h[3]+h[4]+h[5]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[595] = exp(h[12]+h[7]+h[8]+h[3]+h[4]+h[11]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[596] = exp(h[12]+h[7]+h[8]+h[3]+h[4]+h[17]+J[4]+J[5]+J[12] - logZ)
	p[597] = exp(h[12]+h[7]+h[8]+h[3]+h[10]+h[5]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[598] = exp(h[12]+h[7]+h[8]+h[3]+h[10]+h[11]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[599] = exp(h[12]+h[7]+h[8]+h[3]+h[10]+h[17]+J[4]+J[5]+J[7]+J[10] - logZ)
	p[600] = exp(h[12]+h[7]+h[8]+h[3]+h[16]+h[5]+J[3]+J[5]+J[13] - logZ)
	p[601] = exp(h[12]+h[7]+h[8]+h[3]+h[16]+h[11]+J[3]+J[5]+J[8]+J[11] - logZ)
	p[602] = exp(h[12]+h[7]+h[8]+h[3]+h[16]+h[17]+J[3]+J[4]+J[5]+J[14] - logZ)
	p[603] = exp(h[12]+h[7]+h[8]+h[9]+h[4]+h[5]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[604] = exp(h[12]+h[7]+h[8]+h[9]+h[4]+h[11]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[605] = exp(h[12]+h[7]+h[8]+h[9]+h[4]+h[17]+J[4]+J[5]+J[6]+J[9] - logZ)
	p[606] = exp(h[12]+h[7]+h[8]+h[9]+h[10]+h[5]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[607] = exp(h[12]+h[7]+h[8]+h[9]+h[10]+h[11]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[608] = exp(h[12]+h[7]+h[8]+h[9]+h[10]+h[17]+J[4]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[609] = exp(h[12]+h[7]+h[8]+h[9]+h[16]+h[5]+J[3]+J[5]+J[6]+J[9] - logZ)
	p[610] = exp(h[12]+h[7]+h[8]+h[9]+h[16]+h[11]+J[3]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[611] = exp(h[12]+h[7]+h[8]+h[9]+h[16]+h[17]+J[3]+J[4]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[612] = exp(h[12]+h[7]+h[8]+h[15]+h[4]+h[5]+J[2]+J[5]+J[14] - logZ)
	p[613] = exp(h[12]+h[7]+h[8]+h[15]+h[4]+h[11]+J[2]+J[5]+J[8]+J[11] - logZ)
	p[614] = exp(h[12]+h[7]+h[8]+h[15]+h[4]+h[17]+J[2]+J[4]+J[5]+J[13] - logZ)
	p[615] = exp(h[12]+h[7]+h[8]+h[15]+h[10]+h[5]+J[2]+J[5]+J[7]+J[10] - logZ)
	p[616] = exp(h[12]+h[7]+h[8]+h[15]+h[10]+h[11]+J[2]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[617] = exp(h[12]+h[7]+h[8]+h[15]+h[10]+h[17]+J[2]+J[4]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[618] = exp(h[12]+h[7]+h[8]+h[15]+h[16]+h[5]+J[2]+J[3]+J[5]+J[12] - logZ)
	p[619] = exp(h[12]+h[7]+h[8]+h[15]+h[16]+h[11]+J[2]+J[3]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[620] = exp(h[12]+h[7]+h[8]+h[15]+h[16]+h[17]+J[2]+J[3]+J[4]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[621] = exp(h[12]+h[7]+h[14]+h[3]+h[4]+h[5]+J[1]+J[12]+J[13]+J[14] - logZ)
	p[622] = exp(h[12]+h[7]+h[14]+h[3]+h[4]+h[11]+J[1]+J[8]+J[12] - logZ)
	p[623] = exp(h[12]+h[7]+h[14]+h[3]+h[4]+h[17]+J[1]+J[4]+J[11]+J[12] - logZ)
	p[624] = exp(h[12]+h[7]+h[14]+h[3]+h[10]+h[5]+J[1]+J[7]+J[13] - logZ)
	p[625] = exp(h[12]+h[7]+h[14]+h[3]+h[10]+h[11]+J[1]+J[7]+J[8]+J[14] - logZ)
	p[626] = exp(h[12]+h[7]+h[14]+h[3]+h[10]+h[17]+J[1]+J[4]+J[7]+J[11] - logZ)
	p[627] = exp(h[12]+h[7]+h[14]+h[3]+h[16]+h[5]+J[1]+J[3]+J[10]+J[13] - logZ)
	p[628] = exp(h[12]+h[7]+h[14]+h[3]+h[16]+h[11]+J[1]+J[3]+J[8]+J[10] - logZ)
	p[629] = exp(h[12]+h[7]+h[14]+h[3]+h[16]+h[17]+J[1]+J[3]+J[4]+J[10]+J[11]+J[14] - logZ)
	p[630] = exp(h[12]+h[7]+h[14]+h[9]+h[4]+h[5]+J[1]+J[6]+J[14] - logZ)
	p[631] = exp(h[12]+h[7]+h[14]+h[9]+h[4]+h[11]+J[1]+J[6]+J[8]+J[13] - logZ)
	p[632] = exp(h[12]+h[7]+h[14]+h[9]+h[4]+h[17]+J[1]+J[4]+J[6]+J[11] - logZ)
	p[633] = exp(h[12]+h[7]+h[14]+h[9]+h[10]+h[5]+J[1]+J[6]+J[7]+J[12] - logZ)
	p[634] = exp(h[12]+h[7]+h[14]+h[9]+h[10]+h[11]+J[1]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[635] = exp(h[12]+h[7]+h[14]+h[9]+h[10]+h[17]+J[1]+J[4]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[636] = exp(h[12]+h[7]+h[14]+h[9]+h[16]+h[5]+J[1]+J[3]+J[6]+J[10] - logZ)
	p[637] = exp(h[12]+h[7]+h[14]+h[9]+h[16]+h[11]+J[1]+J[3]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[638] = exp(h[12]+h[7]+h[14]+h[9]+h[16]+h[17]+J[1]+J[3]+J[4]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[639] = exp(h[12]+h[7]+h[14]+h[15]+h[4]+h[5]+J[1]+J[2]+J[9]+J[14] - logZ)
	p[640] = exp(h[12]+h[7]+h[14]+h[15]+h[4]+h[11]+J[1]+J[2]+J[8]+J[9] - logZ)
	p[641] = exp(h[12]+h[7]+h[14]+h[15]+h[4]+h[17]+J[1]+J[2]+J[4]+J[9]+J[11]+J[13] - logZ)
	p[642] = exp(h[12]+h[7]+h[14]+h[15]+h[10]+h[5]+J[1]+J[2]+J[7]+J[9] - logZ)
	p[643] = exp(h[12]+h[7]+h[14]+h[15]+h[10]+h[11]+J[1]+J[2]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[644] = exp(h[12]+h[7]+h[14]+h[15]+h[10]+h[17]+J[1]+J[2]+J[4]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[645] = exp(h[12]+h[7]+h[14]+h[15]+h[16]+h[5]+J[1]+J[2]+J[3]+J[9]+J[10]+J[12] - logZ)
	p[646] = exp(h[12]+h[7]+h[14]+h[15]+h[16]+h[11]+J[1]+J[2]+J[3]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[647] = exp(h[12]+h[7]+h[14]+h[15]+h[16]+h[17]+J[1]+J[2]+J[3]+J[4]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[648] = exp(h[12]+h[13]+h[2]+h[3]+h[4]+h[5]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[649] = exp(h[12]+h[13]+h[2]+h[3]+h[4]+h[11]+J[0]+J[9]+J[10]+J[12] - logZ)
	p[650] = exp(h[12]+h[13]+h[2]+h[3]+h[4]+h[17]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[651] = exp(h[12]+h[13]+h[2]+h[3]+h[10]+h[5]+J[0]+J[9]+J[11]+J[13] - logZ)
	p[652] = exp(h[12]+h[13]+h[2]+h[3]+h[10]+h[11]+J[0]+J[9]+J[14] - logZ)
	p[653] = exp(h[12]+h[13]+h[2]+h[3]+h[10]+h[17]+J[0]+J[4]+J[8]+J[9] - logZ)
	p[654] = exp(h[12]+h[13]+h[2]+h[3]+h[16]+h[5]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[655] = exp(h[12]+h[13]+h[2]+h[3]+h[16]+h[11]+J[0]+J[3]+J[7]+J[9] - logZ)
	p[656] = exp(h[12]+h[13]+h[2]+h[3]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[657] = exp(h[12]+h[13]+h[2]+h[9]+h[4]+h[5]+J[0]+J[10]+J[11]+J[14] - logZ)
	p[658] = exp(h[12]+h[13]+h[2]+h[9]+h[4]+h[11]+J[0]+J[10]+J[13] - logZ)
	p[659] = exp(h[12]+h[13]+h[2]+h[9]+h[4]+h[17]+J[0]+J[4]+J[8]+J[10] - logZ)
	p[660] = exp(h[12]+h[13]+h[2]+h[9]+h[10]+h[5]+J[0]+J[11]+J[12] - logZ)
	p[661] = exp(h[12]+h[13]+h[2]+h[9]+h[10]+h[11]+J[0]+J[12]+J[13]+J[14] - logZ)
	p[662] = exp(h[12]+h[13]+h[2]+h[9]+h[10]+h[17]+J[0]+J[4]+J[8]+J[12] - logZ)
	p[663] = exp(h[12]+h[13]+h[2]+h[9]+h[16]+h[5]+J[0]+J[3]+J[7]+J[11] - logZ)
	p[664] = exp(h[12]+h[13]+h[2]+h[9]+h[16]+h[11]+J[0]+J[3]+J[7]+J[13] - logZ)
	p[665] = exp(h[12]+h[13]+h[2]+h[9]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14] - logZ)
	p[666] = exp(h[12]+h[13]+h[2]+h[15]+h[4]+h[5]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[667] = exp(h[12]+h[13]+h[2]+h[15]+h[4]+h[11]+J[0]+J[2]+J[6]+J[10] - logZ)
	p[668] = exp(h[12]+h[13]+h[2]+h[15]+h[4]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[669] = exp(h[12]+h[13]+h[2]+h[15]+h[10]+h[5]+J[0]+J[2]+J[6]+J[11] - logZ)
	p[670] = exp(h[12]+h[13]+h[2]+h[15]+h[10]+h[11]+J[0]+J[2]+J[6]+J[14] - logZ)
	p[671] = exp(h[12]+h[13]+h[2]+h[15]+h[10]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13] - logZ)
	p[672] = exp(h[12]+h[13]+h[2]+h[15]+h[16]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[673] = exp(h[12]+h[13]+h[2]+h[15]+h[16]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12] - logZ)
	p[674] = exp(h[12]+h[13]+h[2]+h[15]+h[16]+h[17]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[675] = exp(h[12]+h[13]+h[8]+h[3]+h[4]+h[5]+J[0]+J[12]+J[13]+J[14] - logZ)
	p[676] = exp(h[12]+h[13]+h[8]+h[3]+h[4]+h[11]+J[0]+J[11]+J[12] - logZ)
	p[677] = exp(h[12]+h[13]+h[8]+h[3]+h[4]+h[17]+J[0]+J[4]+J[8]+J[12] - logZ)
	p[678] = exp(h[12]+h[13]+h[8]+h[3]+h[10]+h[5]+J[0]+J[10]+J[13] - logZ)
	p[679] = exp(h[12]+h[13]+h[8]+h[3]+h[10]+h[11]+J[0]+J[10]+J[11]+J[14] - logZ)
	p[680] = exp(h[12]+h[13]+h[8]+h[3]+h[10]+h[17]+J[0]+J[4]+J[8]+J[10] - logZ)
	p[681] = exp(h[12]+h[13]+h[8]+h[3]+h[16]+h[5]+J[0]+J[3]+J[7]+J[13] - logZ)
	p[682] = exp(h[12]+h[13]+h[8]+h[3]+h[16]+h[11]+J[0]+J[3]+J[7]+J[11] - logZ)
	p[683] = exp(h[12]+h[13]+h[8]+h[3]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[14] - logZ)
	p[684] = exp(h[12]+h[13]+h[8]+h[9]+h[4]+h[5]+J[0]+J[9]+J[14] - logZ)
	p[685] = exp(h[12]+h[13]+h[8]+h[9]+h[4]+h[11]+J[0]+J[9]+J[11]+J[13] - logZ)
	p[686] = exp(h[12]+h[13]+h[8]+h[9]+h[4]+h[17]+J[0]+J[4]+J[8]+J[9] - logZ)
	p[687] = exp(h[12]+h[13]+h[8]+h[9]+h[10]+h[5]+J[0]+J[9]+J[10]+J[12] - logZ)
	p[688] = exp(h[12]+h[13]+h[8]+h[9]+h[10]+h[11]+J[0]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	p[689] = exp(h[12]+h[13]+h[8]+h[9]+h[10]+h[17]+J[0]+J[4]+J[8]+J[9]+J[10]+J[12] - logZ)
	p[690] = exp(h[12]+h[13]+h[8]+h[9]+h[16]+h[5]+J[0]+J[3]+J[7]+J[9] - logZ)
	p[691] = exp(h[12]+h[13]+h[8]+h[9]+h[16]+h[11]+J[0]+J[3]+J[7]+J[9]+J[11]+J[13] - logZ)
	p[692] = exp(h[12]+h[13]+h[8]+h[9]+h[16]+h[17]+J[0]+J[3]+J[4]+J[7]+J[8]+J[9]+J[14] - logZ)
	p[693] = exp(h[12]+h[13]+h[8]+h[15]+h[4]+h[5]+J[0]+J[2]+J[6]+J[14] - logZ)
	p[694] = exp(h[12]+h[13]+h[8]+h[15]+h[4]+h[11]+J[0]+J[2]+J[6]+J[11] - logZ)
	p[695] = exp(h[12]+h[13]+h[8]+h[15]+h[4]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[13] - logZ)
	p[696] = exp(h[12]+h[13]+h[8]+h[15]+h[10]+h[5]+J[0]+J[2]+J[6]+J[10] - logZ)
	p[697] = exp(h[12]+h[13]+h[8]+h[15]+h[10]+h[11]+J[0]+J[2]+J[6]+J[10]+J[11]+J[14] - logZ)
	p[698] = exp(h[12]+h[13]+h[8]+h[15]+h[10]+h[17]+J[0]+J[2]+J[4]+J[6]+J[8]+J[10]+J[13] - logZ)
	p[699] = exp(h[12]+h[13]+h[8]+h[15]+h[16]+h[5]+J[0]+J[2]+J[3]+J[6]+J[7]+J[12] - logZ)
	p[700] = exp(h[12]+h[13]+h[8]+h[15]+h[16]+h[11]+J[0]+J[2]+J[3]+J[6]+J[7]+J[11]+J[12] - logZ)
	p[701] = exp(h[12]+h[13]+h[8]+h[15]+h[16]+h[17]+J[0]+J[2]+J[3]+J[4]+J[6]+J[7]+J[8]+J[12]+J[13]+J[14] - logZ)
	p[702] = exp(h[12]+h[13]+h[14]+h[3]+h[4]+h[5]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[703] = exp(h[12]+h[13]+h[14]+h[3]+h[4]+h[11]+J[0]+J[1]+J[5]+J[12] - logZ)
	p[704] = exp(h[12]+h[13]+h[14]+h[3]+h[4]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[705] = exp(h[12]+h[13]+h[14]+h[3]+h[10]+h[5]+J[0]+J[1]+J[5]+J[13] - logZ)
	p[706] = exp(h[12]+h[13]+h[14]+h[3]+h[10]+h[11]+J[0]+J[1]+J[5]+J[14] - logZ)
	p[707] = exp(h[12]+h[13]+h[14]+h[3]+h[10]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11] - logZ)
	p[708] = exp(h[12]+h[13]+h[14]+h[3]+h[16]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[709] = exp(h[12]+h[13]+h[14]+h[3]+h[16]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10] - logZ)
	p[710] = exp(h[12]+h[13]+h[14]+h[3]+h[16]+h[17]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[711] = exp(h[12]+h[13]+h[14]+h[9]+h[4]+h[5]+J[0]+J[1]+J[5]+J[14] - logZ)
	p[712] = exp(h[12]+h[13]+h[14]+h[9]+h[4]+h[11]+J[0]+J[1]+J[5]+J[13] - logZ)
	p[713] = exp(h[12]+h[13]+h[14]+h[9]+h[4]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11] - logZ)
	p[714] = exp(h[12]+h[13]+h[14]+h[9]+h[10]+h[5]+J[0]+J[1]+J[5]+J[12] - logZ)
	p[715] = exp(h[12]+h[13]+h[14]+h[9]+h[10]+h[11]+J[0]+J[1]+J[5]+J[12]+J[13]+J[14] - logZ)
	p[716] = exp(h[12]+h[13]+h[14]+h[9]+h[10]+h[17]+J[0]+J[1]+J[4]+J[5]+J[8]+J[11]+J[12] - logZ)
	p[717] = exp(h[12]+h[13]+h[14]+h[9]+h[16]+h[5]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10] - logZ)
	p[718] = exp(h[12]+h[13]+h[14]+h[9]+h[16]+h[11]+J[0]+J[1]+J[3]+J[5]+J[7]+J[10]+J[13] - logZ)
	p[719] = exp(h[12]+h[13]+h[14]+h[9]+h[16]+h[17]+J[0]+J[1]+J[3]+J[4]+J[5]+J[7]+J[8]+J[10]+J[11]+J[14] - logZ)
	p[720] = exp(h[12]+h[13]+h[14]+h[15]+h[4]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[721] = exp(h[12]+h[13]+h[14]+h[15]+h[4]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9] - logZ)
	p[722] = exp(h[12]+h[13]+h[14]+h[15]+h[4]+h[17]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[723] = exp(h[12]+h[13]+h[14]+h[15]+h[10]+h[5]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9] - logZ)
	p[724] = exp(h[12]+h[13]+h[14]+h[15]+h[10]+h[11]+J[0]+J[1]+J[2]+J[5]+J[6]+J[9]+J[14] - logZ)
	p[725] = exp(h[12]+h[13]+h[14]+h[15]+h[10]+h[17]+J[0]+J[1]+J[2]+J[4]+J[5]+J[6]+J[8]+J[9]+J[11]+J[13] - logZ)
	p[726] = exp(h[12]+h[13]+h[14]+h[15]+h[16]+h[5]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[727] = exp(h[12]+h[13]+h[14]+h[15]+h[16]+h[11]+J[0]+J[1]+J[2]+J[3]+J[5]+J[6]+J[7]+J[9]+J[10]+J[12] - logZ)
	p[728] = exp(h[12]+h[13]+h[14]+h[15]+h[16]+h[17]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14] - logZ)
	return p
